@@ -49,12 +49,13 @@
 #ifdef HAVE_ASSERT_H
 #include <assert.h>
 #endif
-#include <uim/uim-util.h>
 #include "uim-fep.h"
 #include "str.h"
 #include "callbacks.h"
+#include "helper.h"
+#include <uim/uim-im-switcher.h>
+#include <uim/uim-helper.h>
 
-static uim_context s_context;
 /* ơ饤κ */
 static int s_max_width;
 static char *s_commit_str;
@@ -64,10 +65,12 @@ static int s_candidate_col;
 static char *s_index_str;
 static struct preedit_tag *s_preedit;
 static int s_mode;
+static char *s_label_str;
+static const char *s_im_str;
 static char *s_nokori_str;
+static int s_start_callbacks = FALSE;
 
-static void start_callbacks(void);
-static void end_callbacks(void);
+static void configuration_changed_cb(void *ptr);
 static void activate_cb(void *ptr, int nr, int display_limit);
 static void select_cb(void *ptr, int index);
 static void shift_page_cb(void *ptr, int direction);
@@ -76,12 +79,14 @@ static void clear_cb(void *ptr);
 static void pushback_cb(void *ptr, int attr, const char *str);
 static void update_cb(void *ptr);
 static void mode_update_cb(void *ptr, int mode);
+static void prop_list_update_cb(void *ptr, const char *str);
+static void prop_label_update_cb(void *ptr, const char *str);
 static struct preedit_tag *dup_preedit(struct preedit_tag *p);
 static void make_page_strs(void);
 static int numwidth(int n);
 static int index2page(int index);
 static void reset_candidate(void);
-static void get_candidate(void);
+static void set_candidate(void);
 
 struct candidate_tag {
   /* ο */
@@ -119,9 +124,8 @@ static struct candidate_tag s_candidate = {
 /*
  * 
  */
-void init_callbacks(uim_context context)
+void init_callbacks(void)
 {
-  s_context = context;
   s_max_width = g_win->ws_col;
   if (g_opt.statusline_width != UNDEFINED && g_opt.statusline_width <= s_max_width) {
     s_max_width = g_opt.statusline_width;
@@ -131,22 +135,25 @@ void init_callbacks(uim_context context)
   s_statusline_str = strdup("");
   s_candidate_col = UNDEFINED;
   s_index_str = strdup("");
-  s_mode = uim_get_current_mode(s_context);
+  s_mode = uim_get_current_mode(g_context);
+  s_label_str = strdup("");
   s_preedit = create_preedit();
-  uim_set_preedit_cb(s_context, clear_cb, pushback_cb, update_cb);
-  uim_set_mode_cb(s_context, mode_update_cb);
+  uim_set_preedit_cb(g_context, clear_cb, pushback_cb, update_cb);
+  uim_set_mode_cb(g_context, mode_update_cb);
+  uim_set_prop_list_update_cb(g_context, prop_list_update_cb);
+  uim_set_prop_label_update_cb(g_context, prop_label_update_cb);
+  uim_set_configuration_changed_cb(g_context, configuration_changed_cb);
+  configuration_changed_cb(NULL);
   if (g_opt.status_type != NONE) {
-    uim_set_candidate_selector_cb(s_context, activate_cb, select_cb, shift_page_cb, deactivate_cb);
+    uim_set_candidate_selector_cb(g_context, activate_cb, select_cb, shift_page_cb, deactivate_cb);
   }
 
   if (g_opt.ddskk) {
-    void *cd;
-    const char *nokori_str = "Ĥ";
     const char *enc;
 
     if (uim_iconv->is_convertible(enc = get_enc(), "EUC-JP")) {
-      cd = uim_iconv->create(enc, "EUC-JP");
-      s_nokori_str = uim_iconv->convert(cd, nokori_str);
+      void *cd = uim_iconv->create(enc, "EUC-JP");
+      s_nokori_str = uim_iconv->convert(cd, "Ĥ");
       if (cd) {
         uim_iconv->release(cd);
       }
@@ -168,47 +175,50 @@ int press_key(int key, int key_state)
     debug2(("press key = %d key_state = %d\n", key, key_state));
   }
 #endif
-  start_callbacks();
-  raw = uim_press_key(s_context, key, key_state);
-  uim_release_key(s_context, key, key_state);
-  end_callbacks();
+  raw = uim_press_key(g_context, key, key_state);
+  uim_release_key(g_context, key, key_state);
   return raw;
 }
 
-static void start_callbacks(void)
+
+/*
+ * ̾ʶ路uim¦׵ᤵ줿Ƥ֡
+ */
+void start_callbacks(void)
 {
+  if (s_start_callbacks) {
+    return;
+  }
+  s_start_callbacks = TRUE;
+
   debug2(("\n\nstart_callbacks()\n"));
-  if (s_commit_str != NULL) {
-    free(s_commit_str);
-  }
-  s_commit_str = strdup("");
-  if (s_candidate_str != NULL) {
-    free(s_candidate_str);
-    s_candidate_str = NULL;
-  }
-  if (s_index_str != NULL) {
-    free(s_index_str);
-    s_index_str = NULL;
-  }
-  if (s_statusline_str != NULL) {
-    free(s_statusline_str);
-    s_statusline_str = NULL;
-  }
-  s_candidate_col = UNDEFINED;
-  s_mode = uim_get_current_mode(s_context);
 }
 
-static void end_callbacks(void)
+/*
+ * ХåؿƤФƤʤСFALSE֤
+ */
+int end_callbacks(void)
 {
   debug2(("end_callbacks()\n\n"));
+  if (!s_start_callbacks) {
+    return FALSE;
+  }
+
+  s_start_callbacks = FALSE;
+
   /* cursorꤵƤʤȤϥץꥨǥåȤˤ */
   if (s_preedit->cursor == UNDEFINED) {
     s_preedit->cursor = s_preedit->width;
   }
+
+  free(s_statusline_str);
+  free(s_candidate_str);
+  free(s_index_str);
+
   if (s_candidate.nr != UNDEFINED) {
     s_statusline_str = strdup(s_candidate.page_strs[s_candidate.page]);
     if (s_candidate.index != UNDEFINED) {
-      get_candidate();
+      set_candidate();
     } else {
       s_candidate_str = strdup("");
       s_candidate_col = UNDEFINED;
@@ -220,6 +230,8 @@ static void end_callbacks(void)
     s_candidate_col = UNDEFINED;
     s_index_str = strdup("");
   }
+
+  return TRUE;
 }
 
 /*
@@ -228,7 +240,13 @@ static void end_callbacks(void)
  */
 char *get_commit_str(void)
 {
-  return strdup(s_commit_str);
+  char *return_value = strdup(s_commit_str);
+
+  assert(!s_start_callbacks);
+
+  free(s_commit_str);
+  s_commit_str = strdup("");
+  return return_value;
 }
 
 /*
@@ -237,6 +255,7 @@ char *get_commit_str(void)
  */
 char *get_statusline_str(void)
 {
+  assert(!s_start_callbacks);
   return strdup(s_statusline_str);
 }
 
@@ -246,6 +265,7 @@ char *get_statusline_str(void)
  */
 char *get_candidate_str(void)
 {
+  assert(!s_start_callbacks);
   return strdup(s_candidate_str);
 }
 
@@ -255,6 +275,7 @@ char *get_candidate_str(void)
  */
 int get_candidate_col(void)
 {
+  assert(!s_start_callbacks);
   return s_candidate_col;
 }
 
@@ -264,6 +285,7 @@ int get_candidate_col(void)
  */
 char *get_index_str(void)
 {
+  assert(!s_start_callbacks);
   return strdup(s_index_str);
 }
 
@@ -273,6 +295,8 @@ char *get_index_str(void)
  */
 int get_index_col(void)
 {
+  assert(!s_start_callbacks);
+
   if (s_candidate.index != UNDEFINED) {
     return s_candidate.index_col[s_candidate.page];
   }
@@ -285,6 +309,7 @@ int get_index_col(void)
  */
 struct preedit_tag *get_preedit(void)
 {
+  assert(!s_start_callbacks);
   return dup_preedit(s_preedit);
 }
 
@@ -293,14 +318,31 @@ struct preedit_tag *get_preedit(void)
  */
 int get_mode(void)
 {
+  assert(!s_start_callbacks);
   return s_mode;
 }
 
+/*
+ * ߤΥ⡼ʸ֤
+ * ֤ͤNULLˤʤ뤳ȤϤʤfreeɬפ
+ */
 char *get_mode_str(void)
 {
-  char *mode_str = strdup(uim_get_mode_name(s_context, s_mode));
-  strhead(mode_str, s_max_width);
-  return mode_str;
+  char *str;
+
+  assert(!s_start_callbacks);
+
+  str = malloc(strlen(s_im_str) + strlen(s_label_str) + strlen("[]") + 1);
+  sprintf(str, "%s[%s]", s_im_str, s_label_str);
+  strhead(str, s_max_width);
+
+  return str;
+}
+
+static void configuration_changed_cb(void *ptr)
+{
+  s_im_str = uim_get_current_im_name(g_context);
+  s_im_str = s_im_str != NULL ? s_im_str : "";
 }
 
 /*
@@ -313,6 +355,7 @@ char *get_mode_str(void)
 static void activate_cb(void *ptr, int nr, int display_limit)
 {
   debug2(("activate_cb(nr = %d display_limit = %d)\n", nr, display_limit));
+  start_callbacks();
   reset_candidate();
   s_candidate.nr = nr;
   s_candidate.limit = display_limit;
@@ -330,6 +373,10 @@ static void select_cb(void *ptr, int index)
   debug2(("select_cb(index = %d)\n", index));
   return_if_fail(s_candidate.nr != UNDEFINED);
   return_if_fail(0 <= index && index < s_candidate.nr);
+  if (s_candidate.index == index) {
+    return;
+  }
+  start_callbacks();
   s_candidate.index = index;
   s_candidate.page = index2page(index);
 }
@@ -345,6 +392,7 @@ static void shift_page_cb(void *ptr, int direction)
   int index;
   debug2(("shift_page_cb(direction = %d)\n", direction));
   return_if_fail(s_candidate.nr != UNDEFINED);
+  start_callbacks();
   if (direction == 0) {
     direction = -1;
   }
@@ -353,7 +401,7 @@ static void shift_page_cb(void *ptr, int direction)
   return_if_fail(0 <= index && index < s_candidate.nr);
   s_candidate.page = page;
   s_candidate.index = index;
-  uim_set_candidate_index(s_context, s_candidate.index);
+  uim_set_candidate_index(g_context, s_candidate.index);
 }
 
 /*
@@ -363,6 +411,7 @@ static void shift_page_cb(void *ptr, int direction)
 static void deactivate_cb(void *ptr)
 {
   debug2(("deactivate_cb()\n"));
+  start_callbacks();
   reset_candidate();
 }
 
@@ -371,12 +420,17 @@ void commit_cb(void *ptr, const char *commit_str)
 {
   debug2(("commit_cb(commit_str = \"%s\")\n", commit_str));
   return_if_fail(commit_str != NULL);
+  if (strlen(commit_str) == 0) {
+    return;
+  }
+  start_callbacks();
   s_commit_str = realloc(s_commit_str, strlen(s_commit_str) + strlen(commit_str) + 1);
   strcat(s_commit_str, commit_str);
 }
 
 static void clear_cb(void *ptr)
 {
+  start_callbacks();
   if (s_preedit != NULL) {
     free_preedit(s_preedit);
   }
@@ -393,9 +447,18 @@ static void pushback_cb(void *ptr, int attr, const char *str)
 {
   int width;
   static int cursor = FALSE;
+
   debug2(("pushback_cb(attr = %d str = \"%s\")\n", attr, str));
+
   return_if_fail(str && s_preedit != NULL);
+
   width = strwidth(str);
+
+  if (width == 0 && (attr & UPreeditAttr_Cursor) == 0) {
+    return;
+  }
+
+  start_callbacks();
   /* UPreeditAttr_CursorΤȤ˶ʸȤϸ¤ʤ */
   if (attr & UPreeditAttr_Cursor) {
     /* skkμϿϥ뤬2սꤢ */
@@ -453,9 +516,137 @@ static void update_cb(void *ptr)
 static void mode_update_cb(void *ptr, int mode)
 {
   debug2(("mode_update_cb(mode = %d)\n", mode));
+
+  if (s_mode == mode) {
+    return;
+  }
+
+  start_callbacks();
   s_mode = mode;
 }
 
+static void prop_list_update_cb(void *ptr, const char *str)
+{
+  char *line;
+  int error = TRUE; /* str  "" ΤȤerrorˤ*/
+  char *labels = strdup("");
+  char *dup_str;
+
+  const char *enc;
+  char *message_buf;
+
+  debug(("prop_list_update_cb\n"));
+  debug2(("str = %s", str));
+
+  dup_str = line = strdup(str);
+
+  while (line[0] != '\0') {
+    int i;
+    char *tab;
+    char *eol;
+    char *label;
+    int label_width;
+    int max_label_width = 0;
+
+    error = TRUE;
+
+    /* branch = "branch\t" iconic_label "\t" buttontooltip_string "\n" */
+    if (!str_has_prefix(line, "branch\t")) {
+      break;
+    }
+    label = line + strlen("branch\t");
+
+    if ((tab = strchr(label, '\t')) == NULL) {
+      break;
+    }
+    *tab = '\0';
+
+    if ((eol = strchr(tab + 1, '\n')) == NULL) {
+      break;
+    }
+    line = eol + 1;
+
+    while (str_has_prefix(line, "leaf\t")) {
+      char *leaf_label = line + strlen("leaf\t");
+
+      tab = line + strlen("leaf");
+
+      /* leaf = "leaf\t" iconic_label "\t" menulabel_string "\t" menutooltip_string "\t" menucommand_name "\t" flag "\n" */
+      for (i = 0; i < 4; i++) {
+        if ((tab = strchr(tab + 1, '\t')) == NULL) {
+          goto loop_end;
+        }
+        *tab = '\0';
+      }
+      if ((eol = strchr(tab + 1, '\n')) == NULL) {
+        goto loop_end;
+      }
+      line = eol + 1;
+
+      error = FALSE;
+
+      label_width = strlen(leaf_label);
+      if (label_width > max_label_width) {
+        max_label_width = label_width;
+      }
+    }
+
+    label_width = strwidth(label);
+    labels = realloc(labels, strlen(labels) + strlen(label) + (max_label_width - label_width) + 1);
+    for (i = 0; i < (max_label_width - label_width); i++) {
+      strcat(labels, " ");
+    }
+    strcat(labels, label);
+  }
+
+loop_end:
+
+  free(dup_str);
+
+  if (error) {
+    free(labels);
+  } else {
+    if (strcmp(s_label_str, labels) != 0) {
+      start_callbacks();
+      free(s_label_str);
+      s_label_str = labels;
+    } else {
+      free(labels);
+    }
+  }
+
+  if (!g_focus_in) {
+    return;
+  }
+
+  enc = get_enc();
+  message_buf = malloc(strlen("prop_list_update\ncharset=") + strlen(enc) + strlen("\n") + strlen(str) + 1);
+  sprintf(message_buf, "prop_list_update\ncharset=%s\n%s", enc, str);
+  uim_helper_send_message(g_helper_fd, message_buf);
+  free(message_buf);
+  debug(("prop_list_update_cb send message\n"));
+}
+
+static void prop_label_update_cb(void *ptr, const char *str)
+{
+  const char *enc;
+  char *message_buf;
+
+  debug(("prop_label_update_cb\n"));
+  debug2(("str = %s", str));
+
+  if (!g_focus_in) {
+    return;
+  }
+
+  enc = get_enc();
+  message_buf = malloc(strlen("prop_label_update\ncharset=") + strlen(enc) + strlen("\n") + strlen(str) + 1);
+  sprintf(message_buf, "prop_label_update\ncharset=%s\n%s", enc, str);
+  uim_helper_send_message(g_helper_fd, message_buf);
+  free(message_buf);
+  debug(("prop_label_update_cb send message\n"));
+}
+
 /*
  * ץꥨǥåȤꡤݥ󥿤֤
  */
@@ -534,7 +725,7 @@ static void make_page_strs(void)
     int next = FALSE;
     /* "[10/20]"  */
     int index_width;
-    uim_candidate cand = uim_get_candidate(s_context, index, index_in_page);
+    uim_candidate cand = uim_get_candidate(g_context, index, index_in_page);
     const char *cand_str_label = uim_candidate_get_heading_label(cand);
     char *cand_str_cand = tab2space(uim_candidate_get_cand_str(cand));
     int cand_label_width = strwidth(cand_str_label);
@@ -691,6 +882,7 @@ static void reset_candidate(void)
   if (s_candidate.nr == UNDEFINED) {
     return;
   }
+
   if (s_candidate.page_strs != NULL) {
     int i;
     for (i = 0; i < s_candidate.nr_pages; i++) {
@@ -721,7 +913,7 @@ static void reset_candidate(void)
 /*
  * s_candidate_cols_candidate_strs_index_strꤹ
  */
-static void get_candidate(void)
+static void set_candidate(void)
 {
   uim_candidate cand;
   int cand_width;
@@ -758,7 +950,7 @@ static void get_candidate(void)
     s_candidate_str = strdup("");
     return;
   }
-  cand = uim_get_candidate(s_context, s_candidate.index, 0);
+  cand = uim_get_candidate(g_context, s_candidate.index, 0);
   if (uim_candidate_get_cand_str(cand) == NULL) {
     s_candidate_str = strdup("");
     s_candidate_col = UNDEFINED;
@@ -775,42 +967,36 @@ static void get_candidate(void)
 
 void callbacks_winch(void)
 {
+  start_callbacks();
+
   s_max_width = g_win->ws_col;
   if (g_opt.statusline_width != UNDEFINED && g_opt.statusline_width <= s_max_width) {
     s_max_width = g_opt.statusline_width;
   }
-  if (s_candidate.nr != UNDEFINED) {
-    if (s_candidate.page_strs != NULL) {
-      int i;
-      for (i = 0; i < s_candidate.nr_pages; i++) {
-        free(s_candidate.page_strs[i]);
-      }
-      free(s_candidate.page_strs);
-      s_candidate.page_strs = NULL;
-    }
-    if (s_candidate.page2index != NULL) {
-      free(s_candidate.page2index);
-      s_candidate.page2index = NULL;
-    }
-    if (s_candidate.index_col != NULL) {
-      free(s_candidate.index_col);
-      s_candidate.index_col = NULL;
-    }
-    make_page_strs();
-    if (s_statusline_str != NULL) {
-      free(s_statusline_str);
-    }
-    if (s_candidate_str != NULL) {
-      free(s_candidate_str);
-    }
-    s_statusline_str = strdup(s_candidate.page_strs[s_candidate.page]);
-    if (s_candidate.index != UNDEFINED) {
-      get_candidate();
-    } else {
-      s_candidate_str = strdup("");
-      s_candidate_col = UNDEFINED;
-    }
+
+  if (s_candidate.nr == UNDEFINED) {
+    return;
   }
+
+  /* ɽ */
+  if (s_candidate.page_strs != NULL) {
+    int i;
+    for (i = 0; i < s_candidate.nr_pages; i++) {
+      free(s_candidate.page_strs[i]);
+    }
+    free(s_candidate.page_strs);
+    s_candidate.page_strs = NULL;
+  }
+  if (s_candidate.page2index != NULL) {
+    free(s_candidate.page2index);
+    s_candidate.page2index = NULL;
+  }
+  if (s_candidate.index_col != NULL) {
+    free(s_candidate.index_col);
+    s_candidate.index_col = NULL;
+  }
+
+  make_page_strs();
 }
 
 void callbacks_set_mode(int mode)
