@@ -95,7 +95,6 @@ static const char *s_path_getmode;
 static int s_winch = FALSE;
 
 static void init_backtick(void);
-static void update_backtick(void);
 static void start_preedit(void);
 static void end_preedit(void);
 static void draw_statusline(int force, int restore, int visible, int draw_background);
@@ -137,7 +136,7 @@ static void init_backtick(void)
   }
 }
 
-static void update_backtick(void)
+void update_backtick(void)
 {
   char sendbuf[CANDSIZE];
   if (s_candbuf[0] == '\0') {
@@ -152,8 +151,9 @@ static void update_backtick(void)
 
 /*
  * ץꥨǥåȡơ饤褹
+ * ̤ɬפʤFALSE֤
  */
-void draw(void)
+int draw(void)
 {
   char *commit_str;
 
@@ -161,18 +161,26 @@ void draw(void)
 
   int i;
 
-  /* üѹ줿Ȥs_headѹ */
+  if (!end_callbacks()) {
+    return FALSE;
+  }
+
+  prev_preedit = s_preedit;
+  s_preedit = get_preedit();
+  commit_str = get_commit_str();
+
+  /* üѹ줿Ȥs_headѹpreeditʤäȤˤ */
   if (s_winch && g_start_preedit) {
     if (g_opt.no_report_cursor) {
       s_preedit->cursor = 0;
     } else {
       s_head = get_cursor_position();
     }
+    end_preedit();
+    free_preedit(prev_preedit);
+    prev_preedit = create_preedit();
   }
-
-  prev_preedit = s_preedit;
-  s_preedit = get_preedit();
-  commit_str = get_commit_str();
+  s_winch = FALSE;
 
   debug2(("draw()\n"));
   debug2(("commit_str = \"%s\"\n", commit_str));
@@ -231,9 +239,9 @@ void draw(void)
   free_preedit(prev_preedit);
   free(commit_str);
   put_cursor_normal();
-  s_winch = FALSE;
 
   debug2(("\ndraw end\n"));
+  return TRUE;
 }
 
 /*
@@ -313,6 +321,7 @@ void draw_commit_and_preedit(void)
   start_preedit();
   /* prev_preedit϶ */
   draw_preedit(s_preedit, &s_save_preedit);
+  put_cursor_normal();
 }
 
 /*
@@ -328,7 +337,7 @@ static void draw_statusline(int force, int restore, int visible, int draw_backgr
   static int statusline_str_width = 0;
   static char *candidate_str = NULL;
   static int candidate_col = UNDEFINED;
-  static int mode = UNDEFINED;
+  static char *mode_str = NULL;
   static char *index_str = NULL;
   static int index_col = UNDEFINED;
 
@@ -336,7 +345,7 @@ static void draw_statusline(int force, int restore, int visible, int draw_backgr
   int prev_statusline_str_width;
   char *prev_candidate_str;
   int prev_candidate_col;
-  int prev_mode;
+  char *prev_mode_str;
   char *prev_index_str;
   int prev_index_col;
 
@@ -350,19 +359,22 @@ static void draw_statusline(int force, int restore, int visible, int draw_backgr
   if (index_str == NULL) {
     index_str = strdup("");
   }
+  if (mode_str == NULL) {
+    mode_str = strdup("");
+  }
 
   prev_statusline_str = statusline_str;
   prev_statusline_str_width = statusline_str_width;
   prev_candidate_str = candidate_str;
   prev_candidate_col = candidate_col;
-  prev_mode = mode;
+  prev_mode_str = mode_str;
   prev_index_str = index_str;
   prev_index_col = index_col;
 
   statusline_str = get_statusline_str();
   candidate_str = get_candidate_str();
   candidate_col = get_candidate_col();
-  mode = get_mode();
+  mode_str = get_mode_str();
   index_str = get_index_str();
   index_col = get_index_col();
 
@@ -370,8 +382,8 @@ static void draw_statusline(int force, int restore, int visible, int draw_backgr
   debug2(("statusline_str = \"%s\"\n", statusline_str));
   debug2(("candidate_str = \"%s\"\n", candidate_str));
   debug2(("candidate_col = %d\n", candidate_col));
-  debug2(("prev_mode = %d\n", prev_mode));
-  debug2(("mode = %d\n", mode));
+  debug2(("prev_mode_str = %s\n", prev_mode_str));
+  debug2(("mode_str = %s\n", mode_str));
   debug2(("index_str = \"%s\"\n", index_str));
   debug2(("index_col = %d\n", index_col));
 
@@ -490,42 +502,78 @@ static void draw_statusline(int force, int restore, int visible, int draw_backgr
   }
 end_candidate:
 
-  if (force || mode != prev_mode) {
+  if (force || strcmp(mode_str, prev_mode_str) != 0) {
 
     /* ߤΥ⡼ɤUIM_FEP_GETMODE˽񤭹 */
     if (s_path_getmode[0] != '\0') {
       FILE *fp = fopen(s_path_getmode, "wt");
       if (fp) {
+        int mode = get_mode();
         fprintf(fp, "%d\n", mode);
         fclose(fp);
       }
     }
 
     if (g_opt.status_type != NONE && statusline_str[0] == '\0') {
-      char *mode_str = get_mode_str();
-      return_if_fail(mode_str != NULL);
       if (g_opt.status_type == LASTLINE) {
+        int mode_str_width = strwidth(mode_str);
+
+        statusline_str_width = mode_str_width;
+
         if (restore) {
           put_save_cursor();
         }
         put_cursor_invisible();
-        put_goto_lastline(0);
-        put_uim_str(mode_str, UPreeditAttr_None);
-        statusline_str_width = strwidth(mode_str);
-        if (draw_background) {
-          put_clear_to_end_of_line(g_win->ws_col - statusline_str_width);
-        } else if (statusline_str_width < prev_statusline_str_width) {
-          put_clear_to_end_of_line(prev_statusline_str_width - statusline_str_width);
+
+        /* draw_background ʤ force Ǥ */
+        /* ŪˤϴطʤΤ褦ʻȤƤʤ */
+        assert(!draw_background || force);
+
+        if (force) {
+          put_goto_lastline(0);
+          put_uim_str(mode_str, UPreeditAttr_None);
+
+          if (draw_background) {
+            put_clear_to_end_of_line(g_win->ws_col - statusline_str_width);
+          } else if (statusline_str_width < prev_statusline_str_width) {
+            put_clear_to_end_of_line(prev_statusline_str_width - statusline_str_width);
+          }
+
+        } else {
+          /* !force ʤΤ prev_statusline_str_width ϥ⡼ɽĹǤ */
+          int eq_width = compare_str(mode_str, prev_mode_str);
+          int eq_byte = width2byte(mode_str, eq_width)[0];
+          int prev_mode_str_width = strwidth(prev_mode_str);
+
+          put_goto_lastline(eq_width);
+          if (mode_str_width == prev_mode_str_width) {
+            int eq_width_rev = compare_str_rev(mode_str, prev_mode_str);
+
+            if (eq_width_rev > 0) {
+              int draw_byte = width2byte(mode_str + eq_byte, mode_str_width - eq_width - eq_width_rev)[0];
+              put_uim_str_len(mode_str + eq_byte, UPreeditAttr_None, draw_byte);
+
+            } else {
+              put_uim_str(mode_str + eq_byte, UPreeditAttr_None);
+            }
+            
+          } else {
+            put_uim_str(mode_str + eq_byte, UPreeditAttr_None);
+            if (statusline_str_width < prev_statusline_str_width) {
+              put_clear_to_end_of_line(prev_statusline_str_width - statusline_str_width);
+            }
+          }
         }
+
       } else if (g_opt.status_type == BACKTICK) {
-        strncpy(s_modebuf, mode_str, MODESIZE - 1);
+        strncpy(s_modebuf, mode_str, sizeof(s_modebuf) - 1);
       }
-      free(mode_str);
     }
   }
   free(prev_candidate_str);
   free(prev_statusline_str);
   free(prev_index_str);
+  free(prev_mode_str);
   if (restore) {
     put_restore_cursor();
   }
@@ -544,6 +592,12 @@ end_candidate:
  */
 void draw_statusline_restore(void)
 {
+  if (!end_callbacks()) {
+    if (g_opt.status_type == BACKTICK) {
+      update_backtick();
+    }
+    return;
+  }
   draw_statusline(FALSE, TRUE, TRUE, FALSE);
 }
 
@@ -553,6 +607,7 @@ void draw_statusline_restore(void)
  */
 void draw_statusline_force_no_restore(void)
 {
+  end_callbacks();
   draw_statusline(TRUE, FALSE, FALSE, TRUE);
 }
 
@@ -562,6 +617,7 @@ void draw_statusline_force_no_restore(void)
  */
 void draw_statusline_force_restore(void)
 {
+  end_callbacks();
   draw_statusline(TRUE, TRUE, TRUE, TRUE);
 }
 
@@ -595,7 +651,7 @@ static void draw_preedit(struct preedit_tag *preedit, struct preedit_tag *prev_p
   int eq_width;
 
   /* üѹ줿Ȥprev_preedit̵뤹 */
-  eq_width = s_winch ? 0 : compare_preedit(preedit, prev_preedit);
+  eq_width = compare_preedit(preedit, prev_preedit);
 
 #if DEBUG > 2
   debug2(("\neq_width = %d\n", eq_width));
@@ -1074,9 +1130,23 @@ static struct point_tag width2point_col(int width)
 /*
  * üѹ줿Ȥ˸Ƥ
  */
-void draw_winch(void)
+void draw_winch(struct winsize *prev_win)
 {
   s_winch = TRUE;
+  if (g_opt.status_type == LASTLINE) {
+    put_save_cursor();
+    put_cursor_invisible();
+    put_change_scroll_region(0, g_win->ws_row - 1);
+    if (g_win->ws_row > prev_win->ws_row) {
+      struct winsize save_win = *g_win;
+      *g_win = *prev_win;
+      clear_lastline();
+      *g_win = save_win;
+    }
+    draw_statusline_force_no_restore();
+    put_restore_cursor();
+    put_cursor_normal();
+  }
 }
 
 #if defined(DEBUG) && DEBUG > 2
