@@ -45,7 +45,8 @@
 #include <unistd.h>
 #endif
 #include "udsock.h"
-#include "uim-fep.h"
+#define FALSE 0
+#define TRUE 1
 #define EOT 4
 #define BUFSIZE 600
 
@@ -54,8 +55,8 @@ static void usage(void);
 static void version(void);
 int main(int argc, char **argv)
 {
-  char *buf = malloc(BUFSIZE + 1);
-  char *prev_buf = malloc(BUFSIZE + 1);
+  char buf[BUFSIZE + 1];
+  char prev_buf[BUFSIZE + 1];
   const char *sock_path = NULL;
   int op;
 
