@@ -53,13 +53,13 @@
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
-#include <uim/uim.h>
 #ifdef HAVE_STRING_H
 #include <string.h>
 #endif
 #ifdef HAVE_STRINGS_H
 #include <strings.h>
 #endif
+/* solarisǤ term.hcurses.hɬ */
 #ifdef HAVE_CURSES_H
 #include <curses.h>
 #endif
@@ -117,6 +117,7 @@
 #include "escseq.h"
 #include "key.h"
 #include "read.h"
+#include "helper.h"
 
 #define DEFAULT_STATUS LASTLINE
 
@@ -135,8 +136,8 @@ struct opt_tag g_opt = {
 int g_win_in = STDIN_FILENO;
 int g_win_out = STDOUT_FILENO;
 struct winsize *g_win;
+uim_context g_context;
 
-static uim_context s_context;
 /* ümasterΥե뵭һ */
 static int s_master;
 /* ưü */
@@ -147,8 +148,20 @@ static struct termios s_save_tios;
 static char s_path_setmode[MAXPATHLEN];
 static char s_path_getmode[MAXPATHLEN];
 static int s_setmode_fd = -1;
+#ifndef HAVE_SIG_ATOMIC_T
+typedef int sig_atomic_t;
+#endif
+static volatile sig_atomic_t s_signal_flag;
+static sigset_t s_orig_sigmask;
 
-static void init_agent(const char *engine);
+#define SIG_FLAG_DONE    1
+#define SIG_FLAG_RECOVER (1 << 1)
+#define SIG_FLAG_WINCH   (1 << 2)
+#define SIG_FLAG_USR1    (1 << 3)
+#define SIG_FLAG_USR2    (1 << 4)
+#define SIG_FLAG_TSTP    (1 << 5)
+
+static void init_uim(const char *engine);
 static const char *get_default_im_name(void);
 static int make_color_escseq(const char *instr, struct attribute_tag *attr);
 static int colorname2n(const char *name);
@@ -157,12 +170,13 @@ static void main_loop(void);
 static void recover_loop(void);
 static struct winsize *get_winsize(void);
 static void set_signal_handler(void);
-static void recover(int sig_no);
-static void sigtstp_handler(int sig_no);
-static void sigcont_handler(int sig_no);
-static void sigwinch_handler(int sig_no);
-static void sigusr1_handler(int sig_no);
-static void sigusr2_handler(int sig_no);
+static void reset_signal_handler(void);
+static void signal_handler(int sig_no);
+static void recover(void);
+static void sigtstp_handler(void);
+static void sigwinch_handler(void);
+static void sigusr1_handler(void);
+static void sigusr2_handler(void);
 static void usage(void);
 static void version(void);
 
@@ -171,7 +185,7 @@ static void version(void);
  * uim
  * engine Ѵ󥸥̾
  */
-static void init_agent(const char *engine)
+static void init_uim(const char *engine)
 {
   int nr;
   int i;
@@ -179,17 +193,17 @@ static void init_agent(const char *engine)
     printf("uim_init error\n");
     exit(EXIT_FAILURE);
   }
-  s_context = uim_create_context(NULL, get_enc(), NULL, engine, uim_iconv, commit_cb);
-  nr = uim_get_nr_im(s_context);
+  g_context = uim_create_context(NULL, get_enc(), NULL, engine, uim_iconv, commit_cb);
+  nr = uim_get_nr_im(g_context);
   for (i = 0; i < nr; i++) {
-    if (strcmp(engine, uim_get_im_name(s_context, i)) == 0) {
+    if (strcmp(engine, uim_get_im_name(g_context, i)) == 0) {
       break;
     }
   }
   if (i == nr) {
     printf("%s is not a input method\n\n", engine);
     usage();
-    exit(1);
+    exit(EXIT_FAILURE);
   }
 }
 
@@ -224,14 +238,14 @@ int main(int argc, char **argv)
     UNDEFINED  /* background */
   };
   FILE *fp;
+  const char *suffix = NULL;
+  const char *tmp_dir;
+  const char *sty_str;
+  const char *win_str;
+  struct stat stat_buf;
 
   int op;
 
-  if (getenv("UIM_FEP_PID")) {
-    puts("uim-fep is already running");
-    return EXIT_FAILURE;
-  }
-
   if ((command[0] = getenv("SHELL")) == NULL || *command[0] == '\0') {
     struct passwd *pw;
     if ((pw = getpwuid(getuid())) == NULL || *(command[0] = pw->pw_shell) == '\0') {
@@ -243,7 +257,7 @@ int main(int argc, char **argv)
   init_str();
   engine = get_default_im_name();
 
-  while ((op = getopt(argc, argv, "e:s:u:b:w:t:C:SciodKvh")) != -1) {
+  while ((op = getopt(argc, argv, "e:s:u:b:w:t:C:f:SXciodKvh")) != -1) {
     int i;
     switch (op) {
       case 'e':
@@ -277,6 +291,10 @@ int main(int argc, char **argv)
         g_opt.no_report_cursor = TRUE;
         break;
 
+      case 'X':
+        gnu_screen = TRUE;
+        break;
+
       case 'd':
         g_opt.ddskk = TRUE;
         break;
@@ -307,6 +325,10 @@ int main(int argc, char **argv)
         sock_path = optarg;
         break;
 
+      case 'f':
+        suffix = optarg;
+        break;
+
       case 'w':
         g_opt.statusline_width = atoi(optarg);
         if (g_opt.statusline_width <= 0) {
@@ -350,13 +372,6 @@ opt_end:
     return EXIT_FAILURE;
   }
 
-  if (attr_uim.foreground == UNDEFINED) {
-    attr_uim.foreground = FALSE;
-  }
-  if (attr_uim.background == UNDEFINED) {
-    attr_uim.background = FALSE;
-  }
-
   if (gnu_screen) {
     g_opt.status_type = BACKTICK;
     s_master = PROC_FILENO;
@@ -364,6 +379,26 @@ opt_end:
     g_win_out = WIN_OUT_FILENO;
   }
 
+  if (g_opt.status_type == BACKTICK) {
+    init_sendsocket(sock_path);
+  }
+
+  if (getenv("UIM_FEP_PID")) {
+    if (gnu_screen) {
+      sendline("uim-fep is already running");
+    } else {
+      puts("uim-fep is already running");
+    }
+    return EXIT_FAILURE;
+  }
+
+  if (attr_uim.foreground == UNDEFINED) {
+    attr_uim.foreground = FALSE;
+  }
+  if (attr_uim.background == UNDEFINED) {
+    attr_uim.background = FALSE;
+  }
+
   if (!isatty(g_win_in)) {
     g_win_in = open("/dev/tty", O_RDONLY);
   }
@@ -371,18 +406,64 @@ opt_end:
   tcgetattr(g_win_in, &s_save_tios);
   setupterm(NULL, g_win_out, NULL);
 
-  if (getenv("TMP")) {
-    struct passwd *pw = getpwuid(getuid());
-    /* Generate get mode filepath */
-    snprintf(s_path_getmode, sizeof(s_path_getmode), "%s/uim-fep-%s-%d-getmode", getenv("TMP"), pw->pw_name, getpid());
-    /* Generate set mode filepath */
-    snprintf(s_path_setmode, sizeof(s_path_setmode), "%s/uim-fep-%s-%d-setmode", getenv("TMP"), pw->pw_name, getpid());
+  if ((tmp_dir = getenv("TMP")) == NULL) {
+    tmp_dir = "/tmp";
+  }
+
+  sty_str = getenv("STY");
+  win_str = getenv("WINDOW");
+  if (gnu_screen) {
+    if (!(sty_str != NULL && win_str != NULL)) {
+      puts("STY and WINDOW are not defined");
+      return EXIT_FAILURE;
+    }
+    snprintf(s_path_getmode, sizeof(s_path_getmode), "%s/uim-fep-getmode-%s-%s", tmp_dir, sty_str, win_str);
+    snprintf(s_path_setmode, sizeof(s_path_setmode), "%s/uim-fep-setmode-%s-%s", tmp_dir, sty_str, win_str);
+    if (stat(s_path_getmode, &stat_buf) == 0 || stat(s_path_setmode, &stat_buf) == 0) {
+      char msg[100];
+      snprintf(msg, 100, "uim-fep is already running on window %s", win_str);
+      sendline(msg);
+      return EXIT_FAILURE;
+    }
+    if (suffix != NULL) {
+      snprintf(s_path_getmode, sizeof(s_path_getmode), "%s/uim-fep-getmode-%s", tmp_dir, suffix);
+      snprintf(s_path_setmode, sizeof(s_path_setmode), "%s/uim-fep-setmode-%s", tmp_dir, suffix);
+    } else {
+      snprintf(s_path_getmode, sizeof(s_path_getmode), "%s/uim-fep-getmode-%s-%s-screen", tmp_dir, sty_str, win_str);
+      snprintf(s_path_setmode, sizeof(s_path_setmode), "%s/uim-fep-setmode-%s-%s-screen", tmp_dir, sty_str, win_str);
+    }
   } else {
-    struct passwd *pw = getpwuid(getuid());
-    /* Generate get mode filepath */
-    snprintf(s_path_getmode, sizeof(s_path_getmode), "/tmp/uim-fep-%s-%d-getmode", pw->pw_name, getpid());
-    /* Generate set mode filepath */
-    snprintf(s_path_setmode, sizeof(s_path_setmode), "/tmp/uim-fep-%s-%d-setmode", pw->pw_name, getpid());
+    if (sty_str != NULL && win_str != NULL) {
+      snprintf(s_path_getmode, sizeof(s_path_getmode), "%s/uim-fep-getmode-%s-%s-screen", tmp_dir, sty_str, win_str);
+      snprintf(s_path_setmode, sizeof(s_path_setmode), "%s/uim-fep-setmode-%s-%s-screen", tmp_dir, sty_str, win_str);
+      if (stat(s_path_getmode, &stat_buf) == 0 || stat(s_path_setmode, &stat_buf) == 0) {
+        printf("uim-fep is already running on window %s as filter\n", win_str);
+        return EXIT_FAILURE;
+      }
+      if (suffix != NULL) {
+        snprintf(s_path_getmode, sizeof(s_path_getmode), "%s/uim-fep-getmode-%s", tmp_dir, suffix);
+        snprintf(s_path_setmode, sizeof(s_path_setmode), "%s/uim-fep-setmode-%s", tmp_dir, suffix);
+      } else {
+        snprintf(s_path_getmode, sizeof(s_path_getmode), "%s/uim-fep-getmode-%s-%s", tmp_dir, sty_str, win_str);
+        snprintf(s_path_setmode, sizeof(s_path_setmode), "%s/uim-fep-setmode-%s-%s", tmp_dir, sty_str, win_str);
+      }
+    } else {
+      if (suffix != NULL) {
+        snprintf(s_path_getmode, sizeof(s_path_getmode), "%s/uim-fep-getmode-%s", tmp_dir, suffix);
+        snprintf(s_path_setmode, sizeof(s_path_setmode), "%s/uim-fep-setmode-%s", tmp_dir, suffix);
+      } else {
+        int file_suffix = 1;
+        int pid = getpid();
+
+        snprintf(s_path_getmode, sizeof(s_path_getmode), "%s/uim-fep-getmode-%d", tmp_dir, pid);
+        snprintf(s_path_setmode, sizeof(s_path_setmode), "%s/uim-fep-setmode-%d", tmp_dir, pid);
+        while (stat(s_path_getmode, &stat_buf) == 0 || stat(s_path_setmode, &stat_buf) == 0) {
+          snprintf(s_path_getmode, sizeof(s_path_getmode), "%s/uim-fep-getmode-%d-%d", tmp_dir, pid, file_suffix);
+          snprintf(s_path_setmode, sizeof(s_path_setmode), "%s/uim-fep-setmode-%d-%d", tmp_dir, pid, file_suffix);
+          file_suffix++;
+        }
+      }
+    }
   }
 
   env_buf = malloc(30);
@@ -399,6 +480,7 @@ opt_end:
     s_path_getmode[0] = '\0';
   }
 
+  unlink(s_path_setmode);
   if (mkfifo(s_path_setmode, 0600) != -1) {
     unlink(s_path_setmode);
     env_buf = malloc(strlen("UIM_FEP_SETMODE=") + strlen(s_path_setmode) + 1);
@@ -434,14 +516,13 @@ opt_end:
     g_opt.statusline_width = CANDSIZE / 2;
   }
 
-  if (g_opt.status_type == BACKTICK) {
-    init_sendsocket(sock_path);
-  }
-  init_agent(engine);
+  init_uim(engine);
   if (gnu_screen) {
-    uim_set_mode(s_context, 1);
+    uim_set_mode(g_context, 1);
   }
-  init_callbacks(s_context);
+  init_helper();
+  init_callbacks();
+  focus_in();
   init_draw(s_master, s_path_getmode);
   init_escseq(&attr_uim);
   set_signal_handler();
@@ -587,7 +668,7 @@ static pid_t my_forkpty(int *amaster, struct termios *termp, struct winsize *win
     return pid;
   }
 }
-#elif defined(__svr4__) 
+#elif defined(__svr4__) || defined(__sgi__)
 static pid_t my_forkpty(int *amaster, struct termios *termp, struct winsize *winp)
 {
   pid_t pid;
@@ -660,6 +741,9 @@ static void main_loop(void)
       nfd = s_setmode_fd;
     }
   }
+  if (g_helper_fd > nfd) {
+    nfd = g_helper_fd;
+  }
   nfd++;
 
   while (TRUE) {
@@ -669,7 +753,7 @@ static void main_loop(void)
       FD_ZERO(&fds);
       FD_SET(g_win_in, &fds);
       FD_SET(s_master, &fds);
-      if (s_setmode_fd > 0) {
+      if (s_setmode_fd >= 0) {
         FD_SET(s_setmode_fd, &fds);
       }
       t.tv_sec = 0;
@@ -681,21 +765,51 @@ static void main_loop(void)
         debug2(("<end draw_commit_and_preedit>"));
       }
     }
+
     FD_ZERO(&fds);
     FD_SET(g_win_in, &fds);
     FD_SET(s_master, &fds);
-    if (s_setmode_fd > 0) {
+    if (s_setmode_fd >= 0) {
       FD_SET(s_setmode_fd, &fds);
     }
-    if (my_select(nfd, &fds, NULL) <= 0) {
+    if (g_helper_fd >= 0) {
+      FD_SET(g_helper_fd, &fds);
+    }
+
+    if (my_pselect(nfd, &fds, &s_orig_sigmask) <= 0) {
       /* signalǳޤ줿Ȥˤ롣select֤ͤ-1errno==EINTR */
+      debug(("signal flag = %x\n", s_signal_flag));
+      if ((s_signal_flag & SIG_FLAG_DONE   ) != 0) {
+        s_signal_flag &= ~SIG_FLAG_DONE;
+        done(1);
+      }
+      if ((s_signal_flag & SIG_FLAG_RECOVER) != 0) {
+        s_signal_flag &= ~SIG_FLAG_RECOVER;
+        recover();
+      }
+      if ((s_signal_flag & SIG_FLAG_WINCH  ) != 0) {
+        s_signal_flag &= ~SIG_FLAG_WINCH;
+        sigwinch_handler();
+      }
+      if ((s_signal_flag & SIG_FLAG_USR1   ) != 0) {
+        s_signal_flag &= ~SIG_FLAG_USR1;
+        sigusr1_handler();
+      }
+      if ((s_signal_flag & SIG_FLAG_USR2   ) != 0) {
+        s_signal_flag &= ~SIG_FLAG_USR2;
+        sigusr2_handler();
+      }
+      if ((s_signal_flag & SIG_FLAG_TSTP   ) != 0) {
+        s_signal_flag &= ~SIG_FLAG_TSTP;
+        sigtstp_handler();
+      }
       continue;
     }
 
-
     /* ⡼ɤѹ */
-    if (s_setmode_fd > 0 && FD_ISSET(s_setmode_fd, &fds)) {
+    if (s_setmode_fd >= 0 && FD_ISSET(s_setmode_fd, &fds)) {
       int start, end;
+
 #ifdef __CYGWIN32__
       if ((len = read(s_setmode_fd, buf, sizeof(buf) - 1)) <= 0) {
         debug2(("pipe closed\n"));
@@ -705,17 +819,19 @@ static void main_loop(void)
 #else
       len = read(s_setmode_fd, buf, sizeof(buf) - 1);
 #endif
+
       for (end = len - 1; end >= 0 && !isdigit((unsigned char)buf[end]); --end);
       /* ץꥨǥåȤԽǤʤХ⡼ɤѹ */
       if (end >= 0 && !g_start_preedit) {
         int mode;
+
         for (start = end; start > 0 && isdigit((unsigned char)buf[start - 1]); --start);
         buf[end + 1] = '\0';
         mode = atoi(&buf[start]);
-        if (mode != uim_get_current_mode(s_context)) {
+        if (mode != uim_get_current_mode(g_context)) {
           debug2(("mode change %d\n", mode));
-          uim_set_mode(s_context, mode);
-          callbacks_set_mode(uim_get_current_mode(s_context));
+          uim_set_mode(g_context, mode);
+          /* callbacks_set_mode(uim_get_current_mode(g_context)); */
           draw_statusline_restore();
         }
       }
@@ -725,6 +841,9 @@ static void main_loop(void)
     /* ܡ(stdin) */
     if (FD_ISSET(g_win_in, &fds)) {
       int key_state = 0;
+      if (!g_focus_in) {
+        focus_in();
+      }
 
       if ((len = read_stdin(buf, sizeof(buf) - 1)) <= 0) {
         /* ˤϤʤȻפ */
@@ -788,7 +907,11 @@ static void main_loop(void)
             print_key(key, key_state);
           } else {
             int raw = press_key(key, key_state);
-            draw();
+            if (!draw()) {
+              if (g_opt.status_type == BACKTICK) {
+                update_backtick();
+              }
+            }
             if (raw && !g_start_preedit) {
               if (key_state & UMod_Alt) {
                 write(s_master, buf + i - 1, key_len + 1);
@@ -834,6 +957,11 @@ static void main_loop(void)
         put_pty_str(buf, len);
       }
     }
+
+    if (g_helper_fd >= 0 && FD_ISSET(g_helper_fd, &fds)) {
+      helper_handler();
+      draw();
+    }
   }
 }
 
@@ -882,6 +1010,10 @@ static struct winsize *get_winsize(void)
   if (g_opt.status_type == LASTLINE) {
     win->ws_row--;
   }
+#ifdef PTY_TEST
+  win->ws_col = 140;
+  win->ws_row = 50;
+#endif
   return win;
 }
 
@@ -891,41 +1023,87 @@ static struct winsize *get_winsize(void)
 static void set_signal_handler(void)
 {
   struct sigaction act;
+  sigset_t sigmask;
+
+  sigemptyset(&sigmask);
+  sigaddset(&sigmask, SIGHUP);
+  sigaddset(&sigmask, SIGTERM);
+  sigaddset(&sigmask, SIGQUIT);
+  sigaddset(&sigmask, SIGINT);
+  sigaddset(&sigmask, SIGWINCH);
+  sigaddset(&sigmask, SIGUSR1);
+  sigaddset(&sigmask, SIGUSR2);
+  sigaddset(&sigmask, SIGTSTP);
+  sigaddset(&sigmask, SIGCONT);
+  sigprocmask(SIG_BLOCK, &sigmask, &s_orig_sigmask);
+
   /* ʥ֥åʤ */
   sigemptyset(&act.sa_mask);
   act.sa_flags = 0;
   /* ƥॳƼ¹Ԥ(¸) */
   /* act.sa_flags = SA_RESTART; */
 
-  act.sa_handler = done;
+  act.sa_handler = signal_handler;
   sigaction(SIGHUP, &act, NULL);
   sigaction(SIGTERM, &act, NULL);
-
-  /* act.sa_handler = reload_uim; */
   sigaction(SIGQUIT, &act, NULL);
-
-  act.sa_handler = recover;
   sigaction(SIGINT, &act, NULL);
-
-
-  act.sa_handler = sigwinch_handler;
   sigaction(SIGWINCH, &act, NULL);
-
-  act.sa_handler = sigusr1_handler;
   sigaction(SIGUSR1, &act, NULL);
-
-  act.sa_handler = sigusr2_handler;
   sigaction(SIGUSR2, &act, NULL);
-
-  act.sa_handler = sigtstp_handler;
   sigaction(SIGTSTP, &act, NULL);
-
-  act.sa_handler = sigcont_handler;
   sigaction(SIGCONT, &act, NULL);
 }
 
-static void recover(int sig_no)
+static void reset_signal_handler(void)
 {
+  struct sigaction act;
+
+  sigprocmask(SIG_SETMASK, &s_orig_sigmask, NULL);
+
+  sigemptyset(&act.sa_mask);
+  act.sa_flags = 0;
+  act.sa_handler = SIG_DFL;
+  sigaction(SIGHUP, &act, NULL);
+  sigaction(SIGTERM, &act, NULL);
+  sigaction(SIGQUIT, &act, NULL);
+  sigaction(SIGINT, &act, NULL);
+  sigaction(SIGWINCH, &act, NULL);
+  sigaction(SIGUSR1, &act, NULL);
+  sigaction(SIGUSR2, &act, NULL);
+  sigaction(SIGTSTP, &act, NULL);
+  sigaction(SIGCONT, &act, NULL);
+}
+
+static void signal_handler(int sig_no)
+{
+  switch (sig_no) {
+    case SIGHUP:
+    case SIGTERM:
+    case SIGQUIT:
+      s_signal_flag |= SIG_FLAG_DONE;
+      break;
+    case SIGINT:
+      s_signal_flag |= SIG_FLAG_RECOVER;
+      break;
+    case SIGWINCH:
+      s_signal_flag |= SIG_FLAG_WINCH;
+      break;
+    case SIGUSR1:
+      s_signal_flag |= SIG_FLAG_USR1;
+      break;
+    case SIGUSR2:
+      s_signal_flag |= SIG_FLAG_USR2;
+      break;
+    case SIGTSTP:
+      s_signal_flag |= SIG_FLAG_TSTP;
+      break;
+  }
+}
+
+static void recover(void)
+{
+  reset_signal_handler();
   put_exit_attribute_mode();
   put_restore_cursor();
   put_cursor_normal();
@@ -933,10 +1111,13 @@ static void recover(int sig_no)
   done(EXIT_SUCCESS);
 }
 
-static void sigtstp_handler(int sig_no)
+/*
+ * ٻߥʥ
+ */
+static void sigtstp_handler(void)
 {
   struct sigaction act;
-  sigset_t mask;
+  sigset_t sigmask;
 
   quit_escseq();
   put_save_cursor();
@@ -945,48 +1126,38 @@ static void sigtstp_handler(int sig_no)
   sigemptyset(&act.sa_mask);
   act.sa_flags = 0;
   act.sa_handler = SIG_DFL;
-
-  sigemptyset(&mask);
-  sigaddset(&mask, SIGTSTP);
-  sigprocmask(SIG_UNBLOCK, &mask, NULL);
-
   sigaction(SIGTSTP, &act, NULL);
+  sigaction(SIGCONT, &act, NULL);
+
+  sigemptyset(&sigmask);
+  sigaddset(&sigmask, SIGTSTP);
+  sigaddset(&sigmask, SIGCONT);
+  sigprocmask(SIG_UNBLOCK, &sigmask, NULL);
+
   kill(getpid(), SIGTSTP);
-  
-  act.sa_handler = sigtstp_handler;
-  sigaction(SIGTSTP, &act, NULL);
-}
 
-static void sigcont_handler(int sig_no)
-{
+  sigprocmask(SIG_BLOCK, &sigmask, NULL);
+  
+  act.sa_handler = signal_handler;
+  sigaction(SIGTSTP, &act, NULL);
+  sigaction(SIGCONT, &act, NULL);
   fixtty();
 }
 
 /*
  * üΥѤäȤ˲ü礭碌롣
  */
-static void sigwinch_handler(int sig_no)
+static void sigwinch_handler(void)
 {
   struct winsize *prev_win = g_win;
   g_win = get_winsize();
+
   debug2(("g_win->ws_row = %d g_win->ws_col = %d\n", g_win->ws_row, g_win->ws_col));
+
   escseq_winch();
   callbacks_winch();
-  draw_winch();
-  if (g_opt.status_type == LASTLINE) {
-    put_save_cursor();
-    put_cursor_invisible();
-    if (g_win->ws_row > prev_win->ws_row) {
-      struct winsize save_win = *g_win;
-      g_win->ws_row = prev_win->ws_row;
-      clear_lastline();
-      *g_win = save_win;
-    }
-    draw_statusline_force_no_restore();
-    put_change_scroll_region(0, g_win->ws_row - 1);
-    put_restore_cursor();
-    put_cursor_normal();
-  }
+  draw_winch(prev_win);
+
   free(prev_win);
   ioctl(s_master, TIOCSWINSZ, g_win);
 }
@@ -995,11 +1166,12 @@ void done(int exit_value)
 {
   uim_quit();
   quit_escseq();
+  quit_helper();
   if (g_opt.status_type == BACKTICK) {
     clear_backtick();
   }
   tcsetattr(g_win_in, TCSAFLUSH, &s_save_tios);
-  if (s_setmode_fd > 0) {
+  if (s_setmode_fd >= 0) {
     close(s_setmode_fd);
   }
   if (s_path_setmode[0] != '\0') {
@@ -1011,13 +1183,13 @@ void done(int exit_value)
   exit(exit_value);
 }
 
-static void sigusr1_handler(int sig_no)
+static void sigusr1_handler(void)
 {
   press_key(UKey_Private1, 0);
   draw();
 }
 
-static void sigusr2_handler(int sig_no)
+static void sigusr2_handler(void)
 {
   press_key(UKey_Private2, 0);
   draw();
@@ -1037,13 +1209,19 @@ static void usage(void)
   uim_init();
   context = uim_create_context(NULL, get_enc(), NULL, NULL, uim_iconv, commit_cb);
 
-  printf("usage: uim-fep [OPTIONS]\n"
-      "\n"
+  printf("uim-fep version %s\n", PACKAGE_VERSION);
+  printf("Usage: uim-fep [OPTIONS]\n");
+  printf("  or   uim-fep [OPTIONS] -e command arg1 arg2 ...\n");
+  printf("  or   uim-fep [OPTIONS] -S\n");
+  printf("  or   uim-fep [-t <sec>] -K\n");
+  printf("\n");
+  printf("Options\n"
       "-u <input method>                         input method      [default=%s]\n"
       "-s <lastline/backtick/none>               statusline type   [default=%s]\n"
       "-b <file>                                 socket file       [default=%s]\n"
       "-w <width>                                statusline width\n"
       "%s"
+      "%s"
       "-e command arg1 arg2 ...                  executed command  [default=%s]\n"
       "%s",
       get_default_im_name(),
@@ -1055,8 +1233,9 @@ static void usage(void)
       "-c                                        reverse cursor\n"
       "-i                                        use cursor_invisible(civis)\n"
       "-o                                        on the spot\n"
+      "-d                                        ddskk like candidate style\n",
+      "-f                                        file name suffix of $UIM_FEP_SETMODE and $UIM_FEP_GETMODE\n"
       "-S                                        GNU screen mode\n"
-      "-d                                        ddskk like candidate style\n"
       "-K                                        show key code\n",
       getenv("SHELL") != NULL ? getenv("SHELL") : "/bin/sh",
       "-h                                        display this help\n"
@@ -1109,7 +1288,7 @@ static void usage(void)
  */
 static void version(void)
 {
-  printf("uim-fep %s\n", PACKAGE_VERSION);
+  printf("uim-fep version %s\n", PACKAGE_VERSION);
 }
 
 #if defined(DEBUG) && DEBUG > 1
