@@ -45,11 +45,11 @@
 #include <unistd.h>
 #include <locale.h>
 
+#include "config.h"
 #include "uim/uim.h"
 #include "uim/uim-util.h"
 #include "uim/uim-helper.h"
 #include "uim/uim-im-switcher.h"
-#include "uim/config.h"
 #include "uim/gettext.h"
 #include "uim/uim-compat-scm.h"
 #include "uim-cand-win-gtk.h"
@@ -61,7 +61,6 @@ void im_module_list(const GtkIMContextInfo ***contexts, int *n_contexts);
 void im_module_exit(void);
 void im_module_init(GTypeModule *type_module);
 
-/**/
 #define NR_CANDIDATES 20
 #define DEFAULT_SEPARATOR_STR "|"
 
@@ -75,21 +74,25 @@ struct preedit_segment {
 };
 
 typedef struct _IMUIMContext {
-  /**/
   struct _GtkIMContext parent;
   struct _GtkIMContext *slave;
   uim_context uc;
   UIMCandWinGtk *cwin;
   gboolean cwin_is_active;
   int nr_psegs;
+  int prev_preedit_len;
   struct preedit_segment *pseg;
-  /**/
+
   GtkWidget *menu;
   GdkWindow *win;
   GdkWindow *toplevel;
   GtkWidget *caret_state_indicator;
-  GdkRectangle preedit_pos; /* preedit_pos not always point the cursor location */
-  /**/
+  GdkRectangle preedit_pos;
+
+  /* following two members are used when use_preedit == FALSE */
+  GtkWidget *preedit_window;
+  gulong preedit_handler_id;
+
   struct _IMUIMContext *prev, *next;
 } IMUIMContext;
 
@@ -113,18 +116,12 @@ static void im_uim_init(IMUIMContext *uic);
 static void show_preedit(GtkIMContext *ic, GtkWidget *preedit_label);
 
 static void im_uim_helper_disconnect_cb(void);
-/* #ifndef GDK_DISABLE_DEPRECATED */
 static gboolean helper_read_cb(GIOChannel *channel, GIOCondition c, gpointer p);
-static GdkFilterReturn
-toplevel_window_candidate_cb(GdkXEvent *xevent, GdkEvent *ev, gpointer data);
-
-static void
-im_uim_parse_helper_str(const char *str);
+static GdkFilterReturn toplevel_window_candidate_cb(GdkXEvent *xevent, GdkEvent *ev, gpointer data);
 
+static void im_uim_parse_helper_str(const char *str);
 static gboolean get_user_defined_color(PangoColor *color, const gchar *uim_symbol);
-
-static gboolean
-uim_key_snoop(GtkWidget *grab_widget, GdkEventKey *key, gpointer data);
+static gboolean uim_key_snoop(GtkWidget *grab_widget, GdkEventKey *key, gpointer data);
 
 static const GTypeInfo class_info = {
   sizeof(IMContextUIMClass),
@@ -132,7 +129,7 @@ static const GTypeInfo class_info = {
   (GBaseFinalizeFunc) NULL,
   (GClassInitFunc) im_uim_class_init,
   (GClassFinalizeFunc)im_uim_class_finalize,
-  NULL,/*for class data*/
+  NULL, /* for class data */
   sizeof(IMUIMContext), /* size of instance */
   0,
   (GInstanceInitFunc) im_uim_init, /* constructor */
@@ -177,10 +174,11 @@ clear_cb(void *ptr)
 {
   IMUIMContext *uic = (IMUIMContext *)ptr;
   int i;
-  for (i = 0; i < uic->nr_psegs; i++) {
+
+  for (i = 0; i < uic->nr_psegs; i++)
     free(uic->pseg[i].str);
-  }
   free(uic->pseg);
+
   uic->pseg = NULL;
   uic->nr_psegs = 0;
 }
@@ -192,23 +190,46 @@ pushback_cb(void *ptr, int attr, const char *str)
   g_return_if_fail(str);
 
   if (!strcmp(str, "")
-     && !(attr & (UPreeditAttr_Cursor | UPreeditAttr_Separator)))
+      && !(attr & (UPreeditAttr_Cursor | UPreeditAttr_Separator)))
     return;
 
   uic->pseg = realloc(uic->pseg,
-		      sizeof(struct preedit_segment) *
-		      (uic->nr_psegs + 1));
+		      sizeof(struct preedit_segment) * (uic->nr_psegs + 1));
   uic->pseg[uic->nr_psegs].str = g_strdup(str);
   uic->pseg[uic->nr_psegs].attr = attr;
-  uic->nr_psegs ++;
+  uic->nr_psegs++;
+}
+
+static int
+preedit_strlen(IMUIMContext *uic)
+{
+  int i, len = 0;
+  
+  for (i = 0; i < uic->nr_psegs; i++)
+    len += strlen(uic->pseg[i].str);
+
+  return len;
 }
 
 static void
 update_cb(void *ptr)
 {
   IMUIMContext *uic = (IMUIMContext *)ptr;
+  int preedit_len;
+
   g_return_if_fail(uic);
+
+  preedit_len = preedit_strlen(uic);
+
+  if (uic->prev_preedit_len == 0 && preedit_len)
+    g_signal_emit_by_name(uic, "preedit_start");
+
   g_signal_emit_by_name(uic, "preedit_changed");
+  
+  if (uic->prev_preedit_len && preedit_len == 0)
+    g_signal_emit_by_name(uic, "preedit_end");
+
+  uic->prev_preedit_len = preedit_len;
 }
 
 static int
@@ -248,15 +269,15 @@ convert_keyval(int key)
   case GDK_Hyper_R: return UKey_Hyper_key;
   }
 
-  if (key >= GDK_F1 && key <= GDK_F35) {
+  if (key >= GDK_F1 && key <= GDK_F35)
     return key - GDK_F1 + UKey_F1;
-  }
-  if (key >= GDK_KP_0 && key <= GDK_KP_9) {
+
+  if (key >= GDK_KP_0 && key <= GDK_KP_9)
     return key - GDK_KP_0 + UKey_0;
-  }
-  if (key < 256) {
+
+  if (key < 256)
     return key;
-  }
+
   return UKey_Other;
 }
 
@@ -264,51 +285,50 @@ static int
 convert_modifier(int mod)
 {
   int rv = 0;
-  if (mod & GDK_SHIFT_MASK) {
+
+  if (mod & GDK_SHIFT_MASK)
     rv |= UMod_Shift;
-  }
-  if (mod & GDK_CONTROL_MASK) {
+
+  if (mod & GDK_CONTROL_MASK)
     rv |= UMod_Control;
-  }
-  if (mod & GDK_MOD1_MASK) {
+
+  if (mod & GDK_MOD1_MASK)
     rv |= UMod_Alt;
-  }
-  if (mod & GDK_MOD3_MASK) {  /* assuming mod3 */
+
+  if (mod & GDK_MOD3_MASK)  /* assuming mod3 */
     rv |= UMod_Super;
-  }
-  if (mod & GDK_MOD4_MASK) {  /* assuming mod4 */
+
+  if (mod & GDK_MOD4_MASK)  /* assuming mod4 */
     rv |= UMod_Hyper;
-  }
+
   return rv;
 }
 
-
 /*
  * KEY EVENT HANDLER
  */
 static gboolean
-filter_keypress(GtkIMContext *ic,
-		GdkEventKey *key)
+filter_keypress(GtkIMContext *ic, GdkEventKey *key)
 {
   IMUIMContext *uic = IM_UIM_CONTEXT(ic);
 
-  /*** Hack for combination of xchat + GTK+ 2.6 ***/
+  /* Hack for combination of xchat + GTK+ 2.6 */
   if (snooper_installed == FALSE) {
     int rv;
     int kv = convert_keyval(key->keyval);
     int mod = convert_modifier(key->state);
 
-    if (key->type == GDK_KEY_RELEASE) {
+    if (key->type == GDK_KEY_RELEASE)
       rv = uim_release_key(uic->uc, kv, mod);
-    } else {
+    else
       rv = uim_press_key(uic->uc, kv, mod);
-    }
-    if (rv) {
+
+    if (rv)
       return gtk_im_context_filter_keypress(uic->slave, key);
-    }
+
     return TRUE;
   }
-  /*** Hack for combination of xchat + GTK+ 2.6 ***/
+  /* Hack for combination of xchat + GTK+ 2.6 */
 
   return gtk_im_context_filter_keypress(uic->slave, key);
 }
@@ -318,36 +338,41 @@ get_user_defined_color(PangoColor *color, const gchar *uim_symbol)
 {
   gboolean parsed = FALSE;
   gchar *literal = uim_symbol_value_str(uim_symbol);
-  if (literal != NULL && literal[0] != '\0') {
+
+  if (literal != NULL && literal[0] != '\0')
     parsed = pango_color_parse(color, literal);
-  }
+
   g_free(literal);
+
   return parsed;
 }
 
 static char *
-get_preedit_segment(struct preedit_segment *ps,
-		    PangoAttrList *attrs,
-		    char *str)
+get_preedit_segment(struct preedit_segment *ps, PangoAttrList *attrs, char *str)
 {
   PangoAttribute *attr;
   const gchar *segment_str = ps->str;
-  if ((ps->attr & UPreeditAttr_Separator) && !strcmp(segment_str, "")) {
+
+  if ((ps->attr & UPreeditAttr_Separator) && !strcmp(segment_str, ""))
     segment_str = DEFAULT_SEPARATOR_STR;
-  }
+
   if (attrs) {
     PangoColor color;
     int begin, end;
+
     begin = strlen(str);
     end = begin + strlen(segment_str);
+
     if (ps->attr & UPreeditAttr_UnderLine) {
       attr = pango_attr_underline_new(PANGO_UNDERLINE_SINGLE);
       attr->start_index = begin;
       attr->end_index = end;
       pango_attr_list_change(attrs, attr);
     }
+
     if (ps->attr & UPreeditAttr_Separator) {
       const gchar *separator_fg_symbol, *separator_bg_symbol;
+
       if (ps->attr & UPreeditAttr_Reverse) {
 	separator_fg_symbol = "reversed-separator-foreground";
 	separator_bg_symbol = "reversed-separator-background";
@@ -355,16 +380,15 @@ get_preedit_segment(struct preedit_segment *ps,
 	separator_fg_symbol = "separator-foreground";
 	separator_bg_symbol = "separator-background";
       }
-      if (get_user_defined_color(&color, separator_fg_symbol))
-      {
+
+      if (get_user_defined_color(&color, separator_fg_symbol)) {
 	attr = pango_attr_foreground_new(color.red, color.green, color.blue);
 	attr->start_index = begin;
 	attr->end_index = end;
 	pango_attr_list_change(attrs, attr);
       }
 
-      if (get_user_defined_color(&color, separator_bg_symbol))
-      {
+      if (get_user_defined_color(&color, separator_bg_symbol)) {
 	attr = pango_attr_background_new(color.red, color.green, color.blue);
 	attr->start_index = begin;
 	attr->end_index = end;
@@ -372,8 +396,7 @@ get_preedit_segment(struct preedit_segment *ps,
       }
     } else if (ps->attr & UPreeditAttr_Reverse) {
       if (get_user_defined_color(&color, "reversed-preedit-foreground")
-	  || pango_color_parse(&color, "#fff"))
-      {
+	  || pango_color_parse(&color, "#fff")) {
 	attr = pango_attr_foreground_new(color.red, color.green, color.blue);
 	attr->start_index = begin;
 	attr->end_index = end;
@@ -381,8 +404,7 @@ get_preedit_segment(struct preedit_segment *ps,
       }
 
       if (get_user_defined_color(&color, "reversed-preedit-background")
-	  || pango_color_parse(&color, "#000"))
-      {
+	  || pango_color_parse(&color, "#000")) {
 	attr = pango_attr_background_new(color.red, color.green, color.blue);
 	attr->start_index = begin;
 	attr->end_index = end;
@@ -390,48 +412,47 @@ get_preedit_segment(struct preedit_segment *ps,
       }
     }
   }
+
   str = (char *)realloc(str, strlen(str) + strlen(segment_str) + 1);
   strcat(str, segment_str);
+
   return str;
 }
 
 
 static void
-im_uim_get_preedit_string(GtkIMContext *ic, gchar **str,
-			  PangoAttrList **attrs,
+im_uim_get_preedit_string(GtkIMContext *ic, gchar **str, PangoAttrList **attrs,
 			  gint *cursor_pos)
 {
   char *tmp;
   int i, pos = 0;
   IMUIMContext *uic = IM_UIM_CONTEXT(ic);
 
-  if (attrs) {
+  if (attrs)
     *attrs = pango_attr_list_new();
-  }
+
   tmp = g_strdup("");
+
   for (i = 0; i < uic->nr_psegs; i++) {
-    if (uic->pseg[i].attr & UPreeditAttr_Cursor) {
+    if (uic->pseg[i].attr & UPreeditAttr_Cursor)
       pos = g_utf8_strlen(tmp, -1);
-    }
-    if (attrs) {
+
+    if (attrs)
       tmp = get_preedit_segment(&uic->pseg[i], *attrs, tmp);
-    } else {
+    else
       tmp = get_preedit_segment(&uic->pseg[i], NULL, tmp);
-    }
   }
-  if (cursor_pos) {
+  if (cursor_pos)
     *cursor_pos = pos;
-  }
-  if (str) {
+
+  if (str)
     *str = tmp;
-  } else {
+  else
     free(tmp);
-  }
 }
 
 static void
-im_uim_set_cursor_location(GtkIMContext *ic,
-			   GdkRectangle *area)
+im_uim_set_cursor_location(GtkIMContext *ic, GdkRectangle *area)
 {
   IMUIMContext *uic = IM_UIM_CONTEXT(ic);
 
@@ -442,9 +463,7 @@ im_uim_set_cursor_location(GtkIMContext *ic,
 
 
 static void
-im_uim_commit_cb(GtkIMContext *ic,
-		 const gchar *str,
-		 IMUIMContext *is)
+im_uim_commit_cb(GtkIMContext *ic, const gchar *str, IMUIMContext *is)
 {
   g_return_if_fail(str);
   g_signal_emit_by_name(is, "commit", str);
@@ -488,14 +507,12 @@ focus_in(GtkIMContext *ic)
   uim_prop_label_update(uic->uc);
 
   for (cc = context_list.next; cc != &context_list; cc = cc->next) {
-    if (cc != uic && cc->cwin) {
+    if (cc != uic && cc->cwin)
       gtk_widget_hide(GTK_WIDGET(cc->cwin));
-    }
   }
 
-  if (uic->cwin && uic->cwin_is_active) {
+  if (uic->cwin && uic->cwin_is_active)
     gtk_widget_show(GTK_WIDGET(uic->cwin));
-  }
 }
 
 static void
@@ -509,12 +526,11 @@ focus_out(GtkIMContext *ic)
   }
 
   check_helper_connection();
-
   uim_helper_client_focus_out(uic->uc);
 
-  if (uic->cwin) {
+  if (uic->cwin)
     gtk_widget_hide(GTK_WIDGET(uic->cwin));
-  }
+
   gtk_widget_hide(uic->caret_state_indicator);
 }
 
@@ -528,20 +544,31 @@ im_uim_reset(GtkIMContext *ic)
 static void
 set_use_preedit(GtkIMContext *ic, gboolean use_preedit)
 {
-  GtkWidget *preedit_window;
-  GtkWidget *preedit_label;
-  if (use_preedit == FALSE ) {
-    preedit_window = gtk_window_new(GTK_WINDOW_POPUP);
-    preedit_label = gtk_label_new("");
-    gtk_container_add(GTK_CONTAINER(preedit_window), preedit_label);
+  IMUIMContext *uic = IM_UIM_CONTEXT(ic);
+  GtkWidget *preedit_label = NULL;
 
-    g_signal_connect(G_OBJECT(ic), "preedit-changed",
-		     G_CALLBACK(show_preedit), preedit_label );
-    gtk_widget_show_all(preedit_window);
+  if (use_preedit == FALSE) {
+    if (!uic->preedit_window) {
+      uic->preedit_window = gtk_window_new(GTK_WINDOW_POPUP);
+      preedit_label = gtk_label_new("");
+      gtk_container_add(GTK_CONTAINER(uic->preedit_window), preedit_label);
+    }
+    uic->preedit_handler_id =
+      g_signal_connect(G_OBJECT(ic), "preedit-changed",
+		       G_CALLBACK(show_preedit), preedit_label);
+    gtk_widget_show_all(uic->preedit_window);
+  } else {
+    if (uic->preedit_handler_id) {
+      g_signal_handler_disconnect(G_OBJECT(ic), uic->preedit_handler_id);
+      uic->preedit_handler_id = 0;
+    }
+    if (uic->preedit_window) {
+      gtk_widget_destroy(uic->preedit_window);
+      uic->preedit_window = NULL;
+    }
   }
 }
 
-
 static void
 show_preedit(GtkIMContext *ic, GtkWidget *preedit_label)
 {
@@ -555,37 +582,34 @@ show_preedit(GtkIMContext *ic, GtkWidget *preedit_label)
 
   gtk_im_context_get_preedit_string(ic, &str, &attrs, &cursor_pos);
 
-
   if (strlen(str) > 0) {
-      gint x,y,w,h;
-      PangoLayout *layout;
+    gint x, y, w, h;
+    PangoLayout *layout;
 
-      gtk_label_set_text(GTK_LABEL(preedit_label), str);
-      gtk_label_set_attributes(GTK_LABEL(preedit_label), attrs);
+    gtk_label_set_text(GTK_LABEL(preedit_label), str);
+    gtk_label_set_attributes(GTK_LABEL(preedit_label), attrs);
 
-      gdk_window_get_origin(uic->win, &x, &y);
+    gdk_window_get_origin(uic->win, &x, &y);
 
-      gtk_window_move(GTK_WINDOW(preedit_window),
-		      x + uic->preedit_pos.x,
-		      y + uic->preedit_pos.y );
+    gtk_window_move(GTK_WINDOW(preedit_window),
+		    x + uic->preedit_pos.x,
+		    y + uic->preedit_pos.y);
 
-      layout = gtk_label_get_layout(GTK_LABEL(preedit_label));
+    layout = gtk_label_get_layout(GTK_LABEL(preedit_label));
 
-      pango_layout_get_cursor_pos(layout, 0, NULL, NULL);
+    pango_layout_get_cursor_pos(layout, 0, NULL, NULL);
 
-      pango_layout_get_pixel_size(layout, &w, &h);
-      gtk_window_resize(GTK_WINDOW(preedit_window), w, h);
-
-      gtk_widget_show(preedit_window);
+    pango_layout_get_pixel_size(layout, &w, &h);
+    gtk_window_resize(GTK_WINDOW(preedit_window), w, h);
 
+    gtk_widget_show(preedit_window);
   } else {
-      gtk_label_set_text(GTK_LABEL(preedit_label), "");
-      gtk_widget_hide(preedit_window);
-      gtk_window_resize(GTK_WINDOW(preedit_window), 0, 0);
+    gtk_label_set_text(GTK_LABEL(preedit_label), "");
+    gtk_widget_hide(preedit_window);
+    gtk_window_resize(GTK_WINDOW(preedit_window), 0, 0);
   }
   g_free(str);
   pango_attr_list_unref(attrs);
-
 }
 
 static GdkFilterReturn
@@ -603,6 +627,7 @@ toplevel_window_candidate_cb(GdkXEvent *xevent, GdkEvent *ev, gpointer data)
     gdk_window_get_origin(uic->win, &x, &y);
     uim_cand_win_gtk_layout(uic->cwin, x, y, width, height);
   }
+
   return GDK_FILTER_CONTINUE;
 }
 
@@ -635,16 +660,19 @@ im_uim_init(IMUIMContext *uic)
 {
   uic->win = NULL;
   uic->menu = NULL;
-  uic->pseg = 0;
+  uic->caret_state_indicator = NULL;
+  uic->pseg = NULL;
   uic->nr_psegs = 0;
+  uic->prev_preedit_len = 0;
 
   uic->cwin = uim_cand_win_gtk_new();
   uic->cwin_is_active = FALSE;
+  uic->preedit_window = NULL;
+  uic->preedit_handler_id = 0;
   g_signal_connect(G_OBJECT(uic->cwin), "index-changed",
 		   G_CALLBACK(index_changed_cb), uic);
 }
 
-
 /*
  * DESTRUCTOR
  */
@@ -666,16 +694,25 @@ im_uim_finalize(GObject *obj)
     uic->cwin = NULL;
   }
   if (uic->caret_state_indicator) {
-    guint tag = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(uic->caret_state_indicator), "timeout-tag"));
-    g_source_remove(tag);
+    guint tag = GPOINTER_TO_UINT(g_object_get_data(G_OBJECT(uic->caret_state_indicator), "timeout-tag"));
+    if (tag > 0)
+      g_source_remove(tag);
     gtk_widget_destroy(uic->caret_state_indicator);
     uic->caret_state_indicator = NULL;
   }
 
+  if (uic->preedit_handler_id) {
+    g_signal_handler_disconnect(obj, uic->preedit_handler_id);
+    uic->preedit_handler_id = 0;
+  }
+  if (uic->preedit_window) {
+    gtk_widget_destroy(uic->preedit_window);
+    uic->preedit_window = NULL;
+  }
+
   uim_release_context(uic->uc);
 
-  g_signal_handlers_disconnect_by_func(uic->slave,
-				       (gpointer)im_uim_commit_cb,
+  g_signal_handlers_disconnect_by_func(uic->slave, (gpointer)im_uim_commit_cb,
 				       uic);
   g_object_unref(uic->slave);
   parent_class->finalize(obj);
@@ -690,6 +727,7 @@ static void
 im_uim_class_init(GtkIMContextClass *class)
 {
   GObjectClass *object_class = G_OBJECT_CLASS(class);
+
   parent_class = g_type_class_peek_parent(class);
   class->set_client_window = set_client_window;
   class->filter_keypress = filter_keypress;
@@ -707,50 +745,52 @@ im_uim_class_init(GtkIMContextClass *class)
 static void
 im_uim_class_finalize(GtkIMContextClass *class)
 {
-
 }
 
 static void
 update_prop_list_cb(void *ptr, const char *str)
 {
   IMUIMContext *uic = (IMUIMContext *)ptr;
-  GString *tmp;
+  GString *prop_list;
 
   if (uic != focused_context || disable_focused_context)
     return;
 
-  tmp = g_string_new("");
-  g_string_printf(tmp, "prop_list_update\ncharset=UTF-8\n%s", str);
+  prop_list = g_string_new("");
+  g_string_printf(prop_list, "prop_list_update\ncharset=UTF-8\n%s", str);
 
-  uim_helper_send_message(im_uim_fd, tmp->str);
-  g_string_free(tmp, TRUE);
+  uim_helper_send_message(im_uim_fd, prop_list->str);
+  g_string_free(prop_list, TRUE);
 }
 
 static void
 update_prop_label_cb(void *ptr, const char *str)
 {
   IMUIMContext *uic = (IMUIMContext *)ptr;
-  GString *tmp;
+  GString *prop_label;
   gint x, y;
   uim_bool show_state;
 
   if (uic != focused_context || disable_focused_context)
     return;
 
-  tmp = g_string_new("");
-  g_string_printf(tmp, "prop_label_update\ncharset=UTF-8\n%s", str);
+  prop_label = g_string_new("");
+  g_string_printf(prop_label, "prop_label_update\ncharset=UTF-8\n%s", str);
 
-  uim_helper_send_message(im_uim_fd, tmp->str);
-  g_string_free(tmp, TRUE);
+  uim_helper_send_message(im_uim_fd, prop_label->str);
+  g_string_free(prop_label, TRUE);
 
   show_state = uim_scm_symbol_value_bool("bridge-show-input-state?");
-  if (show_state == UIM_TRUE) {
+  if (show_state == UIM_TRUE && uic->win) {
     gint timeout;
+
     gdk_window_get_origin(uic->win, &x, &y);
     caret_state_indicator_update(uic->caret_state_indicator, x, y, str);
     timeout = uim_scm_symbol_value_int("bridge-show-input-state-time-length");
-    if(timeout != 0)
-      caret_state_indicator_set_timeout(uic->caret_state_indicator, timeout * 1000);
+
+    if (timeout != 0)
+      caret_state_indicator_set_timeout(uic->caret_state_indicator,
+					timeout * 1000);
     gtk_widget_show_all(uic->caret_state_indicator);
   }
 }
@@ -770,30 +810,88 @@ im_uim_send_im_list(void)
 
   msg = g_string_new("im_list\ncharset=UTF-8\n");
   for (i = 0; i < nr; i++) {
+    /*
+     * Return value of uim_get_im_language() is an ISO 639-1
+     * compatible language code such as "ja". Since it is unfriendly
+     * for human reading, we convert it into friendly one by
+     * uim_get_language_name_from_locale() here.
+     */
     const char *name = uim_get_im_name(focused_context->uc, i);
-    /* return value of uim_get_im_language() is an ISO 639-1
-       compatible language code such as "ja". Since it is unfriendly
-       for human reading, we convert it into friendly one by
-       uim_get_language_name_from_locale() here */
     const char *langcode = uim_get_im_language(focused_context->uc, i);
     const char *lang = uim_get_language_name_from_locale(langcode);
     const char *short_desc = uim_get_im_short_desc(focused_context->uc, i);
 
     g_string_append(msg, name);
     g_string_append(msg, "\t");
-    if (lang) g_string_append(msg, lang);
+    if (lang)
+      g_string_append(msg, lang);
     g_string_append(msg, "\t");
-    if (short_desc) g_string_append(msg, short_desc);
+    if (short_desc)
+      g_string_append(msg, short_desc);
     g_string_append(msg, "\t");
-    if (strcmp(name, current_im_name) == 0) {
+    if (strcmp(name, current_im_name) == 0)
       g_string_append(msg, "selected");
-    }
     g_string_append(msg, "\n");
   }
   uim_helper_send_message(im_uim_fd, msg->str);
   g_string_free(msg, TRUE);
 }
 
+/* Copied from helper-common-gtk.c. Maybe we need common GTK+ utility file. */
+static gchar *
+get_charset(gchar *line)
+{
+  gchar **splitted = NULL;
+
+  splitted = g_strsplit(line, "=", 0);
+
+  if (splitted && splitted[0] && splitted[1]
+      && strcmp("charset", splitted[0]) == 0) {
+    gchar *charset = g_strdup(splitted[1]);
+    g_strfreev(splitted);
+    return charset;
+  } else {
+    g_strfreev(splitted);
+    return NULL;
+  }
+}
+
+static void
+commit_string_from_other_process(const gchar *str)
+{
+  gchar **lines = g_strsplit(str, "\n", 0);
+  gchar *commit_string;
+
+  if (!lines || !lines[0] || !lines[1] || !lines[2])
+    return; /* Message is broken, do nothing. */
+
+  /*
+   * If second line exists, we assume the first line as a charset
+   * specifier.  This (rotten) convention is influenced by old design
+   * mistake (character encoding was forgotten!).
+   */
+  if (strcmp(lines[2], "") != 0) {
+    gchar *encoding, *commit_string_utf8;
+
+    encoding = get_charset(lines[1]);
+    commit_string = lines[2];
+    commit_string_utf8 = g_convert(commit_string, strlen(commit_string),
+				   "UTF-8", encoding,
+				   NULL, /* gsize *bytes_read */
+				   NULL, /* size *bytes_written */
+				   NULL); /* GError **error */
+    g_signal_emit_by_name(focused_context, "commit", commit_string_utf8);
+    g_free(encoding);
+    g_free(commit_string_utf8);
+  } else {
+    /* Assuming character encoding as UTF-8. */
+    commit_string = lines[1];
+    g_signal_emit_by_name(focused_context, "commit", commit_string);
+  }
+
+  g_strfreev(lines);
+}
+
 static void
 cand_activate_cb(void *ptr, int nr, int display_limit)
 {
@@ -824,16 +922,13 @@ cand_activate_cb(void *ptr, int nr, int display_limit)
     GdkWindow *toplevel;
 
     toplevel = gdk_window_get_toplevel(uic->win);
-    gdk_window_add_filter(toplevel,
-			  toplevel_window_candidate_cb,
-			  uic);
+    gdk_window_add_filter(toplevel, toplevel_window_candidate_cb, uic);
   }
 }
 
-
-
-/* This function called by libuim.
-   Selected by keyboard, etc. */
+/*
+ * This function called by libuim.  Selected by keyboard, etc.
+ */
 static void
 cand_select_cb(void *ptr, int index)
 {
@@ -842,7 +937,9 @@ cand_select_cb(void *ptr, int index)
 
   gdk_window_get_geometry(uic->win, &x, &y, &width, &height, &depth);
   gdk_window_get_origin(uic->win, &x, &y);
+
   uim_cand_win_gtk_layout(uic->cwin, x, y, width, height);
+
   g_signal_handlers_block_by_func(uic->cwin, (gpointer)index_changed_cb, uic);
   uim_cand_win_gtk_set_index(uic->cwin, index);
   g_signal_handlers_unblock_by_func(uic->cwin, (gpointer)index_changed_cb, uic);
@@ -856,7 +953,9 @@ cand_shift_page_cb(void *ptr, int direction)
 
   gdk_window_get_geometry(uic->win, &x, &y, &width, &height, &depth);
   gdk_window_get_origin(uic->win, &x, &y);
+
   uim_cand_win_gtk_layout(uic->cwin, x, y, width, height);
+
   g_signal_handlers_block_by_func(uic->cwin, (gpointer)index_changed_cb, uic);
   uim_cand_win_gtk_shift_page(uic->cwin, direction);
   uim_set_candidate_index(uic->uc, uic->cwin->candidate_index);
@@ -874,6 +973,7 @@ cand_deactivate_cb(void *ptr)
     gtk_widget_hide(GTK_WIDGET(uic->cwin));
     uim_cand_win_gtk_clear_candidates(uic->cwin);
   }
+
   if (uic->win) {
     GdkWindow *toplevel;
 
@@ -882,7 +982,6 @@ cand_deactivate_cb(void *ptr)
   }
 }
 
-
 GtkIMContext *
 im_module_create(const gchar *context_id)
 {
@@ -907,32 +1006,21 @@ im_module_create(const gchar *context_id)
 
   check_helper_connection();
 
-  /**/
-
-  uim_set_preedit_cb(uic->uc, clear_cb,
-		     pushback_cb, update_cb);
-  uim_set_prop_list_update_cb(uic->uc,
-			      update_prop_list_cb);
-  uim_set_prop_label_update_cb(uic->uc,
-			      update_prop_label_cb);
-  uim_set_candidate_selector_cb(uic->uc,
-				cand_activate_cb,
-				cand_select_cb,
-				cand_shift_page_cb,
-				cand_deactivate_cb);
+  uim_set_preedit_cb(uic->uc, clear_cb, pushback_cb, update_cb);
+  uim_set_prop_list_update_cb(uic->uc, update_prop_list_cb);
+  uim_set_prop_label_update_cb(uic->uc, update_prop_label_cb);
+  uim_set_candidate_selector_cb(uic->uc, cand_activate_cb, cand_select_cb,
+				cand_shift_page_cb, cand_deactivate_cb);
 
   uim_prop_list_update(uic->uc);
 
-
   /* slave exists for using gtk+'s table based input method */
-  uic->slave = g_object_new(GTK_TYPE_IM_CONTEXT_SIMPLE,
-			    NULL);
+  uic->slave = g_object_new(GTK_TYPE_IM_CONTEXT_SIMPLE, NULL);
   g_signal_connect(G_OBJECT(uic->slave), "commit",
 		   G_CALLBACK(im_uim_commit_cb), uic);
-  
+
   uic->caret_state_indicator = caret_state_indicator_new();
 
-  /**/
   uic->next = context_list.next;
   uic->prev = (IMUIMContext *)&context_list;
   context_list.next->prev = uic;
@@ -955,7 +1043,9 @@ im_uim_parse_helper_str_im_change(const char *str)
   gchar **lines = g_strsplit(str, "\n", -1);
   gchar *im_name = lines[1];
   GString *im_name_sym = g_string_new(im_name);
+
   g_string_prepend_c(im_name_sym, '\'');
+
   if (g_str_has_prefix(str, "im_change_this_text_area_only") == TRUE) {
     if (focused_context && disable_focused_context == FALSE) {
       uim_switch_im(focused_context->uc, im_name);
@@ -993,6 +1083,7 @@ im_uim_parse_helper_str(const char *str)
     im_uim_parse_helper_str_im_change(str);
   } else if (g_str_has_prefix(str, "prop_update_custom") == TRUE) {
     IMUIMContext *cc;
+
     lines = g_strsplit(str, "\n", 0);
     if (lines && lines[0] && lines[1] && lines[2]) {
       for (cc = context_list.next; cc != &context_list; cc = cc->next) {
@@ -1001,6 +1092,8 @@ im_uim_parse_helper_str(const char *str)
       }
       g_strfreev(lines);
     }
+  } else if (g_str_has_prefix(str, "custom_reload_notify") == TRUE) {
+    uim_prop_reload_configs();
   } else if (focused_context && !disable_focused_context) {
     if (g_str_has_prefix(str, "prop_list_get") == TRUE) {
       uim_prop_list_update(focused_context->uc);
@@ -1015,13 +1108,13 @@ im_uim_parse_helper_str(const char *str)
     } else if (g_str_has_prefix(str, "im_list_get") == TRUE) {
       im_uim_send_im_list();
     } else if (g_str_has_prefix(str, "commit_string")) {
-      lines = g_strsplit(str, "\n", 0);
-      if (lines && lines[0] && lines[1])
-	g_signal_emit_by_name(focused_context, "commit", lines[1]);
+      commit_string_from_other_process(str);
     } else if (g_str_has_prefix(str, "focus_in") == TRUE) {
       disable_focused_context = TRUE;
-      /* We shouldn't do "focused_context = NULL" here, because some
-	 window manager has some focus related bugs. */
+      /*
+       * We don't set "focused_context = NULL" here, because some
+       * window managers have some focus related bugs??
+       */
     }
   }
 }
@@ -1029,12 +1122,13 @@ im_uim_parse_helper_str(const char *str)
 static gboolean
 helper_read_cb(GIOChannel *channel, GIOCondition c, gpointer p)
 {
-  char *tmp;
+  char *msg;
   int fd = g_io_channel_unix_get_fd(channel);
+
   uim_helper_read_proc(fd);
-  while ((tmp = uim_helper_get_message())) {
-    im_uim_parse_helper_str(tmp);
-    free(tmp);
+  while ((msg = uim_helper_get_message())) {
+    im_uim_parse_helper_str(msg);
+    free(msg);
   }
   return TRUE;
 }
@@ -1055,14 +1149,13 @@ uim_key_snoop(GtkWidget *grab_widget, GdkEventKey *key, gpointer data)
     int kv = convert_keyval(key->keyval);
     int mod = convert_modifier(key->state);
 
-    if (key->type == GDK_KEY_RELEASE) {
+    if (key->type == GDK_KEY_RELEASE)
       rv = uim_release_key(focused_context->uc, kv, mod);
-    } else {
+    else
       rv = uim_press_key(focused_context->uc, kv, mod);
-    }
-    if (rv) {
+
+    if (rv)
       return FALSE;
-    }
     return TRUE;
   }
 
@@ -1072,17 +1165,13 @@ uim_key_snoop(GtkWidget *grab_widget, GdkEventKey *key, gpointer data)
 void
 im_module_init(GTypeModule *type_module)
 {
-  if (uim_init() == -1) {
+  if (uim_init() == -1)
     return;
-  }
 
   context_list.next = (IMUIMContext *)&context_list;
   context_list.prev = (IMUIMContext *)&context_list;
-  type_im_uim =
-    g_type_module_register_type(type_module,
-				GTK_TYPE_IM_CONTEXT,
-				"GtkIMContextUIM",
-				&class_info, 0);
+  type_im_uim = g_type_module_register_type(type_module, GTK_TYPE_IM_CONTEXT,
+					    "GtkIMContextUIM", &class_info, 0);
   uim_cand_win_gtk_register_type(type_module);
 
   /* XXX:This is not recommended way!! */
@@ -1093,9 +1182,9 @@ im_module_init(GTypeModule *type_module)
 void
 im_module_exit(void)
 {
-  if (im_uim_fd != -1) {
+  if (im_uim_fd != -1)
     uim_helper_close_client_fd(im_uim_fd);
-  }
+
   gtk_key_snooper_remove(snooper_id);
   uim_quit();
 }
