@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2004 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2004,2005 uim Project http://uim.freedesktop.org/
 
   All rights reserved.
 
@@ -31,6 +31,7 @@
 
 */
 
+#include <stdlib.h>
 #include <gdk/gdkkeysyms.h>
 #include "uim/config.h"
 #include "uim/gettext.h"
@@ -195,6 +196,22 @@ translate_func(const gchar *path, gpointer data)
   return _(path);
 }
 
+static void
+warn_dict_open()
+{
+  GtkWidget *dialog;
+  const gchar *message;
+
+  message = N_("Couldn't open a library for manipulating the dictionary.\n");
+  dialog = gtk_message_dialog_new(NULL,
+		  		  GTK_DIALOG_MODAL,
+				  GTK_MESSAGE_WARNING,
+				  GTK_BUTTONS_OK,
+				  _(message));
+  gtk_dialog_run(GTK_DIALOG(dialog));
+  gtk_widget_destroy(GTK_WIDGET(dialog));
+}
+
 static void
 word_list_window_init (WordListWindow *window)
 {
@@ -253,6 +270,10 @@ word_list_window_init (WordListWindow *window)
 
 #if 1 /* FIXME! currently the identifier of Anthy is hard coded */
   dict = uim_dict_open(N_("Anthy private dictionary"));
+  if (!dict) {
+    warn_dict_open();
+    exit(EXIT_FAILURE);
+  }
 #endif
   word_list_view_set_dict(WORD_LIST_VIEW(word_list), dict);
 
