@@ -536,7 +536,7 @@ main(int argc, char *argv[])
 
   if (result == -1) {
     fprintf(stderr, "Error:%s\n", get_error_msg());
-    exit(-1);
+    exit(EXIT_FAILURE);
   }
 
   /* connect to uim helper message bus */
