@@ -47,6 +47,8 @@
 
 extern gboolean uim_pref_gtk_value_changed;
 
+extern void uim_pref_gtk_mark_value_changed(void);
+
 static GtkSizeGroup *spin_button_sgroup = NULL;
 
 static struct OListPrefWin {
@@ -93,7 +95,7 @@ custom_check_button_toggled_cb(GtkToggleButton *button, gpointer user_data)
   rv = uim_custom_set(custom);
 
   if (rv) {
-    uim_pref_gtk_value_changed = TRUE;
+    uim_pref_gtk_mark_value_changed();
   } else {
     g_printerr("Failed to set bool value for \"%s\".\n", custom->symbol);
     /* FIXME! reset the widget */
@@ -175,7 +177,7 @@ custom_spin_button_value_changed(GtkSpinButton *spin, gpointer user_data)
   rv = uim_custom_set(custom);
 
   if (rv) {
-    uim_pref_gtk_value_changed = TRUE;
+    uim_pref_gtk_mark_value_changed();
   } else {
     g_printerr("Failed to set int value for \"%s\".\n", custom->symbol);
     /* FIXME! reset the widget */
@@ -299,7 +301,7 @@ custom_entry_changed_cb(GtkEntry *entry, gpointer user_data)
   }
 
   if (rv) {
-    uim_pref_gtk_value_changed = TRUE;
+    uim_pref_gtk_mark_value_changed();
   } else {
     g_printerr("Failed to set str value for \"%s\".\n", custom->symbol);
     /* FIXME! reset the widget */
@@ -432,6 +434,7 @@ add_custom_type_pathname(GtkWidget *vbox, struct uim_custom *custom)
 
   g_signal_connect(G_OBJECT(button), "clicked",
 		   G_CALLBACK(custom_pathname_button_clicked_cb), entry);
+  uim_custom_cb_add(custom->symbol, entry, update_custom_type_string_cb);
 }
 
 
@@ -476,7 +479,7 @@ custom_combo_box_changed(GtkComboBox *combo_box, gpointer user_data)
   rv = uim_custom_set(custom);
 
   if (rv) {
-    uim_pref_gtk_value_changed = TRUE;
+    uim_pref_gtk_mark_value_changed();
   } else {
     g_printerr("Failed to set str value for \"%s\".\n", custom->symbol);
     /* FIXME! reset the widget */
@@ -803,7 +806,7 @@ olist_pref_up_button_clicked_cb(GtkWidget *widget, GtkEntry *olist_entry)
   if (urv == UIM_FALSE)
     return;
 
-  uim_pref_gtk_value_changed = TRUE;
+  uim_pref_gtk_mark_value_changed();
 
   /* sync the view */
   rv = gtk_tree_model_get_iter(model, &iter2, path);
@@ -886,7 +889,7 @@ olist_pref_down_button_clicked_cb(GtkWidget *widget, GtkEntry *olist_entry)
   if (urv == UIM_FALSE)
     goto ERROR;
 
-  uim_pref_gtk_value_changed = TRUE;
+  uim_pref_gtk_mark_value_changed();
 
   /* sync the view */
   gtk_list_store_swap(GTK_LIST_STORE(model), &iter1, &iter2);
@@ -976,7 +979,7 @@ olist_pref_left_button_clicked_cb(GtkWidget *widget, GtkEntry *olist_entry)
   if (urv != UIM_FALSE) {
     sync_value_olist(GTK_ENTRY(olist_entry));
     olist_pref_tree_view_set_value(GTK_ENTRY(olist_entry), TRUE, TRUE);
-    uim_pref_gtk_value_changed = TRUE;
+    uim_pref_gtk_mark_value_changed();
   } else {
     /* error message */
   }
@@ -1043,7 +1046,7 @@ olist_pref_right_button_clicked_cb(GtkWidget *widget, GtkEntry *olist_entry)
   if (urv != UIM_FALSE) {
     sync_value_olist(GTK_ENTRY(olist_entry));
     olist_pref_tree_view_set_value(GTK_ENTRY(olist_entry), TRUE, TRUE);
-    uim_pref_gtk_value_changed = TRUE;
+    uim_pref_gtk_mark_value_changed();
     /* FIXME! reset the selection */
   } else {
     /* error message */
@@ -1582,7 +1585,7 @@ key_pref_add_button_clicked_cb(GtkWidget *widget, GtkEntry *key_entry)
   rv = uim_custom_set(custom);
 
   if (rv != UIM_FALSE) {
-    uim_pref_gtk_value_changed = TRUE;
+    uim_pref_gtk_mark_value_changed();
     model = gtk_tree_view_get_model(GTK_TREE_VIEW(key_pref_win.tree_view));
     gtk_list_store_append(GTK_LIST_STORE(model), &iter);
     gtk_list_store_set(GTK_LIST_STORE(model), &iter,
@@ -1646,7 +1649,7 @@ key_pref_remove_button_clicked_cb(GtkWidget *widget, GtkEntry *key_entry)
     rv = uim_custom_set(custom);
 
     if (rv != UIM_FALSE) {
-      uim_pref_gtk_value_changed = TRUE;
+      uim_pref_gtk_mark_value_changed();
       gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
 
       sync_value_key(key_entry);
