@@ -47,6 +47,7 @@
 
 #define DEFAULT_WINDOW_WIDTH_MAX 800
 #define DEFAULT_WINDOW_HEIGHT_MAX 600
+#define USE_CHANGES_SENSITIVE_OK_BUTTON 0
 
 static GtkWidget *pref_window = NULL;
 static GtkWidget *pref_tree_view = NULL;
@@ -54,6 +55,8 @@ static GtkWidget *pref_hbox = NULL;
 static GtkWidget *current_group_widget = NULL;
 
 gboolean uim_pref_gtk_value_changed = FALSE;
+static GtkWidget *pref_apply_button = NULL;
+static GtkWidget *pref_ok_button = NULL;
 
 enum
 {
@@ -62,6 +65,9 @@ enum
   NUM_COLUMNS
 };
 
+void uim_pref_gtk_mark_value_changed(void);
+void uim_pref_gtk_unmark_value_changed(void);
+
 static gboolean	pref_tree_selection_changed(GtkTreeSelection *selection,
 					     gpointer data);
 static GtkWidget *create_pref_treeview(void);
@@ -69,6 +75,26 @@ static GtkWidget *create_group_widget(const char *group_name);
 static void create_sub_group_widgets(GtkWidget *parent_widget,
 				     const char *parent_group);
 
+void
+uim_pref_gtk_mark_value_changed(void)
+{
+  uim_pref_gtk_value_changed = TRUE;
+  gtk_widget_set_sensitive(pref_apply_button, TRUE);
+#if USE_CHANGES_SENSITIVE_OK_BUTTON
+  gtk_widget_set_sensitive(pref_ok_button, TRUE);
+#endif
+}
+
+void
+uim_pref_gtk_unmark_value_changed(void)
+{
+  uim_pref_gtk_value_changed = FALSE;
+  gtk_widget_set_sensitive(pref_apply_button, FALSE);
+#if USE_CHANGES_SENSITIVE_OK_BUTTON
+  gtk_widget_set_sensitive(pref_ok_button, FALSE);
+#endif
+}
+
 /*
  *  2005-02-10 Takuro Ashie <ashie@homa.ne.jp>
  *    This feature is disabled according to [Anthy-dev 1795].
@@ -188,7 +214,7 @@ quit_confirm(void)
 				    GTK_MESSAGE_QUESTION,
 				    GTK_BUTTONS_YES_NO,
 				    _("Some value(s) have been changed.\n"
-				      "Do you realy quit this program?"));
+				      "Do you really quit this program?"));
     gtk_window_set_transient_for(GTK_WINDOW(dialog), GTK_WINDOW(pref_window));
     gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER_ON_PARENT);
     g_signal_connect(G_OBJECT(dialog), "response",
@@ -269,8 +295,8 @@ ok_button_clicked(GtkButton *button, gpointer user_data)
 
   if (uim_pref_gtk_value_changed) {
     uim_custom_save();
-    uim_custom_broadcast();
-    uim_pref_gtk_value_changed = FALSE;
+    uim_custom_broadcast_reload_request();
+    uim_pref_gtk_unmark_value_changed();
   }
 
   gtk_main_quit();
@@ -283,8 +309,8 @@ apply_button_clicked(GtkButton *button, gpointer user_data)
 
   if (uim_pref_gtk_value_changed) {
     uim_custom_save();
-    uim_custom_broadcast();
-    uim_pref_gtk_value_changed = FALSE;
+    uim_custom_broadcast_reload_request();
+    uim_pref_gtk_unmark_value_changed();
   }
 }
 
@@ -296,6 +322,7 @@ set_to_default_cb(GtkWidget *widget, gpointer data)
   if (GTK_IS_CONTAINER(widget))
     gtk_container_foreach(GTK_CONTAINER(widget),
 			  (GtkCallback) (set_to_default_cb), NULL);
+  uim_pref_gtk_mark_value_changed();
 }
 
 static void
@@ -310,6 +337,7 @@ create_setting_button_box(const char *group_name)
 {
   GtkWidget *setting_button_box;
   GtkWidget *button;
+  GtkTooltips *tooltip;
 
   setting_button_box = gtk_hbutton_box_new();
   gtk_button_box_set_layout(GTK_BUTTON_BOX(setting_button_box), GTK_BUTTONBOX_END);
@@ -320,24 +348,38 @@ create_setting_button_box(const char *group_name)
   g_signal_connect(G_OBJECT(button), "clicked",
 		   G_CALLBACK(defaults_button_clicked), (gpointer) group_name);
   gtk_box_pack_start(GTK_BOX(setting_button_box), button, TRUE, TRUE, 8);
+  tooltip = gtk_tooltips_new();
+  gtk_tooltips_set_tip(tooltip, button, _("Revert all changes to default"), NULL);
+
 
   /* Apply button */
-  button = gtk_button_new_from_stock(GTK_STOCK_APPLY);
-  g_signal_connect(G_OBJECT(button), "clicked",
+  pref_apply_button = gtk_button_new_from_stock(GTK_STOCK_APPLY);
+  g_signal_connect(G_OBJECT(pref_apply_button), "clicked",
 		   G_CALLBACK(apply_button_clicked), (gpointer) group_name);
-  gtk_box_pack_start(GTK_BOX(setting_button_box), button, TRUE, TRUE, 8);
+  gtk_widget_set_sensitive(pref_apply_button, FALSE);
+  gtk_box_pack_start(GTK_BOX(setting_button_box), pref_apply_button, TRUE, TRUE, 8);
+  tooltip = gtk_tooltips_new();
+  gtk_tooltips_set_tip(tooltip, pref_apply_button, _("Apply all changes"), NULL);
 
   /* Cancel button */
   button = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
   g_signal_connect(G_OBJECT(button), "clicked",
 		   G_CALLBACK(quit_confirm), NULL);
   gtk_box_pack_start(GTK_BOX(setting_button_box), button, TRUE, TRUE, 8);
+  tooltip = gtk_tooltips_new();
+  gtk_tooltips_set_tip(tooltip, button, _("Quit this application without applying changes"), NULL);
 
   /* OK button */
-  button = gtk_button_new_from_stock(GTK_STOCK_OK);
-  g_signal_connect(G_OBJECT(button), "clicked",
+  pref_ok_button = gtk_button_new_from_stock(GTK_STOCK_OK);
+  g_signal_connect(G_OBJECT(pref_ok_button), "clicked",
 		   G_CALLBACK(ok_button_clicked), (gpointer) group_name);
-  gtk_box_pack_start(GTK_BOX(setting_button_box), button, TRUE, TRUE, 8);
+  gtk_box_pack_start(GTK_BOX(setting_button_box), pref_ok_button, TRUE, TRUE, 8);
+#if USE_CHANGES_SENSITIVE_OK_BUTTON
+  gtk_widget_set_sensitive(pref_ok_button, FALSE);
+#endif
+  tooltip = gtk_tooltips_new();
+  gtk_tooltips_set_tip(tooltip, pref_ok_button, _("Quit this application with applying changes"), NULL);
+
   return setting_button_box;
 }
 
@@ -346,7 +388,6 @@ create_group_widget(const char *group_name)
 {
   GtkWidget *vbox;
   GtkWidget *group_label;
-  GtkWidget *setting_button_box;
   struct uim_custom_group *group;
   char *label_text;
   vbox = gtk_vbox_new(FALSE, 8);
@@ -370,9 +411,6 @@ create_group_widget(const char *group_name)
 
   uim_custom_group_free(group);
 
-  setting_button_box = create_setting_button_box(group_name);
-  gtk_box_pack_end(GTK_BOX(vbox), setting_button_box, FALSE, FALSE, 8);
-
   return vbox;
 }
 
@@ -439,16 +477,18 @@ create_pref_window(void)
 {
   GtkWidget *window;
   GtkWidget *scrolled_win; /* treeview container */
+  GtkWidget *vbox;
   GdkPixbuf *icon;
 
   pref_window = window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
   
-  icon = gdk_pixbuf_new_from_file(UIM_PIXMAPSDIR "/uim-icon.png", NULL);
+  icon = gdk_pixbuf_new_from_file(UIM_PIXMAPSDIR"/uim-icon.png", NULL);
   gtk_window_set_icon(GTK_WINDOW(pref_window), icon);
 
   g_signal_connect(G_OBJECT (window), "delete_event",
 		   G_CALLBACK (delete_event_cb), NULL);
 
+
   pref_hbox = gtk_hbox_new(FALSE, 8);
 
   scrolled_win = gtk_scrolled_window_new(NULL, NULL);
@@ -460,7 +500,12 @@ create_pref_window(void)
   gtk_box_pack_start(GTK_BOX(pref_hbox), scrolled_win, FALSE, TRUE, 0);
 
   gtk_container_add(GTK_CONTAINER(scrolled_win), create_pref_treeview());
-  gtk_container_add(GTK_CONTAINER(window), pref_hbox);
+
+  vbox = gtk_vbox_new(FALSE, 8);
+  gtk_container_set_border_width(vbox, 8);
+  gtk_box_pack_start(GTK_BOX(vbox), pref_hbox, TRUE, TRUE, 0);
+  gtk_box_pack_start(GTK_BOX(vbox), create_setting_button_box("dummy-group-name"), FALSE, TRUE, 0);
+  gtk_container_add(GTK_CONTAINER(window), vbox);
 
   {
     GdkScreen *scr;
