@@ -76,7 +76,7 @@
 		 (lambda (ac) ;; action handler
 		   (anthy-prepare-activation ac)
 		   (anthy-context-set-on! ac #t)
-		   (anthy-context-set-kana-mode! ac anthy-type-hiragana)))
+                   (anthy-context-change-kana-mode! ac anthy-type-hiragana)))
 
 (register-action 'action_anthy_katakana
 ;;		 (indication-alist-indicator 'action_anthy_katakana
@@ -93,12 +93,7 @@
 		 (lambda (ac)
 		   (anthy-prepare-activation ac)
 		   (anthy-context-set-on! ac #t)
-
-		   (if (= anthy-input-rule-kana
-			  (anthy-context-input-rule ac))
-		       (rk-context-set-rule! (anthy-context-rkc ac)
-					     ja-kana-katakana-rule))
-		   (anthy-context-set-kana-mode! ac anthy-type-katakana)))
+                   (anthy-context-change-kana-mode! ac anthy-type-katakana)))
 
 (register-action 'action_anthy_hankana
 ;;		 (indication-alist-indicator 'action_anthy_hankana
@@ -115,7 +110,7 @@
 		 (lambda (ac)
 		   (anthy-prepare-activation ac)
 		   (anthy-context-set-on! ac #t)
-		   (anthy-context-set-kana-mode! ac anthy-type-hankana)))
+                   (anthy-context-change-kana-mode! ac anthy-type-hankana)))
 
 (register-action 'action_anthy_direct
 ;;		 (indication-alist-indicator 'action_anthy_direct
@@ -179,13 +174,8 @@
 		      anthy-input-rule-kana))
 		 (lambda (ac)
 		   (anthy-prepare-activation ac)
-		   (rk-context-set-rule! (anthy-context-rkc ac)
-					 (if (= (anthy-context-kana-mode ac)
-						anthy-type-katakana)
-						ja-kana-katakana-rule
-						ja-kana-hiragana-rule))
-
 		   (anthy-context-set-input-rule! ac anthy-input-rule-kana)
+		   (anthy-context-change-kana-mode! ac (anthy-context-kana-mode ac))
 		   ;;(define-key anthy-kana-toggle-key? "")
 		   ;;(define-key anthy-latin-key? generic-on-key?)
 		   ;;(define-key anthy-wide-latin-key? "")
@@ -277,7 +267,19 @@
   (lambda (ac)
     (let* ((kana (anthy-context-kana-mode ac))
 	   (opposite-kana (multi-segment-opposite-kana kana)))
-      (anthy-context-set-kana-mode! ac opposite-kana))))
+      (anthy-context-change-kana-mode! ac opposite-kana))))
+
+(define anthy-context-change-kana-mode!
+  (lambda (ac kana-mode)
+    (if (= (anthy-context-input-rule ac)
+           anthy-input-rule-kana)
+        (rk-context-set-rule!
+          (anthy-context-rkc ac)
+          (cond
+            ((= kana-mode anthy-type-hiragana) ja-kana-hiragana-rule)
+            ((= kana-mode anthy-type-katakana) ja-kana-katakana-rule)
+            ((= kana-mode anthy-type-hankana)  ja-kana-hankana-rule))))
+    (anthy-context-set-kana-mode! ac kana-mode)))
 
 ;; TODO: generarize as multi-segment procedure
 ;; side effect: none. rkc will not be altered
@@ -417,6 +419,14 @@
 	  (direct (ja-direct (charcode->string key)))
 	  (rule (anthy-context-input-rule ac)))
       (cond
+       ((and anthy-use-with-vi?
+             (anthy-vi-escape-key? key key-state))
+	(begin
+	  (anthy-flush ac)
+	  (anthy-context-set-on! ac #f)
+	  (anthy-context-set-wide-latin! ac #f)
+          (anthy-commit-raw ac)))
+
        ((anthy-wide-latin-key? key key-state)
 	(begin
 	  (anthy-flush ac)
@@ -436,7 +446,7 @@
 	(anthy-commit-raw ac))
        
        ((anthy-hankaku-kana-key? key key-state)
-	(anthy-context-set-kana-mode! ac anthy-type-hankana))
+	(anthy-context-change-kana-mode! ac anthy-type-hankana))
 
        ((anthy-kana-toggle-key? key key-state)
 	(anthy-context-kana-toggle ac))
@@ -465,17 +475,20 @@
 				   res)
 		(ustr-insert-elem! (anthy-context-raw-ustr ac)
 				   key-str))
-	      (if (not (rk-pending rkc)
-                  (anthy-commit-raw ac))))))))))
+	      (if (not (rk-pending rkc))
+                  (anthy-commit-raw ac)))))))))
 
 (define anthy-has-preedit?
   (lambda (ac)
     (or (not (ustr-empty? (anthy-context-preconv-ustr ac)))
-	(> (length (rk-pending (anthy-context-rkc ac))) 0))))
+	(> (string-length (rk-pending (anthy-context-rkc ac))) 0))))
 
 (define anthy-proc-transposing-state
   (lambda (ac key key-state)
     (cond
+     ((anthy-transpose-as-hiragana-key? key key-state)
+      (anthy-context-set-transposing-type! ac anthy-type-hiragana))
+
      ((anthy-transpose-as-katakana-key? key key-state)
       (anthy-context-set-transposing-type! ac anthy-type-katakana))
 
@@ -483,10 +496,14 @@
       (anthy-context-set-transposing-type! ac anthy-type-hankana))
 
      ((anthy-transpose-as-latin-key? key key-state)
-      (anthy-context-set-transposing-type! ac anthy-type-latin))
+      (if (not (= (anthy-context-input-rule ac)
+                  anthy-input-rule-kana))
+          (anthy-context-set-transposing-type! ac anthy-type-latin)))
 
      ((anthy-transpose-as-wide-latin-key? key key-state)
-      (anthy-context-set-transposing-type! ac anthy-type-wide-latin))
+      (if (not (= (anthy-context-input-rule ac)
+                  anthy-input-rule-kana))
+          (anthy-context-set-transposing-type! ac anthy-type-wide-latin)))
 
      (else
       (begin
@@ -496,7 +513,8 @@
 	(if (not (anthy-commit-key? key key-state))
 	    (begin 
 	      (anthy-context-set-transposing! ac #f)
-	      (anthy-proc-input-state ac key key-state))))))))
+	      (anthy-proc-input-state ac key key-state)
+              (anthy-context-set-commit-raw! ac #f))))))))
 
 (define anthy-proc-input-state-with-preedit
   (lambda (ac key key-state)
@@ -544,7 +562,8 @@
 	  (anthy-flush ac)))
 
        ;; Transposing֤ذܹ
-       ((or (anthy-transpose-as-katakana-key?   key key-state)
+       ((or (anthy-transpose-as-hiragana-key?   key key-state)
+	    (anthy-transpose-as-katakana-key?   key key-state)
 	    (anthy-transpose-as-hankana-key?    key key-state)
 	    (anthy-transpose-as-latin-key?      key key-state)
 	    (anthy-transpose-as-wide-latin-key? key key-state))
@@ -669,6 +688,9 @@
   (lambda (ac)
     (let* ((transposing-type (anthy-context-transposing-type ac)))
       (cond
+       ((= transposing-type anthy-type-hiragana)
+	(anthy-make-whole-string ac #t multi-segment-type-hiragana))
+
        ((= transposing-type anthy-type-katakana)
 	(anthy-make-whole-string ac #t multi-segment-type-katakana))
 
@@ -719,7 +741,7 @@
        (and (not (ustr-cursor-at-beginning? preconv-str))
 	    (cons preedit-underline
 		  (string-append-map-ustr-former extract-kana preconv-str)))
-       (and (> (length pending) 0)
+       (and (> (string-length pending) 0)
 	    (cons preedit-underline pending))
        (and (anthy-has-preedit? ac)
 	    (cons preedit-cursor ""))
@@ -912,6 +934,13 @@
 	   (w (or (ja-direct char)
 		  (ja-wide char))))
       (cond
+       ((and anthy-use-with-vi?
+             (anthy-vi-escape-key? key key-state))
+	(begin
+	  (anthy-flush ac)
+	  (anthy-context-set-wide-latin! ac #f)
+          (anthy-commit-raw ac)))
+
        ((anthy-on-key? key key-state)
 	(anthy-flush ac)
 	(anthy-context-set-on! ac #t))
@@ -938,8 +967,7 @@
 		(anthy-proc-wide-latin ac key key-state)
 		(anthy-proc-raw-state ac key key-state))))
     ;; preedit
-    (anthy-update-preedit ac)
-))
+    (anthy-update-preedit ac)))
 
 
 (define anthy-release-key-handler
