@@ -152,7 +152,7 @@
 	   (and showing-candidate? candidates-changed?)
 	   (and showing-candidate? (not (m17nlib-lib-candidate-show? mid))))
 	  (begin
-	    (im-deactivate-candidate-selector mc mid)
+	    (im-deactivate-candidate-selector mc)
 	    (m17nlib-context-set-showing-candidate! mc #f)))
 
       (if (and
@@ -174,17 +174,18 @@
 	  (im-select-candidate mc (m17nlib-lib-get-candidate-index mid))))))
 
 
-(define m17nlib-append-modifiers
-  (lambda (key key-state key-str)
-    (if (shift-key-mask key-state)
-	(set! key-str (string-append "S-" key-str)))
-    (if (control-key-mask key-state)
-	(set! key-str (string-append "C-" key-str)))
-    (if (alt-key-mask key-state)
-	(set! key-str (string-append "A-" key-str)))
-    (if (meta-key-mask key-state)
-	(set! key-str (string-append "M-" key-str)))
-    key-str))
+(define m17nlib-construct-modifier
+  (lambda (key-state)
+    (let ((key-str ""))
+      (if (shift-key-mask key-state)
+	  (set! key-str (string-append "S-" key-str)))
+      (if (control-key-mask key-state)
+	  (set! key-str (string-append "C-" key-str)))
+      (if (alt-key-mask key-state)
+	  (set! key-str (string-append "A-" key-str)))
+      (if (meta-key-mask key-state)
+	  (set! key-str (string-append "M-" key-str)))
+      key-str)))
 
 (define m17nlib-proc-direct-state
   (lambda (mc key key-state)
@@ -196,52 +197,55 @@
   (lambda (mc)
     (im-commit-raw mc)))
 
-
+(define m17nlib-key-translation-alist
+  '((backspace       . "BackSpace")
+    (delete          . "Delete")
+    (escape          . "Escape")
+    (return          . "Return")
+    (tab             . "Tab")
+    (left            . "Left")
+    (up              . "Up")
+    (right           . "Right")
+    (down            . "Down")
+    (prior           . "Page_Down")
+    (next            . "Page_Up")
+    (home            . "Home")
+    (end             . "End")
+    (zenkaku-hankaku . "")
+    (Multi_key       . "")
+    (Mode_switch     . "")
+    (Henkan_Mode     . "")
+    (Muhenkan        . "")
+    (F1              . "F1")
+    (F2              . "F2")
+    (F3              . "F3")
+    (F4              . "F4")
+    (F5              . "F5")
+    (F6              . "F6")
+    (F7              . "F7")
+    (F8              . "F8")
+    (F9              . "F9")
+    (F10             . "F10")
+    (F11             . "F11")
+    (F12             . "F12")
+    (F13             . "F13")
+    (F14             . "F14")
+    (F15             . "F15")
+    (F16             . "F16")
+    (F17             . "F17")
+    (F18             . "F18")
+    (F19             . "F19")
+    (F20             . "F20")))
+  
 ; Unfortunatelly, we don't have simple way to translate...
 (define m17nlib-translate-ukey-to-mkey
   (lambda (key key-state)
-    (m17nlib-append-modifiers
-     key key-state
-     (cdr (assq key
-		'((backspace       . "BackSpace")
-		  (delete          . "Delete")
-		  (escape          . "Escape")
-		  (return          . "Return")
-		  (tab             . "Tab")
-		  (left            . "Left")
-		  (up              . "Up")
-		  (right           . "Right")
-		  (down            . "Down")
-		  (prior           . "Page_Down")
-		  (next            . "Page_Up")
-		  (home            . "Home")
-		  (end             . "End")
-		  (zenkaku-hankaku . "")
-		  (Multi_key       . "")
-		  (Mode_switch     . "")
-		  (Henkan_Mode     . "")
-		  (Muhenkan        . "")
-		  (F1              . "F1")
-		  (F2              . "F2")
-		  (F3              . "F3")
-		  (F4              . "F4")
-		  (F5              . "F5")
-		  (F6              . "F6")
-		  (F7              . "F7")
-		  (F8              . "F8")
-		  (F9              . "F9")
-		  (F10             . "F10")
-		  (F11             . "F11")
-		  (F12             . "F12")
-		  (F13             . "F13")
-		  (F14             . "F14")
-		  (F15             . "F15")
-		  (F16             . "F16")
-		  (F17             . "F17")
-		  (F18             . "F18")
-		  (F19             . "F19")
-		  (F20             . "F20")))))))
-  
+    (string-append 
+     (m17nlib-construct-modifier key-state)
+     (if (symbol? key)
+	 (cdr (assq key m17nlib-key-translation-alist))
+	 (charcode->string key)))))
+
 (define m17nlib-init-handler
   (lambda (id im arg)
     (m17nlib-context-new id im arg)))
@@ -252,33 +256,31 @@
 
 (define m17nlib-push-key
   (lambda (mc key key-state)
-    (let* ((mid (m17nlib-context-mc-id mc)))
-      (cond
-       ((m17nlib-off-key? key key-state)
-	(m17nlib-context-set-on! mc #f)
-	#t) ;; #t means key event was consumed.
-       ((symbol? key)
-	(let ((mkey (m17nlib-translate-ukey-to-mkey key key-state)))
-	  (m17nlib-lib-push-symbol-key mid mkey)))
-       (else
-	(m17nlib-lib-push-key mid key key-state))))))
+    (let* ((mid (m17nlib-context-mc-id mc))
+	   (mkey (m17nlib-translate-ukey-to-mkey key key-state)))
+      (m17nlib-lib-push-symbol-key mid mkey))))
 
 (define m17nlib-press-key-handler
   (lambda (mc key key-state)
     (let* ((mid (m17nlib-context-mc-id mc)))
       (if (m17nlib-context-on mc)
 	  (if (m17nlib-push-key mc key key-state)
-	      #f ;; Discard key event
+	      #f  ; Key event is consumed in m17n-push-key
 	      (let* ((result (m17nlib-lib-get-result mid))
 		     (consumed? (car result))
 		     (commit-str (cdr result)))
-		(if (string=? commit-str "")
-		    (im-commit-raw mc)
+		(if (m17nlib-off-key? key key-state)
 		    (begin
-		      (im-commit mc commit-str)
-		      (m17nlib-lib-commit mid)
-		      (if (not consumed?)
-			  (im-commit-raw mc))))))
+		      (m17nlib-context-set-on! mc #f)
+		      (if (not (string=? commit-str ""))
+			  (im-commit mc commit-str)))
+		    (if (string=? commit-str "")
+			(im-commit-raw mc)
+			(begin
+			  (im-commit mc commit-str)
+			  (m17nlib-lib-commit mid)
+			  (if (not consumed?)
+			      (im-commit-raw mc)))))))
 	  (m17nlib-proc-direct-state mc key key-state))
       (m17nlib-update-preedit mc)
       (m17nlib-update-candidate mc))))
