@@ -542,6 +542,94 @@
 		 (uim '(string-append-map car '(("c" "C") ("a" "A") ("r" "R")))))))
 
 (define-uim-test-case "testcase util misc"
+  ("test R6RS(SRFI-75) and C89 compliant escape sequences of Gauche"
+   (assert-false (string=? "t" "\t"))  ;; #\tab
+   (assert-false (string=? "n" "\n"))  ;; #\linefeed
+   (assert-false (string=? "f" "\f"))  ;; #\page
+   (assert-false (string=? "r" "\r"))  ;; #\return
+
+   ;; not supported by Gauche 0.8.6
+   ;; FIXME: enable when Gauche support it
+;;   (assert-false (string=? "a" "\a"))  ;; #\alarm
+;;   (assert-false (string=? "b" "\b"))  ;; #\backspace
+;;   (assert-false (string=? "v" "\v"))  ;; #\vtab
+   )
+
+  ("test string-escape"
+   ;; empty string
+   (assert-equal "\"\""
+		 (uim '(string-escape "")))
+   ;; single character
+   ;; R5RS
+   (assert-equal "\"\\\"\""
+		 (uim '(string-escape "\"")))
+   (assert-equal "\"\\\\\""
+		 (uim '(string-escape "\\")))
+
+   ;; R6RS(SRFI-75) and C89 (uim-sh)
+   (assert-equal "\"\\t\""
+		 (uim '(string-escape "\t")))  ;; #\tab
+   (assert-equal "\"\\n\""
+		 (uim '(string-escape "\n")))  ;; #\linefeed
+   (assert-equal "\"\\f\""
+		 (uim '(string-escape "\f")))  ;; #\page
+   (assert-equal "\"\\r\""
+		 (uim '(string-escape "\r")))  ;; #\return
+
+   ;; R6RS(SRFI-75) and C89 (uim-sh), but cannot test since Gauche
+   ;; does not support the escape sequences.
+   ;; FIXME: enable when Gauche support it
+;;   (assert-equal "\"\\a\""
+;;		 (uim '(string-escape "\a")))  ;; #\alarm
+;;   (assert-equal "\"\\b\""
+;;		 (uim '(string-escape "\b")))  ;; #\backspace
+;;   (assert-equal "\"\\v\""
+;;		 (uim '(string-escape "\v")))  ;; #\vtab
+
+   ;; R5RS
+   (assert-equal "\"a\""
+		 (uim '(string-escape "a")))
+   (assert-equal "\"b\""
+		 (uim '(string-escape "b")))
+   (assert-equal "\"c\""
+		 (uim '(string-escape "c")))
+   (assert-equal "\"a\""
+		 (uim '(string-escape "\a")))
+   (assert-equal "\"b\""
+		 (uim '(string-escape "\b")))
+   (assert-equal "\"c\""
+		 (uim '(string-escape "\c")))
+   (assert-equal "\"A\""
+		 (uim '(string-escape "A")))
+   (assert-equal "\"B\""
+		 (uim '(string-escape "B")))
+   (assert-equal "\"C\""
+		 (uim '(string-escape "C")))
+   (assert-equal "\"A\""
+		 (uim '(string-escape "\A")))
+   (assert-equal "\"B\""
+		 (uim '(string-escape "\B")))
+   (assert-equal "\"C\""
+		 (uim '(string-escape "\C")))
+   ;; 2 characters
+   (assert-equal "\"\\\"\\\"\""
+		 (uim '(string-escape "\"\"")))
+   (assert-equal "\"\\\\\\\"\""
+		 (uim '(string-escape "\\\"")))
+   (assert-equal "\"\\\\\\\\\""
+		 (uim '(string-escape "\\\\")))
+   (assert-equal "\"\\r\\n\""
+		 (uim '(string-escape "\r\n")))
+   (assert-equal "\"aB\""
+		 (uim '(string-escape "aB")))
+   (assert-equal "\"aB\""
+		 (uim '(string-escape "a\B")))
+   (assert-equal "\"aB\""
+		 (uim '(string-escape "\a\B")))
+   ;; complex
+   (assert-equal "\"\\\"a string\\\" in two-line\\nstring\\n\""
+		 (uim '(string-escape "\"a\ string\" in two-line\nstring\n"))))
+
   ("test compose"
    (uim '(define test-list '(0 1 2 3 4 5)))
    (assert-true  (uim-bool '(procedure? (compose))))
