@@ -30,91 +30,20 @@
   SUCH DAMAGE.
 */
 /**/
-#include <dlfcn.h>
 #include <stdlib.h>
+#include <anthy/anthy.h>
+
 #include "uim-scm.h"
 #include "uim-compat-scm.h"
-#include "context.h"
 #include "plugin.h"
 
 #define MAX_CONTEXT 256
 
-#ifdef __APPLE__
-  #define LIBANTHY_SO    "libanthy.dylib"
-  #define LIBANTHYDIC_SO "libanthydic.dylib"
-#else
-  #define LIBANTHY_SO    "libanthy.so"
-  #define LIBANTHYDIC_SO "libanthydic.so"
-
-#endif /*__APPLE__*/
-
-struct anthy_context;
-
-struct anthy_conv_stat{
-  int nr_segment;
-};
-
-struct anthy_segment_stat{
-  int nr_candidate;
-  int seg_len;
-};
-
-static struct anthy_api {
-  void *diclib, *lib;
-
-  int (*init)(void);
-  int (*quit)(void);
-  struct anthy_context *(*create_context)(void);
-  void (*release_context)(struct anthy_context *);
-  void (*set_string)(struct anthy_context *, char *);
-  void (*get_stat)(struct anthy_context *, struct anthy_conv_stat *);
-  void (*get_segment_stat)(struct anthy_context *, int,
-			   struct anthy_segment_stat *);
-  int (*get_segment)(struct anthy_context *, int, int, char *, int);
-  void (*resize_segment)(struct anthy_context *, int, int);
-  void (*commit_segment)(struct anthy_context *, int, int);
-} api;
-
 static struct context {
-  struct anthy_context *ac;
+  anthy_context_t ac;
 } *context_slot;
 
-static int
-get_anthy_api(void)
-{
-  api.diclib = dlopen(LIBANTHYDIC_SO, RTLD_GLOBAL |RTLD_NOW);
-  if (!api.diclib) {
-	/*   fprintf (stdout, "%s\n", dlerror());*/
-    return -1;
-  }
-  api.lib = dlopen(LIBANTHY_SO, RTLD_NOW);
-  if (!api.lib) {
-	/*   fprintf (stdout, "%s\n", dlerror()); */
-    dlclose(api.diclib);
-    return -1;
-  }
-  *(int  **)(&api.init) = dlsym(api.lib, "anthy_init");
-  *(int  **)(&api.quit) = dlsym(api.lib, "anthy_quit");
-  *(struct anthy_context **)(&api.create_context) = dlsym(api.lib, "anthy_create_context");
-  *(void **)(&api.release_context) = dlsym(api.lib, "anthy_release_context");
-  *(void **)(&api.set_string) = dlsym(api.lib, "anthy_set_string");
-  *(void **)(&api.get_stat) = dlsym(api.lib, "anthy_get_stat");
-  *(void **)(&api.get_segment_stat) = dlsym(api.lib, "anthy_get_segment_stat");
-  *(int  **)(&api.get_segment) = dlsym(api.lib, "anthy_get_segment");
-  *(void **)(&api.resize_segment) = dlsym(api.lib, "anthy_resize_segment");
-  *(void **)(&api.commit_segment) = dlsym(api.lib, "anthy_commit_segment");
-  if (api.init &&
-      api.quit &&
-      api.create_context &&
-      api.release_context && api.set_string &&
-      api.get_stat && api.get_segment_stat &&
-      api.get_segment && api.resize_segment && api.commit_segment) {
-    return 0;
-  }
-  return -1;
-}
-
-static struct anthy_context *
+static anthy_context_t 
 get_anthy_context(int id)
 {
   if (id < 0 || id >= MAX_CONTEXT) {
@@ -130,10 +59,7 @@ init_anthy_lib(void)
   if (context_slot) {
     return uim_scm_t();
   }
-  if (get_anthy_api() == -1) {
-    return uim_scm_f();
-  }
-  if (api.init() == -1) {
+  if (anthy_init() == -1) {
     return uim_scm_f();
   }
   context_slot = malloc(sizeof(struct context) *
@@ -157,7 +83,7 @@ create_context(void)
 
   for (i = 0; i < MAX_CONTEXT; i++) {
     if (!context_slot[i].ac) {
-      struct anthy_context *ac = api.create_context();
+      anthy_context_t ac = anthy_create_context();
       if (!ac) {
 	return uim_scm_f();
       }
@@ -174,7 +100,7 @@ release_context(uim_lisp id_)
 {
   int id = uim_scm_c_int(id_);
   if (context_slot[id].ac) {
-    api.release_context(context_slot[id].ac);
+    anthy_release_context(context_slot[id].ac);
     context_slot[id].ac = NULL;
   }
   return uim_scm_f();
@@ -185,12 +111,12 @@ set_string(uim_lisp id_, uim_lisp str_)
 {
   int id = uim_scm_c_int(id_);
   char *str;
-  struct anthy_context *ac = get_anthy_context(id);
+  anthy_context_t ac = get_anthy_context(id);
   if (!ac) {
     return uim_scm_f();
   }
   str = uim_scm_c_str(str_);
-  api.set_string(ac, str);
+  anthy_set_string(ac, str);
   free(str);
   return uim_scm_f();
 }
@@ -200,11 +126,11 @@ get_nr_segments(uim_lisp id_)
 {
   int id = uim_scm_c_int(id_);
   struct anthy_conv_stat acs;
-  struct anthy_context *ac = get_anthy_context(id);
+  anthy_context_t ac = get_anthy_context(id);
   if (!ac) {
     return uim_scm_f();
   }
-  api.get_stat(ac, &acs);
+  anthy_get_stat(ac, &acs);
 
   return uim_scm_make_int(acs.nr_segment);
 }
@@ -213,7 +139,7 @@ static uim_lisp
 get_nr_candidates(uim_lisp id_, uim_lisp nth_)
 {
   int id, nth;
-  struct anthy_context *ac;
+  anthy_context_t ac;
   struct anthy_conv_stat cs;
   id = uim_scm_c_int(id_);
   nth = uim_scm_c_int(nth_);
@@ -221,10 +147,10 @@ get_nr_candidates(uim_lisp id_, uim_lisp nth_)
   if (!ac) {
     return uim_scm_f();
   }
-  api.get_stat(ac, &cs);
+  anthy_get_stat(ac, &cs);
   if (nth < cs.nr_segment) {
     struct anthy_segment_stat ss;
-    api.get_segment_stat(ac, nth, &ss);
+    anthy_get_segment_stat(ac, nth, &ss);
     return uim_scm_make_int(ss.nr_candidate);
   }
   return uim_scm_f();
@@ -239,16 +165,16 @@ get_nth_candidate(uim_lisp id_, uim_lisp seg_, uim_lisp nth_)
   int buflen;
   char *buf;
   uim_lisp buf_;
-  struct anthy_context *ac = get_anthy_context(id);
+  anthy_context_t ac = get_anthy_context(id);
   if (!ac) {
     return uim_scm_f();
   }
-  buflen = api.get_segment(ac, seg, nth, NULL, 0);
+  buflen = anthy_get_segment(ac, seg, nth, NULL, 0);
   if (buflen == -1) {
     return uim_scm_f();
   }
   buf = malloc(buflen+1);
-  api.get_segment(ac, seg, nth, buf, buflen+1);
+  anthy_get_segment(ac, seg, nth, buf, buflen+1);
   buf_ = uim_scm_make_str(buf);
   free(buf);
   return buf_;
@@ -258,7 +184,7 @@ static uim_lisp
 get_segment_length(uim_lisp id_, uim_lisp nth_)
 {
   int id, nth;
-  struct anthy_context *ac;
+  anthy_context_t ac;
   struct anthy_conv_stat cs;
   id = uim_scm_c_int(id_);
   nth = uim_scm_c_int(nth_);
@@ -266,10 +192,10 @@ get_segment_length(uim_lisp id_, uim_lisp nth_)
   if (!ac) {
     return uim_scm_f();
   }
-  api.get_stat(ac, &cs);
+  anthy_get_stat(ac, &cs);
   if (nth < cs.nr_segment) {
     struct anthy_segment_stat ss;
-    api.get_segment_stat(ac, nth, &ss);
+    anthy_get_segment_stat(ac, nth, &ss);
     return uim_scm_make_int(ss.seg_len);
   }
   return uim_scm_f();
@@ -281,8 +207,8 @@ resize_segment(uim_lisp id_, uim_lisp seg_, uim_lisp cnt_)
   int id = uim_scm_c_int(id_);
   int seg = uim_scm_c_int(seg_);
   int cnt = uim_scm_c_int(cnt_);
-  struct anthy_context *ac = get_anthy_context(id);
-  api.resize_segment(ac, seg, cnt);
+  anthy_context_t ac = get_anthy_context(id);
+  anthy_resize_segment(ac, seg, cnt);
   return uim_scm_f();
 }
 
@@ -292,11 +218,62 @@ commit_segment(uim_lisp id_, uim_lisp s_, uim_lisp nth_)
   int id = uim_scm_c_int(id_);
   int s = uim_scm_c_int(s_);
   int nth = uim_scm_c_int(nth_);
-  struct anthy_context *ac = get_anthy_context(id);
-  api.commit_segment(ac, s, nth);
+  anthy_context_t ac = get_anthy_context(id);
+  anthy_commit_segment(ac, s, nth);
   return uim_scm_f();
 }
 
+#ifdef HAS_ANTHY_PREDICTION
+static uim_lisp
+set_prediction_src_string(uim_lisp id_, uim_lisp str_)
+{
+  int id = uim_scm_c_int(id_);
+  const char *str = uim_scm_refer_c_str(str_);
+  anthy_context_t ac = get_anthy_context(id);
+  if (!ac) {
+    return uim_scm_f();
+  }
+  anthy_set_prediction_string(ac, str);
+  return uim_scm_f();
+}
+
+static uim_lisp
+get_nr_predictions(uim_lisp id_)
+{
+  int id = uim_scm_c_int(id_);
+  anthy_context_t ac = get_anthy_context(id);
+  struct anthy_prediction_stat ps;
+  if (!ac) {
+    return uim_scm_f();
+  }
+  anthy_get_prediction_stat(ac, &ps);
+  return uim_scm_make_int(ps.nr_prediction);
+}
+
+static uim_lisp
+get_nth_prediction(uim_lisp id_, uim_lisp nth_)
+{
+  int id  = uim_scm_c_int(id_);
+  int nth = uim_scm_c_int(nth_); 
+  int buflen;
+  char *buf;
+  uim_lisp buf_;
+  anthy_context_t ac = get_anthy_context(id);
+  if (!ac) {
+    return uim_scm_f();
+  }
+  buflen = anthy_get_prediction(ac, nth, NULL, 0);
+  if (buflen == -1) {
+    return uim_scm_f();
+  }
+  buf = (char *)malloc(buflen + 1);
+  anthy_get_prediction(ac, nth, buf, buflen + 1);
+  buf_ = uim_scm_make_str(buf);
+  free(buf);
+  return buf_;
+}
+#endif /* HAS_ANTHY_PREDICTION */
+
 void
 uim_plugin_instance_init(void)
 {
@@ -310,6 +287,11 @@ uim_plugin_instance_init(void)
   uim_scm_init_subr_2("anthy-lib-get-segment-length", get_segment_length);
   uim_scm_init_subr_3("anthy-lib-resize-segment", resize_segment);
   uim_scm_init_subr_3("anthy-lib-commit-segment", commit_segment);
+#ifdef HAS_ANTHY_PREDICTION
+  uim_scm_init_subr_2("anthy-lib-set-prediction-src-string", set_prediction_src_string);
+  uim_scm_init_subr_1("anthy-lib-get-nr-predictions", get_nr_predictions);
+  uim_scm_init_subr_2("anthy-lib-get-nth-prediction", get_nth_prediction);
+#endif /* HAS_ANTHY_PREDICTION */
 }
 
 void
@@ -323,19 +305,12 @@ uim_plugin_instance_quit(void)
 
   for (i = 0; i < MAX_CONTEXT; i++) {
     if (context_slot[i].ac) {
-      api.release_context(context_slot[i].ac);
+      anthy_release_context(context_slot[i].ac);
     }
   }
 
-  if (api.quit) {
-    api.quit();
-  }
-  if (api.diclib) {
-    dlclose(api.diclib);
-  }
-  if (api.lib) {
-    dlclose(api.lib);
-  }
+  anthy_quit();
+
   if (context_slot) {
     free(context_slot);
     context_slot = NULL;
