@@ -40,7 +40,6 @@
 
 #include <stdlib.h>
 #include <string.h>
-#include "config.h"
 #ifdef ENABLE_NLS
 #include <locale.h>
 #endif
@@ -50,7 +49,7 @@
 #include "uim-compat-scm.h"
 
 /* for uim_init_intl_subrs() */
-#include "context.h"
+#include "uim-internal.h"
 
 static uim_lisp
 intl_gettext_package()
