@@ -51,7 +51,7 @@
 #include "uim-compat-scm.h"
 #include "uim-compat-scm.h"
 #include "plugin.h"
-#include "context.h"
+#include "uim-internal.h"
 
 #ifndef HAVE_DLFUNC
 #define dlfunc dlsym
@@ -60,6 +60,24 @@
 #define PLUGIN_PREFIX "libuim-"
 #define PLUGIN_SUFFIX ".so"
 
+/*
+ * SIOD's verbose-level compatible definition.
+ * See sigscheme/operations-siod.c for further information.
+ */
+#define UIM_VLEVEL_PLUGIN 3
+
+#ifdef DEBUG
+#define DPRINTFN(n,x)  if ((n) <= uim_scm_get_verbose_level()) fprintf x;
+#else
+#define DPRINTFN(n,x)
+#endif
+
+#if UIM_SCM_GCC4_READY_GC
+static uim_lisp plugin_unload_internal(uim_lisp _name);
+static void uim_quit_plugin_internal(void);
+#endif
+
+
 static uim_lisp 
 plugin_load(uim_lisp _name)
 {
@@ -80,6 +98,8 @@ plugin_load(uim_lisp _name)
     return uim_scm_f();
   }
 
+  DPRINTFN(UIM_VLEVEL_PLUGIN, (stderr, "Searching libuim-%s.so.\n", plugin_name));
+
   for (path_cdr = lib_path;
        !uim_scm_nullp(path_cdr);
        path_cdr = uim_scm_cdr(path_cdr))
@@ -95,12 +115,14 @@ plugin_load(uim_lisp _name)
     fd = open(plugin_lib_filename, O_RDONLY);
     if (fd >= 0) {
       close(fd);
+      DPRINTFN(UIM_VLEVEL_PLUGIN, (stderr, "Found %s.\n", plugin_lib_filename));
       break;
     }
     free(plugin_lib_filename);
     plugin_lib_filename = NULL;
   }
 
+  DPRINTFN(UIM_VLEVEL_PLUGIN, (stderr, "Searching %s.scm.\n", plugin_name));
   for (path_cdr = scm_path;
        !uim_scm_nullp(path_cdr);
        path_cdr = uim_scm_cdr(path_cdr))
@@ -115,6 +137,7 @@ plugin_load(uim_lisp _name)
     fd = open(plugin_scm_filename, O_RDONLY);
     if (fd >= 0) {
       close(fd);
+      DPRINTFN(UIM_VLEVEL_PLUGIN, (stderr, "Found %s.\n", plugin_scm_filename));
       break;
     }
     free(plugin_scm_filename);
@@ -125,7 +148,8 @@ plugin_load(uim_lisp _name)
     free(plugin_scm_filename);
     return uim_scm_f();
   }
-    
+
+  DPRINTFN(UIM_VLEVEL_PLUGIN, (stderr, "Loading libuim-%s.so.\n", plugin_name));
   library = dlopen(plugin_lib_filename, RTLD_NOW);
   free(plugin_lib_filename);
 
@@ -145,6 +169,7 @@ plugin_load(uim_lisp _name)
     return uim_scm_f();
   }
 
+  DPRINTFN(UIM_VLEVEL_PLUGIN, (stderr, "Calling plugin_instance_init() for %s.\n", plugin_name));
   (plugin_instance_init)();
   if (plugin_scm_filename) {
     uim_bool succeeded;
@@ -174,12 +199,28 @@ plugin_load(uim_lisp _name)
 
 static uim_lisp
 plugin_unload(uim_lisp _name)
+#if UIM_SCM_GCC4_READY_GC
 {
+  uim_lisp ret;
+
+  UIM_SCM_GC_PROTECTED_CALL(ret, uim_lisp, plugin_unload_internal, (_name));
+
+  return ret;
+}
+
+static uim_lisp
+plugin_unload_internal(uim_lisp _name)
+#endif
+{
+#if !UIM_SCM_GCC4_READY_GC
   uim_lisp stack_start;
+#endif
   void *library;
   void (*plugin_instance_quit)(void);
 
+#if !UIM_SCM_GCC4_READY_GC
   uim_scm_gc_protect_stack(&stack_start);
+#endif
 
   UIM_EVAL_FSTRING1(NULL, "(plugin-list-query-library \"%s\")",
 		    uim_scm_refer_c_str(_name));
@@ -198,7 +239,11 @@ plugin_unload(uim_lisp _name)
 
   UIM_EVAL_FSTRING1(NULL, "(plugin-list-delete \"%s\")",
 		    uim_scm_refer_c_str(_name));
+
+#if !UIM_SCM_GCC4_READY_GC
   uim_scm_gc_unprotect_stack(&stack_start);
+#endif
+
   return uim_scm_t();
 }
 
@@ -215,11 +260,24 @@ uim_init_plugin(void)
 /* Called from uim_quit */
 void
 uim_quit_plugin(void)
+#if UIM_SCM_GCC4_READY_GC
 {
+  UIM_SCM_GC_PROTECTED_CALL_VOID(uim_quit_plugin_internal, ());
+}
+
+static void
+uim_quit_plugin_internal(void)
+#endif
+{
+#if !UIM_SCM_GCC4_READY_GC
   uim_lisp stack_start;
+#endif
   uim_lisp alist, rest, entry, name;
 
+#if !UIM_SCM_GCC4_READY_GC
   uim_scm_gc_protect_stack(&stack_start);
+#endif
+
   alist = uim_scm_eval_c_string("plugin-alist");
   for (rest = alist; !uim_scm_nullp(rest); rest = uim_scm_cdr(rest)) {
     entry = uim_scm_car(rest);
@@ -227,5 +285,8 @@ uim_quit_plugin(void)
 
     plugin_unload(name);
   }
+
+#if !UIM_SCM_GCC4_READY_GC
   uim_scm_gc_unprotect_stack(&stack_start);
+#endif
 }
