@@ -156,7 +156,7 @@ create_im_list()
 static uim_lisp
 init_scim()
 {
-    fprintf( stderr, "init_scm()\n" );
+    fprintf( stderr, "init_scim()\n" );
     if ( !initialized )
     {
         context_list.clear();
@@ -175,7 +175,7 @@ init_scim()
             return uim_scm_f();
         }
 
-        config = config_module->create_config( "scim" );
+        config = config_module->create_config("scim");
         if ( config.null() )
         {
             fprintf(stderr, "create_config failed\n");
@@ -369,19 +369,39 @@ static SCIMContext *get_context_from_id(int id)
     return NULL;
 }
 
+static int
+ukey_mod_to_skey_mod(int mod)
+{
+  int rv = 0;
+  if (mod & UMod_Shift) {
+    rv |= SCIM_KEY_ShiftMask;
+  }
+  if (mod & UMod_Control) {
+    rv |= SCIM_KEY_ControlMask;
+  }
+  if (mod & UMod_Alt) {
+    rv |= SCIM_KEY_AltMask;
+  }
+  if (mod & UMod_Super) {  /* assuming mod3 */
+    rv |= SCIM_KEY_Mod3Mask;
+  }
+  if (mod & UMod_Hyper) {  /* assuming mod4 */
+    rv |= SCIM_KEY_Mod3Mask;
+  }
+  return rv;
+}
+
 static uim_lisp
 push_key(uim_lisp id_, uim_lisp key_, uim_lisp mod_)
 {
-    fprintf(stderr, "push_key!!!!!\n");
+    fprintf(stderr, "push_key\n");
     int id = uim_scm_c_int( id_ );
     int code = uim_scm_c_int( key_ );
     int mod = uim_scm_c_int( mod_ );
 
-    // FIXME
-    // adhoc
     KeyEvent scim_key;
     scim_key.code = code;
-    scim_key.mask = mod;
+    scim_key.mask = ukey_mod_to_skey_mod(mod);
 
     SCIMContext *ic = get_context_from_id( id );
     if ( ic->instance->process_key_event( scim_key ) )
@@ -395,20 +415,18 @@ push_key(uim_lisp id_, uim_lisp key_, uim_lisp mod_)
 static uim_lisp
 push_symbol_key(uim_lisp id_, uim_lisp key_, uim_lisp mod_)
 {
-    int id = uim_scm_c_int( id_ );
-    char *sym = uim_scm_c_str( key_ );
-
+    int id = uim_scm_c_int(id_);
+    int mod = uim_scm_c_int(mod_);
+    const char *sym = uim_scm_refer_c_str(key_);
     fprintf(stderr, "push_symbol_key = %s\n", sym);
 
     KeyEvent scim_key;
     uim_keysymbol_to_scim_keysymbol(sym, &scim_key);
-    free( sym );
+    scim_key.mask = ukey_mod_to_skey_mod(mod);
 
     SCIMContext *ic = get_context_from_id( id );
-    if ( ic->instance->process_key_event( scim_key ) )
-    {
-
-        return uim_scm_t();
+    if (ic->instance->process_key_event( scim_key )) {
+      return uim_scm_t();
     }
 
     return uim_scm_f();
@@ -431,7 +449,7 @@ get_nth_candidate( uim_lisp id_, uim_lisp idx_ )
 
 static void cb_commit( IMEngineInstanceBase *instance, const WideString &wstr )
 {
-    fprintf(stdout, "cb_commit\n");
+    fprintf(stderr, "cb_commit\n");
     SCIMContext *ic = static_cast<SCIMContext*>(instance->get_frontend_data());
     if ( !ic )
     {
@@ -449,7 +467,7 @@ static void cb_commit( IMEngineInstanceBase *instance, const WideString &wstr )
 
 static void cb_preedit_update( IMEngineInstanceBase *instance, const WideString &wstr, const AttributeList &attr )
 {    
-    fprintf(stdout, "cb_preedit_update : preedit_str = [%s]\n", WideStr_to_CStr(wstr));
+    fprintf(stderr, "cb_preedit_update : preedit_str = [%s]\n", WideStr_to_CStr(wstr));
 
     SCIMContext *ic = static_cast<SCIMContext*>(instance->get_frontend_data());
     if ( !ic )
@@ -466,7 +484,7 @@ static void cb_preedit_update( IMEngineInstanceBase *instance, const WideString
 }
 static void cb_preedit_hide( IMEngineInstanceBase *instance )
 {
-    fprintf(stdout, "cb_preedit_hide\n");
+    fprintf(stderr, "cb_preedit_hide\n");
 
     SCIMContext *ic = static_cast<SCIMContext*>(instance->get_frontend_data());
     if ( !ic )
@@ -481,7 +499,7 @@ static void cb_preedit_hide( IMEngineInstanceBase *instance )
 }
 static void cb_preedit_caret( IMEngineInstanceBase *instance, int caret )
 {
-    fprintf(stdout, "cb_preedit_caret\n");
+    fprintf(stderr, "cb_preedit_caret\n");
 
     SCIMContext *ic = static_cast<SCIMContext*>(instance->get_frontend_data());
     if ( !ic )
@@ -497,7 +515,7 @@ static void cb_preedit_caret( IMEngineInstanceBase *instance, int caret )
 }
 static void cb_lookup_update( IMEngineInstanceBase *instance, const LookupTable &table )
 {
-    fprintf(stdout, "cb_lookup_update\n");
+    fprintf(stderr, "cb_lookup_update\n");
 
     SCIMContext *ic = static_cast<SCIMContext*>(instance->get_frontend_data());
     if ( !ic )
@@ -508,11 +526,11 @@ static void cb_lookup_update( IMEngineInstanceBase *instance, const LookupTable
 }
 static void cb_lookup_show( IMEngineInstanceBase *instance )
 {
-    fprintf(stdout, "cb_lookup_show\n");
+    fprintf(stderr, "cb_lookup_show\n");
 }
 static void cb_lookup_hide( IMEngineInstanceBase *instance )
 {
-    fprintf(stdout, "cb_lookup_hide\n");
+    fprintf(stderr, "cb_lookup_hide\n");
 }
 
 static void
@@ -599,15 +617,15 @@ static void uim_eval_im_deactivate_candidate_selector(int id)
 extern "C" void
 uim_plugin_instance_init(void)
 {
-    uim_scm_init_subr_0((char*)"scim-lib-init", init_scim);
-    uim_scm_init_subr_0((char*)"scim-lib-nr-input-methods", get_nr_input_methods);
-    uim_scm_init_subr_1((char*)"scim-lib-nth-input-method-lang", get_input_method_lang);
-    uim_scm_init_subr_1((char*)"scim-lib-nth-input-method-name", get_input_method_name);
-    uim_scm_init_subr_1((char*)"scim-lib-alloc-context", alloc_id);
-    uim_scm_init_subr_1((char*)"scim-lib-free-context", free_id);
-    uim_scm_init_subr_3((char*)"scim-lib-push-key", push_key);
-    uim_scm_init_subr_3((char*)"scim-lib-push-symbol-key", push_symbol_key);
-    uim_scm_init_subr_2((char*)"sicm-lib-nth-candidate", get_nth_candidate);
+    uim_scm_init_subr_0("scim-lib-init", init_scim);
+    uim_scm_init_subr_0("scim-lib-nr-input-methods", get_nr_input_methods);
+    uim_scm_init_subr_1("scim-lib-nth-input-method-lang", get_input_method_lang);
+    uim_scm_init_subr_1("scim-lib-nth-input-method-name", get_input_method_name);
+    uim_scm_init_subr_1("scim-lib-alloc-context", alloc_id);
+    uim_scm_init_subr_1("scim-lib-free-context", free_id);
+    uim_scm_init_subr_3("scim-lib-push-key", push_key);
+    uim_scm_init_subr_3("scim-lib-push-symbol-key", push_symbol_key);
+    uim_scm_init_subr_2("sicm-lib-nth-candidate", get_nth_candidate);
 }
 
 extern "C" void
