@@ -215,4 +215,10 @@ typedef void (*C_FUNC) (void);
 
 #define TKBUFFERN 5120
 
+void siod_gc_protect (LISP * location);
+#if (NESTED_REPL_C_STRING)
+void siod_gc_protect_stack(LISP *stack_start);
+void siod_gc_unprotect_stack(LISP *stack_start);
+#endif
+
 #endif /* __SIOD_H__ */
