@@ -52,9 +52,28 @@
 #include "uim-scm.h"
 #include "uim-compat-scm.h"
 #include "uim-custom.h"
-#include "context.h"
+#include "uim-internal.h"
 #include "uim-helper.h"
 
+#if 0
+/*
+ * The UIM_CUSTOM_EXPERIMENTAL_MTIME_SENSING is disabled since:
+ *
+ * - file_content_is_same() has a bug which may return invalid result
+ *   when the file size is greater than 4095 bytes
+ *
+ * - The codes aim to save custom-groups that some changes are
+ *   applied, but it should not be achieved by such violent method
+ *   (comparing entire content of saved files). Observing updated
+ *   group in uim-custom client program is recommended way
+ *
+ * - It breaks original behavior. See the comment of
+ *   custom-reload-user-configs in custom-rt.scm
+ *
+ *  -- YamaKen 2005-09-12
+ */
+#define UIM_CUSTOM_EXPERIMENTAL_MTIME_SENSING
+#endif
 
 typedef void (*uim_custom_cb_update_cb_t)(void *ptr, const char *custom_sym);
 typedef void (*uim_custom_global_cb_update_cb_t)(void *ptr);
@@ -113,6 +132,13 @@ static uim_bool for_each_primary_groups(uim_bool (*func)(const char *));
 static uim_bool uim_custom_load_group(const char *group);
 static uim_bool uim_custom_save_group(const char *group);
 
+#if UIM_SCM_GCC4_READY_GC
+static char *literalize_string_internal(const char *str);
+static uim_bool custom_cb_add_internal(const char *hook, const char *validator,
+				       const char *custom_sym, void *ptr,
+				       const char *gate_func, void (*cb)(void));
+#endif
+
 static const char str_list_arg[] = "uim-custom-c-str-list-arg";
 static const char custom_subdir[] = "customs";
 static const char custom_msg_tmpl[] = "prop_update_custom\n%s\n%s\n";
@@ -122,16 +148,36 @@ static uim_lisp return_val;
 
 static char *
 literalize_string(const char *str)
+#if UIM_SCM_GCC4_READY_GC
 {
+  char *ret;
+
+  UIM_SCM_GC_PROTECTED_CALL(ret, char *, literalize_string_internal, (str));
+
+  return ret;
+}
+
+static char *
+literalize_string_internal(const char *str)
+#endif
+{
+#if !UIM_SCM_GCC4_READY_GC
   uim_lisp stack_start;
+#endif
   uim_lisp form;
   char *escaped;
 
+#if !UIM_SCM_GCC4_READY_GC
   uim_scm_gc_protect_stack(&stack_start);
+#endif
+
   form = uim_scm_list2(uim_scm_make_symbol("string-escape"),
 		       uim_scm_make_str(str));
   escaped = uim_scm_c_str(uim_scm_eval(form));
+
+#if !UIM_SCM_GCC4_READY_GC
   uim_scm_gc_unprotect_stack(&stack_start);
+#endif
 
   return escaped;
 }
@@ -833,6 +879,52 @@ uim_custom_load(void)
   }
 }
 
+#ifdef UIM_CUSTOM_EXPERIMENTAL_MTIME_SENSING
+static uim_bool
+file_content_is_same(const char *a_path, const char *b_path)
+{
+  uim_bool ret;
+  FILE *a, *b;
+  char a_buf[4096], b_buf[4096];
+
+  a = fopen(a_path, "r");
+  b = fopen(b_path, "r");
+
+  while(1) {
+    char *a_eof, *b_eof;
+
+    if (!a || !b) {
+      ret = UIM_FALSE;
+      break;
+    }
+
+    a_eof = fgets(a_buf, sizeof(a_buf), a);
+    b_eof = fgets(b_buf, sizeof(b_buf), b);
+
+    if (!a_eof && !b_eof) {
+      ret = UIM_TRUE;
+      break;
+    }
+
+    if ((!a_eof && b_eof) || (a_eof && !b_eof)) {
+      ret = UIM_FALSE;
+      break;
+    }
+
+    if (strcmp(a_buf, b_buf) != 0) {
+      ret = UIM_FALSE;
+      break;
+    }
+  }
+
+  if (a)
+    fclose(a);
+  if (b)
+    fclose(b);
+  return ret;
+}
+#endif
+
 static uim_bool
 uim_custom_save_group(const char *group)
 {
@@ -872,10 +964,26 @@ uim_custom_save_group(const char *group)
 
   if (fclose(file) < 0)
     goto error;
+
   /* rename prepared temporary file to proper name */
   file_path = custom_file_path(group, 0);
+#ifdef UIM_CUSTOM_EXPERIMENTAL_MTIME_SENSING
+  /*
+   * Avoiding a file saving at here by such method is a layer
+   * violation. Observing updated group is recommended way.
+   *   -- YamaKen 2005-08-09
+   */
+  if (file_content_is_same(tmp_file_path, file_path)) {
+    succeeded = UIM_TRUE;
+    remove(tmp_file_path);
+  } else {
+    succeeded = (rename(tmp_file_path, file_path) == 0);
+  }
+#else
   succeeded = (rename(tmp_file_path, file_path) == 0);
+#endif
   free(file_path);
+
  error:
   free(tmp_file_path);
 
@@ -950,6 +1058,29 @@ uim_custom_broadcast(void)
   return UIM_TRUE;
 }
 
+/**
+ * Broadcasts a request to reload custom files to other uim-enabled
+ * application processes via uim-helper-server.
+ *
+ * @retval UIM_TRUE succeeded
+ * @retval UIM_FALSE failed
+ */
+uim_bool
+uim_custom_broadcast_reload_request(void)
+{
+  if (helper_fd < 0) {
+    helper_fd = uim_helper_init_client_fd(helper_disconnect_cb);
+  }
+
+  uim_helper_send_message(helper_fd, "custom_reload_notify\n");
+
+  if (helper_fd != -1) {
+    uim_helper_close_client_fd(helper_fd);
+  }
+
+  return UIM_TRUE;
+}
+
 /**
  * Returns attributes and current value of a custom variable. Returned value
  * must be freed by uim_custom_free().
@@ -1276,12 +1407,32 @@ static uim_bool
 custom_cb_add(const char *hook, const char *validator,
 	      const char *custom_sym, void *ptr,
 	      const char *gate_func, void (*cb)(void))
+#if UIM_SCM_GCC4_READY_GC
+{
+  uim_bool ret;
+
+  UIM_SCM_GC_PROTECTED_CALL(ret, uim_bool, custom_cb_add_internal,
+			    (hook, validator, custom_sym, ptr, gate_func, cb));
+
+  return ret;
+}
+
+static uim_bool
+custom_cb_add_internal(const char *hook, const char *validator,
+		       const char *custom_sym, void *ptr,
+		       const char *gate_func, void (*cb)(void))
+#endif
 {
   uim_bool succeeded;
+#if !UIM_SCM_GCC4_READY_GC
   uim_lisp stack_start;
+#endif
   uim_lisp form;
 
+#if !UIM_SCM_GCC4_READY_GC
   uim_scm_gc_protect_stack(&stack_start);
+#endif
+
   form = uim_scm_list5(uim_scm_make_symbol(validator),
 		       uim_scm_quote(uim_scm_make_symbol(custom_sym)),
 		       uim_scm_make_ptr(ptr),
@@ -1290,7 +1441,10 @@ custom_cb_add(const char *hook, const char *validator,
   form = uim_scm_cons(uim_scm_quote(uim_scm_make_symbol(hook)), form);
   form = uim_scm_cons(uim_scm_make_symbol("custom-register-cb"), form);
   succeeded = uim_scm_c_bool(uim_scm_eval(form));
+
+#if !UIM_SCM_GCC4_READY_GC
   uim_scm_gc_unprotect_stack(&stack_start);
+#endif
 
   return succeeded;
 }
