@@ -74,10 +74,10 @@ static struct client *clients;
 static char read_buf[BUFFER_SIZE];
 
 /*
-  prepare file descriptor.
+  initialize server's file descriptor.
 */
 static int
-init_serv_fd(char *path)
+init_server_fd(char *path)
 {
   int foo;
   int fd;
@@ -150,8 +150,9 @@ get_unused_client(void)
 }
 
 static void
-free_client(struct client *cl)
+close_client(struct client *cl)
 {
+  close(cl->fd);
   if (cl->rbuf) {
     free(cl->rbuf);
     cl->rbuf = strdup("");
@@ -216,19 +217,122 @@ check_session_alive(void)
 }
 
 
-/* FIXME: This function is too long to read... */
+static uim_bool
+accept_new_connection(int server_fd)
+{
+  struct sockaddr_un clientsoc;
+  socklen_t len = sizeof(clientsoc);
+  int new_fd;
+  int flag;
+  struct client *cl;
+  new_fd = accept(server_fd, (struct sockaddr *)&clientsoc, &len);
+  
+  if (new_fd < 0) {
+    perror("accpet failed");
+    return UIM_FALSE;
+  }
+
+  if ((flag = fcntl(new_fd, F_GETFL)) == -1) {
+    close(new_fd);
+    return UIM_FALSE;
+  }
+
+  flag |= O_NONBLOCK;
+  if (fcntl(new_fd, F_SETFL, flag) == -1) {
+    close(new_fd);
+    return UIM_FALSE;
+  }
+
+  cl = get_unused_client();
+  if (!cl) {
+    close(new_fd);
+    return UIM_FALSE;
+  }
+  cl->fd = new_fd;
+#ifdef LOCAL_CREDS	/* for NetBSD */
+  {
+    char buf[1] = { '\0' };
+    write(cl->fd, buf, 1);
+  }
+#endif
+  FD_SET(cl->fd, &s_fdset_read);
+  if (cl->fd > s_max_fd)
+    s_max_fd = cl->fd;
+
+  return UIM_TRUE;
+}
+
 static void
-uim_helper_server_process_connection(int serv_fd)
+write_message(struct client *cl)
+{
+  int ret, message_len, out_len;
+  char *out;
+  
+  out = cl->wbuf;
+  message_len = out_len = strlen(cl->wbuf);
+  while (out_len > 0) {
+    if ((ret = write(cl->fd, out, out_len)) < 0) {
+      if (errno == EAGAIN) {
+#if 0
+	fprintf(stderr, "EAGAIN: fd = %d\n", cl->fd);
+#endif
+      } else {
+	perror("uim-helper_server write(2) failed");
+	if (errno == EPIPE) {
+	  fprintf(stderr, "fd = %d\n", cl->fd);
+	  FD_CLR(cl->fd, &s_fdset_read);
+	  FD_CLR(cl->fd, &s_fdset_write);
+	  if (cl->fd == s_max_fd)
+	    s_max_fd--;
+	  close_client(cl);
+	}
+      }
+      break;
+    } else {
+      out += ret;
+      out_len -= ret;
+    }
+  }
+  if (out_len == 0) {
+    free(cl->wbuf);
+    cl->wbuf = strdup("");
+    FD_CLR(cl->fd, &s_fdset_write);    
+  } else {
+    uim_helper_buffer_shift(cl->wbuf, message_len - out_len);
+  }
+}
+
+
+static void
+read_message(struct client *cl)
+{
+  int result;
+  result = reflect_message_fragment(cl);
+  
+  if (result < 0) {
+    FD_CLR(cl->fd, &s_fdset_read);
+    FD_CLR(cl->fd, &s_fdset_write);
+    if (cl->fd == s_max_fd)
+      s_max_fd--;
+    close_client(cl);
+  }
+}
+
+
+static void
+uim_helper_server_process_connection(int server_fd)
 {
   int i;
-  fd_set readfds;
-  fd_set writefds;
+  fd_set readfds, writefds;
 
   while (1) {
+    /* Could we replace this memcpy with direct assignment? */
+    /* Copy readfds from s_fdset_read/s_fdset_write because select removes
+       readble/writable fd from readfds/writefds */
     memcpy(&readfds, &s_fdset_read, sizeof(fd_set));
     memcpy(&writefds, &s_fdset_write, sizeof(fd_set));
 
-    /* call select(), waiting until a file descriptor readable */
+    /* call select(), waiting until a file descriptor became readable */
     if (select(s_max_fd + 1, &readfds, &writefds, NULL, NULL) <= 0) {
       perror("uim-helper_server select(2) failed");
       sleep(3);
@@ -236,99 +340,20 @@ uim_helper_server_process_connection(int serv_fd)
     }
 
     /* for accept new connection */
-    if (FD_ISSET(serv_fd, &readfds)) {
-      struct sockaddr_un clientsoc;
-      socklen_t len = sizeof(clientsoc);
-      int new_fd;
-      int flag;
-      struct client *cl;
-      new_fd = accept(serv_fd, (struct sockaddr *)&clientsoc, &len);
-
-      if (new_fd < 0) {
-	perror("accpet failed");
-	continue;
+    if (FD_ISSET(server_fd, &readfds)) {
+      uim_bool accepted;
+      accepted = accept_new_connection(server_fd);
+      if(accepted == UIM_FALSE) {
+	continue; /* acception failed, go next loop without message processing. */
       }
-
-      if ((flag = fcntl(new_fd, F_GETFL)) == -1) {
-	close(new_fd);
-	continue;
-      }
-
-      flag |= O_NONBLOCK;
-      if (fcntl(new_fd, F_SETFL, flag) == -1) {
-	close(new_fd);
-	continue;
-      }
-
-      cl = get_unused_client();
-      if (!cl) {
-	close(new_fd);
-	continue;
-      }
-      cl->fd = new_fd;
-#ifdef LOCAL_CREDS	/* for NetBSD */
-      {
-	char buf[1] = { '\0' };
-	write(cl->fd, buf, 1);
-      }
-#endif
-      FD_SET(cl->fd, &s_fdset_read);
-      if (cl->fd > s_max_fd)
-	s_max_fd = cl->fd;
     } else {
       /* check data to write and from clients reached */
       for (i = 0; i < nr_client_slots; i++) {
 	if (clients[i].fd != -1 && FD_ISSET(clients[i].fd, &writefds)) {
-	  int ret, message_len, out_len;
-	  char *out;
-
-	  out = clients[i].wbuf;
-	  message_len = out_len = strlen(clients[i].wbuf);
-	  while (out_len > 0) {
-	    if ((ret = write(clients[i].fd, out, out_len)) < 0) {
-	      if (errno == EAGAIN) {
-#if 0
-		fprintf(stderr, "EAGAIN: fd = %d\n", clients[i].fd);
-#endif
-	      } else {
-		perror("uim-helper_server write(2) failed");
-		if (errno == EPIPE) {
-		  fprintf(stderr, "fd = %d\n", clients[i].fd);
-		  FD_CLR(clients[i].fd, &s_fdset_read);
-		  FD_CLR(clients[i].fd, &s_fdset_write);
-		  if (clients[i].fd == s_max_fd)
-		    s_max_fd--;
-		  close(clients[i].fd);
-		  free_client(&clients[i]);
-		}
-	      }
-	      break;
-	    } else {
-	      out += ret;
-	      out_len -= ret;
-	    }
-	  }
-	  if (out_len == 0) {
-	    free(clients[i].wbuf);
-	    clients[i].wbuf = strdup("");
-	    FD_CLR(clients[i].fd, &s_fdset_write);
-	  } else {
-	    uim_helper_buffer_shift(clients[i].wbuf, message_len - out_len);
-	  }
+	  write_message(&clients[i]);
 	}
 	if (clients[i].fd != -1 && FD_ISSET(clients[i].fd, &readfds)) {
-	  int result;
-	  /* actual process */
-	  result = reflect_message_fragment(&clients[i]);
-
-	  if (result < 0) {
-	    FD_CLR(clients[i].fd, &s_fdset_read);
-	    FD_CLR(clients[i].fd, &s_fdset_write);
-	    if (clients[i].fd == s_max_fd)
-	      s_max_fd--;
-	    close(clients[i].fd);
-	    free_client(&clients[i]);
-	  }
+	  read_message(&clients[i]);
 	}
       }
     }
@@ -339,11 +364,12 @@ uim_helper_server_process_connection(int serv_fd)
   }
 }
 
+
 int
 main(int argc, char **argv)
 {
   char *path = uim_helper_get_pathname();
-  int serv_fd;
+  int server_fd;
   unlink(path);
 
   clients = NULL;
@@ -352,7 +378,7 @@ main(int argc, char **argv)
   FD_ZERO(&s_fdset_read);
   FD_ZERO(&s_fdset_write);
   s_max_fd = 0;
-  serv_fd = init_serv_fd(path);
+  server_fd = init_server_fd(path);
 
   printf("waiting\n\n");
   fflush(stdout);
@@ -360,7 +386,7 @@ main(int argc, char **argv)
   fclose(stdin);
   fclose(stdout);
 
-  if (serv_fd < 0) {
+  if (server_fd < 0) {
     return 0;
   }
   /*  fprintf(stderr,"Waiting for connection at %s\n", path);*/
@@ -369,7 +395,7 @@ main(int argc, char **argv)
 
   signal(SIGPIPE, SIG_IGN);
 
-  uim_helper_server_process_connection(serv_fd);
+  uim_helper_server_process_connection(server_fd);
 
   return 0;
 }
