@@ -46,8 +46,9 @@
 #include <pwd.h>
 #include <signal.h>
 #include <errno.h>
-#include "context.h"
+#include "uim-internal.h"
 #include "uim-helper.h"
+#include "uim-util.h"
 
 #ifndef HAVE_SIG_T
 typedef void (*sig_t)(int);
