@@ -40,7 +40,7 @@
 #include "siod.h"
 #include "uim-scm.h"
 #include "uim-compat-scm.h"
-#include "context.h"
+#include "uim-internal.h"
 
 /*
   To avoid namespace pollution, all siod functions are defined as
@@ -59,6 +59,14 @@
 
 static void siod_init_subr(const char *name, long type, SUBR_FUNC fcn);
 
+#if UIM_SCM_GCC4_READY_GC
+static int uim_scm_c_int_internal(uim_lisp integer);
+static const char *uim_scm_refer_c_str_internal(uim_lisp str);
+static uim_lisp uim_scm_eval_internal(uim_lisp obj);
+static void siod_init_subr_internal(const char *name, long type, SUBR_FUNC fcn);
+static uim_lisp uim_scm_eval_c_string_internal(const char *str);
+#endif
+
 static uim_lisp true_sym;
 static uim_lisp false_sym;
 static uim_lisp protected_arg0;
@@ -66,6 +74,12 @@ static uim_lisp protected_arg0;
 static int uim_siod_fatal;
 static FILE *uim_output = NULL;
 
+#if UIM_SCM_GCC4_READY_GC
+/* See also the comment about these variables in uim-scm.h */
+uim_lisp *(*volatile uim_scm_gc_protect_stack_ptr)(void)
+  = &uim_scm_gc_protect_stack_internal;
+#endif /* UIM_SCM_GCC4_READY_GC */
+
 
 FILE *
 uim_scm_get_output(void)
@@ -93,14 +107,33 @@ uim_scm_make_bool(uim_bool val)
 
 int
 uim_scm_c_int(uim_lisp integer)
+#if UIM_SCM_GCC4_READY_GC
+{
+  int ret;
+
+  UIM_SCM_GC_PROTECTED_CALL(ret, int, uim_scm_c_int_internal, (integer));
+
+  return ret;
+}
+
+static int
+uim_scm_c_int_internal(uim_lisp integer)
+#endif
 {
   int c_int;
+#if !UIM_SCM_GCC4_READY_GC
   uim_lisp stack_start;
 
-  uim_scm_gc_protect_stack(&stack_start);  /* required for my_err() */
+  /* stack protection is required for my_err() */
+  uim_scm_gc_protect_stack(&stack_start);
+#endif
+
   protected_arg0 = integer;
   c_int = get_c_int((LISP)integer);
+
+#if !UIM_SCM_GCC4_READY_GC
   uim_scm_gc_unprotect_stack(&stack_start);
+#endif
 
   return c_int;
 }
@@ -123,14 +156,33 @@ uim_scm_c_str(uim_lisp str)
 
 const char *
 uim_scm_refer_c_str(uim_lisp str)
+#if UIM_SCM_GCC4_READY_GC
+{
+  const char *ret;
+
+  UIM_SCM_GC_PROTECTED_CALL(ret, const char *, uim_scm_refer_c_str_internal, (str));
+
+  return ret;
+}
+
+static const char *
+uim_scm_refer_c_str_internal(uim_lisp str)
+#endif
 {
   char *c_str;
+#if !UIM_SCM_GCC4_READY_GC
   uim_lisp stack_start;
 
-  uim_scm_gc_protect_stack(&stack_start);  /* required for my_err() */
+  /* stack protection is required for my_err() */
+  uim_scm_gc_protect_stack(&stack_start);
+#endif
+
   protected_arg0 = str;
   c_str = get_c_string((LISP)str);
+
+#if !UIM_SCM_GCC4_READY_GC
   uim_scm_gc_unprotect_stack(&stack_start);
+#endif
 
   return c_str;
 }
@@ -182,13 +234,7 @@ uim_scm_make_func_ptr(uim_func_ptr func_ptr)
 void
 uim_scm_gc_protect(uim_lisp *location)
 {
-  gc_protect((LISP *)location);
-}
-
-void
-uim_scm_gc_protect_stack(uim_lisp *stack_start)
-{
-  siod_gc_protect_stack((LISP *)stack_start);
+  siod_gc_protect((LISP *)location);
 }
 
 void
@@ -197,6 +243,29 @@ uim_scm_gc_unprotect_stack(uim_lisp *stack_start)
   siod_gc_unprotect_stack((LISP *)stack_start);
 }
 
+#if UIM_SCM_GCC4_READY_GC
+uim_lisp *
+uim_scm_gc_protect_stack_internal(void)
+{
+  /*
+   * &stack_start will be relocated to start of the frame of subsequent
+   * function call
+   */
+  LISP stack_start;
+
+  siod_gc_protect_stack(&stack_start);
+
+  /* intentionally returns invalidated local address */
+  return (uim_lisp *)&stack_start;
+}
+#else /* UIM_SCM_GCC4_READY_GC */
+void
+uim_scm_gc_protect_stack(uim_lisp *stack_start)
+{
+  siod_gc_protect_stack((LISP *)stack_start);
+}
+#endif /* UIM_SCM_GCC4_READY_GC */
+
 uim_bool
 uim_scm_is_alive(void)
 {
@@ -291,19 +360,49 @@ uim_scm_string_equal(uim_lisp a, uim_lisp b)
 
 uim_lisp
 uim_scm_eval(uim_lisp obj)
+#if UIM_SCM_GCC4_READY_GC
+{
+  uim_lisp ret;
+
+  UIM_SCM_GC_PROTECTED_CALL(ret, uim_lisp, uim_scm_eval_internal, (obj));
+
+  return ret;
+}
+
+static uim_lisp
+uim_scm_eval_internal(uim_lisp obj)
+#endif
 {
   uim_lisp ret;  /* intentionally outside of next stack_start */
+#if !UIM_SCM_GCC4_READY_GC
   uim_lisp stack_start;
 
   uim_scm_gc_protect_stack(&stack_start);
+#endif
+
   ret = (uim_lisp)leval((LISP)obj, NIL);
+
+#if !UIM_SCM_GCC4_READY_GC
   uim_scm_gc_unprotect_stack(&stack_start);
+#endif
 
   return ret;
 }
 
 uim_lisp
 uim_scm_eval_c_string(const char *str)
+#if UIM_SCM_GCC4_READY_GC
+{
+  uim_lisp ret;
+
+  UIM_SCM_GC_PROTECTED_CALL(ret, uim_lisp, uim_scm_eval_c_string_internal, (str));
+
+  return ret;
+}
+
+static uim_lisp
+uim_scm_eval_c_string_internal(const char *str)
+#endif
 {
   repl_c_string((char *)str, 0, 0);
   return uim_scm_return_value();
@@ -389,6 +488,17 @@ uim_scm_require_file(const char *fn)
 
 static void
 siod_init_subr(const char *name, long type, SUBR_FUNC fcn)
+#if UIM_SCM_GCC4_READY_GC
+{
+  UIM_SCM_GC_PROTECTED_CALL_VOID(siod_init_subr_internal, (name, type, fcn));
+}
+
+static void
+siod_init_subr_internal(const char *name, long type, SUBR_FUNC fcn)
+{
+  init_subr(name, type, fcn);
+}
+#else
 {
   uim_lisp stack_start;
 
@@ -396,6 +506,7 @@ siod_init_subr(const char *name, long type, SUBR_FUNC fcn)
   init_subr(name, type, fcn);
   uim_scm_gc_unprotect_stack(&stack_start);
 }
+#endif
 
 void
 uim_scm_init_subr_0(const char *name, uim_lisp (*fcn)(void))
@@ -447,14 +558,14 @@ uim_scm_init(const char *verbose_level)
   char *siod_argv[] =
     {
       "siod",
-      "-v0",          /* siod_verbose_level */
+      "-v2",          /* siod_verbose_level */
       "-h16384:64",   /* heap_size(unit: lisp objects):nheaps */
       "-t16384",      /* heap_alloc_threshold (unit: lisp objects) */
       "-o1024",       /* obarray_dim (hash size of symbol table) */
       "-s262144",     /* stack_size (unit: bytes) */
       "-n128"         /* inums_dim (preallocated fixnum objects) */
     };
-  char verbose_argv[] = "-v4";
+  char verbose_argv[] = "-v2";
   int siod_argc, warnflag = 1;
 
   if (!uim_output) {
