@@ -37,6 +37,10 @@
 #include "uim-scm.h"
 #include "uim-compat-scm.h"
 
+#ifdef LIBEDIT
+#include "editline.h"
+#endif
+
 #ifdef UIM_SH_USE_EXIT_HOOK
 extern int uim_siod_fatal;
 #endif
@@ -51,6 +55,10 @@ main(int argc, char *argv[]) {
   /* TODO: be able to suppress ordinary initialization process */
   uim_init();
 
+#ifdef LIBEDIT
+  editline_init();
+#endif
+
   verbose = uim_scm_get_verbose_level();
   uim_scm_set_verbose_level(1);
   uim_scm_require_file("uim-sh.scm");
@@ -83,6 +91,10 @@ main(int argc, char *argv[]) {
     return 1;
 #endif
 
+#ifdef LIBEDIT
+  editline_quit();
+#endif
+
   uim_quit();
 
   return 0;
