@@ -35,23 +35,17 @@
 
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <sys/time.h>
 #include <stdlib.h>
 #include <string.h>
 #include <locale.h>
 #include <langinfo.h>
-#include "context.h"
+#include "uim-internal.h"
 #include "gettext.h"
 #include "uim-scm.h"
 #include "uim-compat-scm.h"
 #include "uim-util.h"
 
-#ifndef HAVE_SETENV
-int setenv(const char *, const char *, int);
-#endif
-#ifndef HAVE_UNSETENV
-void unsetenv(const char *);
-#endif
-
 extern char *uim_return_str;
 
 /* define constants as procedure to ensure unmodifiable */
@@ -126,6 +120,20 @@ file_directoryp(uim_lisp filename)
   return file_stat_mode(filename, S_IFDIR);
 }
 
+static uim_lisp
+file_mtime(uim_lisp f)
+{
+  const char *filename = uim_scm_refer_c_str(f);
+  struct stat buf;
+
+  if(stat(filename, &buf) == 0) {
+    return uim_scm_make_int(buf.st_mtime);
+  } else {
+    /* FIXME: Write error handling code. */
+    return uim_scm_make_int(0);
+  }
+}
+
 static uim_lisp
 charcode2string(uim_lisp x)
 {
@@ -493,7 +501,7 @@ iterate_lists(uim_lisp mapper, uim_lisp seed, uim_lisp lists)
   uim_lisp elms, rest, rests, mapped, res, termp, pair, form;
   uim_bool single_listp;
 
-  single_listp = (uim_scm_length(lists) == 1) ? UIM_TRUE : UIM_FALSE;
+  single_listp = uim_scm_nullp(uim_scm_cdr(lists));
   res = seed;
   if (single_listp) {
     rest = uim_scm_car(lists);
@@ -503,8 +511,12 @@ iterate_lists(uim_lisp mapper, uim_lisp seed, uim_lisp lists)
   do {
     if (single_listp) {
       /* fast path */
-      elms = uim_scm_list1(uim_scm_car(rest));
-      rest = uim_scm_cdr(rest);
+      if (uim_scm_nullp(rest)) {
+	elms = uim_scm_null_list();
+      } else {
+	elms = uim_scm_list1(uim_scm_car(rest));
+	rest = uim_scm_cdr(rest);
+      }
     } else {
       pair = shift_elems(rests);
       if (FALSEP(pair)) {
@@ -623,6 +635,7 @@ uim_init_util_subrs(void)
   uim_scm_init_subr_1("file-executable?", file_executablep);
   uim_scm_init_subr_1("file-regular?", file_regularp);
   uim_scm_init_subr_1("file-directory?", file_directoryp);
+  uim_scm_init_subr_1("file-mtime", file_mtime);
   uim_scm_init_subr_2("nthcdr", nthcdr);
   uim_scm_init_subr_1("charcode->string", charcode2string);
   uim_scm_init_subr_1("string->charcode", string2charcode);
