@@ -42,8 +42,9 @@
 #include "uim-im-switcher.h"
 #include "uim-scm.h"
 #include "uim-custom.h"
-#include "context.h"
+#include "uim-internal.h"
 #include "gettext.h"
+#include "uim-util.h"
 
 extern char *uim_return_str;
 extern char *uim_return_str_list[10];
@@ -58,7 +59,6 @@ static uim_context context_array[CONTEXT_ARRAY_SIZE];
 struct uim_im *uim_im_array;
 int uim_nr_im;
 static int uim_initialized;
-static int uim_quiting;
 
 void
 uim_set_preedit_cb(uim_context uc,
@@ -161,6 +161,8 @@ uim_create_context(void *ptr,
   uc->request_surrounding_text_cb = NULL;
   uc->delete_surrounding_text_cb = NULL;
   /**/
+  uc->configuration_changed_cb = NULL;
+  /**/
   uc->nr_candidates = 0;
   uc->candidate_index = 0;
   /**/
@@ -203,6 +205,13 @@ uim_reset_context(uim_context uc)
    uim_release_preedit_segments(uc);
 }
 
+void
+uim_set_configuration_changed_cb(uim_context uc,
+				 void (*changed_cb)(void *ptr))
+{
+  uc->configuration_changed_cb = changed_cb;
+}
+
 void
 uim_switch_im(uim_context uc, const char *engine)
 {
@@ -221,6 +230,7 @@ uim_switch_im(uim_context uc, const char *engine)
 
   UIM_EVAL_FSTRING1(uc, "(release-context %d)", uc->id);
   uim_release_preedit_segments(uc);
+  uim_update_preedit_segments(uc);
 
   UIM_EVAL_FSTRING2(uc, "(create-context %d #f '%s)", id, engine);
   if (uc->current_im_name)
@@ -269,7 +279,9 @@ uim_release_context(uim_context uc)
 uim_context
 uim_find_context(int id)
 {
-  return context_array[id];
+  uim_context uc;
+  uc = context_array[id];
+  return uc;
 }
 
 int
@@ -338,6 +350,23 @@ uim_prop_update_custom(uim_context uc, const char *custom, const char *val)
 		    uc->id, custom, val);
 }
 
+
+/* Tentative name. I followed above uim_prop_update_custom, but prop 
+would not be proper to this function. */
+/*
+ * As I described in doc/HELPER-PROTOCOL, it had wrongly named by my
+ * misunderstanding about what is the 'property' of uim. It should be
+ * renamed along with corresponding procol names when an appropriate
+ * time has come.  -- YamaKen 2005-09-12
+ */
+uim_bool
+uim_prop_reload_configs(void)
+{
+  /* FIXME: handle return value properly. */
+  uim_scm_eval_c_string("(custom-reload-user-configs)");
+  return UIM_TRUE;
+}
+
 int
 uim_get_current_mode(uim_context uc)
 {
@@ -596,7 +625,7 @@ uim_set_surrounding_text(uim_context uc, const char *text,
 }
 
 static void
-uim_init_scm()
+uim_init_scm(void)
 {
   int i;
   char *scm_files = NULL;
@@ -615,10 +644,6 @@ uim_init_scm()
   uim_init_plugin();
   uim_init_im_subrs();
   uim_init_key_subrs();
-#ifdef UIM_COMPAT_TABLE
-  uim_init_table_subrs();
-#endif
-
   
   if (is_setugid() == 0) {
     scm_files = getenv("LIBUIM_SCM_FILES");
@@ -655,14 +680,15 @@ uim_init_scm()
 int
 uim_init(void)
 {
-  if (uim_initialized) {
+  if (uim_initialized)
     return 0;
-  }
+
   uim_last_client_encoding = NULL;
   uim_im_array = NULL;
   uim_nr_im = 0;
   uim_init_scm();
   uim_initialized = 1;
+
   return 0;
 }
 
@@ -671,11 +697,8 @@ uim_quit(void)
 {
   int i;
 
-  if (!uim_initialized || uim_quiting) {
+  if (!uim_initialized)
     return;
-  }
-  /* Some multithreaded applications calls uim_quit bursty. */
-  uim_quiting = 1;
 
   /* release still active contexts */
   for (i = 0; i < CONTEXT_ARRAY_SIZE; i++) {
@@ -689,5 +712,4 @@ uim_quit(void)
   free(uim_last_client_encoding);
   uim_last_client_encoding = NULL;
   uim_initialized = 0;
-  uim_quiting = 0;
 }
