@@ -46,7 +46,9 @@ public:
     void show();
     void hide();
     void move(int x, int y);
-    void show_caret_state(const char *str);
+    void show_caret_state(const char *str, int timeout);
+    void update_caret_state();
+    void hide_caret_state();
 private:
     void check_connection();
 };
