@@ -97,27 +97,15 @@ init_default_xftfont() {
     }
 }
 
-static char *
-dequote(const char *str)
-{
-   char *ret = NULL;
-
-   if (str) {
-	int len = strlen(str);
-	if (str[0] == '"' && str[len - 1] == '"') {
-	    ret = strdup(++str);
-	    ret[len - 2] = '\0';
-	} else
-	    ret = strdup(str);
-   }
-
-   return ret;
-}
-
 void
-update_default_xftfont(const char *s) {
-    char *fontname = dequote(s);
+update_default_xftfont() {
+    char *fontname;
+    
+    if (!uim_scm_symbol_value_bool("uim-xim-use-xft-font?"))
+      return;
 
+    fontname = uim_scm_symbol_value_str("uim-xim-xft-font-name");
+    
     if (fontname) {
 	XftFont *xftfont = XftFontOpen(XimServer::gDpy,
 			DefaultScreen(XimServer::gDpy),
@@ -125,12 +113,15 @@ update_default_xftfont(const char *s) {
 			XFT_PIXEL_SIZE, XftTypeDouble, (double)DEFAULT_FONT_SIZE,
 			NULL);
 	if (xftfont) {
-	    XftFontClose(XimServer::gDpy, gXftFont);
+	    if (gXftFont)
+		XftFontClose(XimServer::gDpy, gXftFont);
 	    free(gXftFontName);
 	    free(gXftFontLocale);
 	    gXftFont = xftfont;
 	    gXftFontName = fontname;
 	    gXftFontLocale = strdup(setlocale(LC_CTYPE, NULL));
+	} else {
+	    free(fontname);
 	}
     }
 }
@@ -225,7 +216,6 @@ protected:
     XftDraw *mXftDraw;
     XftColor mXftColorFg;
     XftColor mXftColorFgRev;
-    int mGlyphWidth;
 #endif
     XFontSet mFontset;
     const char *mEncoding;
@@ -241,13 +231,18 @@ public:
     virtual ~PeLineWin();
 
     void draw_pe(pe_stat *p);
-
-    void draw_segment(pe_ustring *s);
+    int mCandWinXOff;
 
 private:
     void calc_extent(pe_stat *p);
+    int calc_segment_extent(pe_ustring *s);
+    void draw_segment(pe_ustring *s);
+    void draw_cursor();
+    int get_char_width(uchar ch);
 
     int m_x;
+    int mCharPos;
+    int mCursorX;
 };
 
 // window for over the spot style
@@ -260,6 +255,7 @@ public:
     virtual ~PeOvWin();
 private:
     void draw_a_ce(char_ent *ce);
+    void draw_cursor(char_ent *ce);
     Pixmap m_mask_pix;
     GC m_mask_pix_gc;
 };
@@ -272,7 +268,6 @@ public:
     virtual void clear_preedit();
     virtual void update_icxatr();
     virtual void move_candwin();
-    virtual void set_im_lang(const char *im_lang);
     virtual bool use_xft();
 private:
     bool check_win();
@@ -281,7 +276,6 @@ private:
     void make_ce_array();
     void draw_preedit();
     void do_draw_preedit();
-    int calc_ce_width(int b, int e);
 #ifdef FLASHPLAYER_WORKAROUND
     int get_ce_font_height(char_ent *ce, int len);
     int revised_spot_y;
@@ -291,6 +285,8 @@ private:
 
     char_ent *m_ce;
     int m_ce_len;
+    int m_candwin_x_off;
+    int m_candwin_y_off;
     PeOvWin *m_ov_win;
 };
 
@@ -381,6 +377,8 @@ PeWin::PeWin(Window pw, const char *im_lang, const char *encoding, const char *l
     if (mConvdisp->use_xft() == true) {
 #if HAVE_XFT_UTF8_STRING
 	mXftFontSize = DEFAULT_FONT_SIZE;
+	if (!gXftFont)
+	    init_default_xftfont();
 	if (!strcmp(gXftFontLocale, locale)) {
 	    mXftFont = gXftFont;
 	} else {
@@ -472,7 +470,6 @@ void PeWin::draw_char(int x, int y, uchar ch, int stat)
 #ifdef HAVE_XFT_UTF8_STRING
 	XGlyphInfo ginfo;
 	XftTextExtentsUtf8(XimServer::gDpy, mXftFont, (unsigned char *)utf8, len, &ginfo);
-	mGlyphWidth = ginfo.xOff;
 	if (stat & PE_REVERSE) {
 	    XftDrawRect(mXftDraw, &mXftColorFg, x, y - (mXftFontSize - 2), ginfo.xOff, mXftFontSize);
 	    XftDrawStringUtf8(mXftDraw, &mXftColorFgRev, mXftFont, x, y, (unsigned char *)utf8, len);
@@ -548,6 +545,9 @@ void PeWin::set_xftfont(const char *xfld)
 {
 	int size = get_fontsize(xfld);
 	const char *locale = mConvdisp->get_locale_name();
+
+	if (!gXftFont)
+	    init_default_xftfont();
 	if (size != -1 && (mXftFontSize != size || strcmp(locale, gXftFontLocale))) {
 	    if (mXftFont != gXftFont)
 		XftFontClose(XimServer::gDpy, mXftFont);
@@ -653,9 +653,14 @@ void PeWin::unmap()
 //
 // PeLineWin
 //
+#define PE_LINE_WIN_WIDTH	400
+#define PE_LINE_WIN_HEIGHT	28
+#define PE_LINE_WIN_FONT_POS_Y	20
+#define PE_LINE_WIN_MARGIN_X	2
+
 PeLineWin::PeLineWin(Window w, const char *im_lang, const char *encoding, const char *locale, Convdisp *cd) : PeWin(w, im_lang, encoding, locale, cd)
 {
-    set_size(400, 28); // set window height wider than its font height 16
+    set_size(PE_LINE_WIN_WIDTH, PE_LINE_WIN_HEIGHT);
     clear();
 }
 
@@ -667,43 +672,121 @@ void PeLineWin::draw_pe(pe_stat *p)
 {
     clear();
     calc_extent(p);
-    m_x = 0;
+    m_x = PE_LINE_WIN_MARGIN_X;
+    mCursorX = m_x;
+    mCharPos = 0;
     std::list<pe_ustring>::iterator i;
     for (i = p->ustrings.begin(); i != p->ustrings.end(); i++) {
 	draw_segment(&(*i));
     }
+    draw_cursor();
+}
+
+void PeLineWin::draw_cursor()
+{
+    XDrawLine(XimServer::gDpy, mPixmap, mGC,
+		    mCursorX,
+		    (PE_LINE_WIN_HEIGHT - PE_LINE_WIN_FONT_POS_Y) / 2 + 1,
+		    mCursorX,
+		    PE_LINE_WIN_FONT_POS_Y + 1);
+}
+
+int PeLineWin::get_char_width(uchar ch)
+{
+    int width = 0;
+    char utf8[6];
+
+    int len = utf8_wctomb((unsigned char *)utf8, ch);
+    utf8[len] = '\0';
+
+    if (mConvdisp->use_xft() == true) {
+#ifdef HAVE_XFT_UTF8_STRING
+	XGlyphInfo ginfo;
+	XftTextExtentsUtf8(XimServer::gDpy, mXftFont, (unsigned char *)utf8,
+			len, &ginfo);
+	width = ginfo.xOff;
+#endif
+    } else {
+	XRectangle ink, logical;
+
+	if (!strcmp(mEncoding, "UTF-8")) {
+	    XwcTextExtents(mFontset, &ch, 1, &ink, &logical);
+	} else {
+	    char *native_str;
+	    XimIM *im = get_im_by_id(mConvdisp->get_context()->get_ic()->get_imid());
+
+	    native_str = im->utf8_to_native_str(utf8);
+	    if (!native_str)
+		return 0;
+	    len = strlen(native_str);
+	    XmbTextExtents(mFontset, native_str, len, &ink, &logical);
+	    free(native_str);
+	}
+	width = logical.width;
+    }
+
+    return width;
 }
 
 void PeLineWin::draw_segment(pe_ustring *s)
 {
     uString::iterator i;
+    int caret_pos = mConvdisp->get_caret_pos();
+
     for (i = s->s.begin(); i != s->s.end(); i++) {
 	uchar ch = *i;
-	draw_char(m_x, 20, ch, s->stat);
-
-	int width;
-
-#if HAVE_XFT_UTF8_STRING
-	if (mConvdisp->use_xft() == true)
-	    width = mGlyphWidth;
-	else
-#endif
-	    width = 16;
+	int width = get_char_width(ch);
+	draw_char(m_x, PE_LINE_WIN_FONT_POS_Y, ch, s->stat);
+	mCharPos++;
 
 	if (s->stat & PE_UNDERLINE) {
 	    XDrawLine(XimServer::gDpy, mPixmap, mGC,
-			    m_x, 20 + UNDERLINE_HEIGHT,
-			    m_x + width, 20 + UNDERLINE_HEIGHT);
+			    m_x, PE_LINE_WIN_FONT_POS_Y + UNDERLINE_HEIGHT,
+			    m_x + width, PE_LINE_WIN_FONT_POS_Y + UNDERLINE_HEIGHT);
 	}
 	m_x += width;
+	if (mCharPos == caret_pos)
+	    mCursorX= m_x;
     }
+
+    switch (XimServer::gCandWinPosType) {
+    case Caret:
+	mCandWinXOff = mCursorX;
+	break;
+    case Right:
+	mCandWinXOff = m_x;
+	break;
+    case Left:
+    default:
+	mCandWinXOff = 0;
+	break;
+    }
+}
+
+int PeLineWin::calc_segment_extent(pe_ustring *s)
+{
+    int width = 0;
+    uString::iterator i;
+
+    for (i = s->s.begin(); i != s->s.end(); i++) {
+	uchar ch = *i;
+	width += get_char_width(ch);
+    }
+    return width;
 }
 
 void PeLineWin::calc_extent(pe_stat *p)
 {
-    int c;
-    c = p->get_char_count();
-    set_size(400, 28); // XXX need to extent
+    int width = 0;
+    std::list<pe_ustring>::iterator i;
+
+    for (i = p->ustrings.begin(); i != p->ustrings.end(); i++)
+	width += calc_segment_extent(&(*i));	
+
+    if (width < PE_LINE_WIN_WIDTH)
+	set_size(PE_LINE_WIN_WIDTH, PE_LINE_WIN_HEIGHT);
+    else
+	set_size(width + PE_LINE_WIN_MARGIN_X * 2, PE_LINE_WIN_HEIGHT);
 }
 
 
@@ -758,17 +841,20 @@ void PeOvWin::draw_ce(char_ent *ce, int len)
     for (i = 0; i < len; i++) {
 	draw_a_ce(&ce[i]);
     }
+    draw_cursor(ce);
     XShapeCombineMask(XimServer::gDpy, mWin, ShapeBounding,
 		      0, 0, m_mask_pix, ShapeSet);
     do_map();
 }
 
+#define CURSOR_WIDTH	1
 void PeOvWin::draw_a_ce(char_ent *ce)
 {
     draw_char(ce->x, ce->y, ce->c, ce->stat);
+
     XFillRectangle(XimServer::gDpy, m_mask_pix, m_mask_pix_gc,
 		   ce->x, ce->y - ce->height + 2,
-		   ce->width, ce->height + UNDERLINE_HEIGHT - 1);
+		   ce->width + CURSOR_WIDTH, ce->height + UNDERLINE_HEIGHT - 1);
     if (ce->stat & PE_UNDERLINE) {
 	XDrawLine(XimServer::gDpy, mPixmap, mGC,
 		  ce->x, ce->y + UNDERLINE_HEIGHT,
@@ -776,6 +862,25 @@ void PeOvWin::draw_a_ce(char_ent *ce)
     }
 }
 
+void PeOvWin::draw_cursor(char_ent *ce)
+{
+    int x;
+    int caret_pos = mConvdisp->get_caret_pos();
+    char_ent *caret_ce;
+
+    if (caret_pos == 0) {
+	caret_ce = &ce[caret_pos];
+	x = caret_ce->x;
+    } else {
+	caret_ce = &ce[caret_pos - 1];
+	x = caret_ce->x + caret_ce->width;
+    }
+
+    XDrawLine(XimServer::gDpy, mPixmap, mGC,
+		    x, caret_ce->y - caret_ce->height,
+		    x, caret_ce->y);
+}
+
 //
 //
 //
@@ -839,6 +944,29 @@ InputContext *Convdisp::get_context()
     return mKkContext;
 }
 
+int Convdisp::get_caret_pos()
+{
+    if (!m_pe)
+	return 0;
+    return m_pe->caret_pos;
+}
+
+void Convdisp::update_caret_state()
+{
+    if (!uim_scm_symbol_value_bool("bridge-show-input-state?"))
+	return;
+
+    Canddisp *disp = canddisp_singleton();
+    InputContext *focusedContext = InputContext::focusedContext();
+
+    if (focusedContext && focusedContext == mKkContext) {
+	if (mKkContext->isCaretStateShown())
+	    disp->update_caret_state();
+	else
+	    disp->hide_caret_state();
+    }
+}
+
 // Root window style
 ConvdispRw::ConvdispRw(InputContext *k, icxatr *a) : Convdisp(k, a)
 {
@@ -858,6 +986,8 @@ void ConvdispRw::update_preedit()
 
     if (!m_pe->get_char_count()) {
 	clear_preedit();
+	move_candwin(); // reset candwin position
+	update_caret_state();
 	return;
     }
 
@@ -880,6 +1010,7 @@ void ConvdispRw::update_preedit()
     mPeWin->draw();
 
     move_candwin();
+    update_caret_state();
 }
 
 void ConvdispRw::clear_preedit()
@@ -894,6 +1025,10 @@ void ConvdispRw::update_icxatr()
 
 void ConvdispRw::move_candwin()
 {
+    InputContext *focusedContext = InputContext::focusedContext();
+    if (!focusedContext || focusedContext != mKkContext)
+	return;
+
     if (m_atr->has_atr(ICA_ClientWindow)) {
 	int x, y;
 	Window win;
@@ -906,6 +1041,9 @@ void ConvdispRw::move_candwin()
 	Canddisp *disp = canddisp_singleton();
 
 	XGetWindowAttributes(XimServer::gDpy, m_atr->client_window, &xattr);
+
+	if (mPeWin)
+	    x += mPeWin->mCandWinXOff;
 	disp->move(x, y + xattr.height + 28); // lower-left side under the preedit window
     }
 }
@@ -919,6 +1057,8 @@ bool ConvdispRw::use_xft()
 ConvdispOv::ConvdispOv(InputContext *k, icxatr *a) : Convdisp(k, a)
 {
     m_ov_win = 0;
+    m_candwin_x_off = 0;
+    m_candwin_y_off = 0;
 #ifdef FLASHPLAYER_WORKAROUND
     revised_spot_y = -1;
 #endif
@@ -930,20 +1070,19 @@ ConvdispOv::~ConvdispOv()
 	delete m_ov_win;
 }
 
-void ConvdispOv::set_im_lang(const char *im_lang)
-{
-    mIMLang = im_lang;
-}
-
-
 void ConvdispOv::update_preedit()
 {
     draw_preedit();
     move_candwin();
+    update_caret_state();
 }
 
 void ConvdispOv::move_candwin()
 {
+    InputContext *focusedContext = InputContext::focusedContext();
+    if (!focusedContext || focusedContext != mKkContext)
+	return;
+
     if (m_atr->has_atr(ICA_SpotLocation) ) {
 	int x = -1, y = -1;
 	Window win;
@@ -975,8 +1114,12 @@ void ConvdispOv::move_candwin()
 	}
 #endif
 	if (x > -1 && y > -1) {
+	    x += m_candwin_x_off;
+	    y += m_candwin_y_off;
+
 	    Canddisp *disp = canddisp_singleton();
 	    disp->move(x, y + UNDERLINE_HEIGHT + 1);
+	    m_atr->unset_change_mask(ICA_SpotLocation);
 	}
 #if 0
     } else if (m_atr->has_atr(ICA_SpotLocation)) {
@@ -999,6 +1142,8 @@ void ConvdispOv::clear_preedit()
 {
     delete m_ov_win;
     m_ov_win = NULL;
+    m_candwin_x_off = 0;
+    m_candwin_y_off = 0;
 }
 
 void ConvdispOv::validate_area()
@@ -1020,6 +1165,12 @@ void ConvdispOv::validate_area()
 
 void ConvdispOv::update_icxatr()
 {
+
+    if (m_atr->is_changed(ICA_SpotLocation)) {
+	move_candwin();
+	update_caret_state();
+    }
+
     if (!m_ov_win)
 	return;
     
@@ -1055,7 +1206,10 @@ void ConvdispOv::update_icxatr()
 	m_atr->unset_change_mask(ICA_FontSet);
     }
   
-    update_preedit();
+    if (m_atr->is_changed(ICA_SpotLocation))
+	move_candwin();
+
+    draw_preedit();
 }
 
 void ConvdispOv::draw_preedit()
@@ -1072,7 +1226,7 @@ void ConvdispOv::draw_preedit()
     if (!check_win())
 	return;
 
-    m_ce = (char_ent *)malloc(sizeof(char_ent)*m_ce_len);
+    m_ce = (char_ent *)malloc(sizeof(char_ent) * m_ce_len);
     make_ce_array();
     layoutCharEnt();
     do_draw_preedit();
@@ -1221,7 +1375,7 @@ bool ConvdispOv::check_atr()
 	    m_atr->font_set = choose_default_fontset(mIMLang, mLocaleName);
     }
     if (!m_atr->has_atr(ICA_LineSpace)) {
-	m_atr->line_space = 16;
+	m_atr->line_space = DEFAULT_FONT_SIZE;
     }
 
     if (!m_atr->has_atr(ICA_Foreground))
@@ -1256,6 +1410,8 @@ void ConvdispOv::layoutCharEnt()
 {
     int i;
     int x, y;
+    int caret_pos = get_caret_pos();
+    int right_limit = m_atr->area.width + m_atr->area.x;
 
     x = m_atr->spot_location.x;
     y = m_atr->spot_location.y;
@@ -1300,40 +1456,39 @@ void ConvdispOv::layoutCharEnt()
 	    }
 	}
 
-	int right_limit = m_atr->area.width;
-	if (m_atr->has_atr(ICA_Area))
-	    right_limit += m_atr->area.x;
-
-	if (m_ce[i].width + x >
-	    right_limit) {
+	if (m_ce[i].width + x > right_limit) {
 	    // goto next line
 	    x = m_atr->area.x;
-	    y += m_atr->line_space;
+	    y += (m_atr->line_space + UNDERLINE_HEIGHT);
 	}
-	m_ce[i].x = x;
-	m_ce[i].y = y;
+	m_ce[i].x = x - m_atr->area.x;
+	m_ce[i].y = y - m_atr->area.y;
+#ifdef FLASHPLAYER_WORKAROUND
+	// workaround for brain damaged flash player plugin
+	if (m_ce[i].y == 0)
+	    m_ce[i].y += m_ce[i].height;
+#endif
 	x += m_ce[i].width;
     }
-    if (m_atr->has_atr(ICA_Area)) {
-	for (i = 0; i < m_ce_len; i++) {
-	    m_ce[i].x -= m_atr->area.x;
-	    m_ce[i].y -= m_atr->area.y;
-#ifdef FLASHPLAYER_WORKAROUND
-	    // workaround for brain damaged flash player plugin
-	    if (m_ce[i].y == 0)
-		m_ce[i].y += m_ce[i].height;
-#endif
-	}
-    }
-}
 
-int ConvdispOv::calc_ce_width(int b, int e)
-{
-    int i, w = 0;
-    for (i = b; i < e; i++) {
-	w += m_ce[i].width;
+    switch (XimServer::gCandWinPosType) {
+    case Caret:
+	if (caret_pos == 0) {
+	    m_candwin_x_off = m_ce[caret_pos].x - m_atr->spot_location.x;
+	    m_candwin_y_off = m_ce[caret_pos].y - m_atr->spot_location.y;
+	} else {
+	    m_candwin_x_off = m_ce[caret_pos - 1].x + m_ce[caret_pos - 1].width - m_atr->spot_location.x;
+	    m_candwin_y_off = m_ce[caret_pos - 1].y - m_atr->spot_location.y;
+	}
+	break;
+    case Right:
+	m_candwin_x_off = m_ce[m_ce_len - 1].x + m_ce[m_ce_len - 1].width - m_atr->spot_location.x;
+	m_candwin_y_off = m_ce[m_ce_len - 1].y - m_atr->spot_location.y;
+	break;
+    case Left:
+    default:
+	break;
     }
-    return w;
 }
 
 #ifdef FLASHPLAYER_WORKAROUND
@@ -1424,9 +1579,13 @@ void ConvdispOs::update_preedit()
 	mConn->push_passive_packet(t);
     }
 }
-      
+
 void ConvdispOs::move_candwin()
 {
+    InputContext *focusedContext = InputContext::focusedContext();
+    if (!focusedContext || focusedContext != mKkContext)
+	return;
+
     if (m_atr->has_atr(ICA_ClientWindow)) {
 	int x, y;
 	Window win;
