@@ -32,13 +32,12 @@
 
 // uim-helper connection
 
+#define _GNU_SOURCE // for asprintf on stdio.h with old glibc/gcc
+
 #ifdef HAVE_CONFIG_H
 # include "config.h"
 #endif
 
-#ifndef _GNU_SOURCE
-#define _GNU_SOURCE
-#endif
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
@@ -187,7 +186,27 @@ helper_str_parse(char *str)
 		*eol = '\0';
 	    else
 		return;
-	    focusedContext->extra_input(line);
+	    if (!strncmp(line, "charset=", 8)) {
+		const char *charset = line + 8;
+		line = eol + 1;
+		eol = strchr(line, '\n');
+		if (eol != NULL)
+		    *eol = '\0';
+		else
+		    return;
+		if (!strcmp(charset, "UTF-8"))
+		    focusedContext->extra_input(line);
+		else {
+		    int len = strlen(line);
+		    char *utf8_str = (char *)malloc(len * 6 + 1);
+		    if (!utf8_str)
+			return;
+		    mb_string_to_utf8(utf8_str, line, len * 6, charset);
+		    focusedContext->extra_input(utf8_str);
+		    free(utf8_str);
+		}
+	    } else
+		focusedContext->extra_input(line);
 	    return;
 	}
     }
@@ -224,6 +243,12 @@ helper_str_parse(char *str)
 	    (*it).second->customContext(custom, val);
 	}
 	return;
+    } else if (strcmp("custom_reload_notify", line) == 0) {
+	std::map<Window, XimServer *>::iterator it;
+	for (it = XimServer::gServerMap.begin(); it != XimServer::gServerMap.end(); it++) {
+	    (*it).second->reloadConfigs();
+	}
+	return;
     }
 }
 
