@@ -32,13 +32,12 @@
 
 // Locale dependent routines
 
+#define _GNU_SOURCE // for asprintf on stdio.h with old glibc/gcc
+
 #ifdef HAVE_CONFIG_H
 # include "config.h"
 #endif
 
-#ifndef _GNU_SOURCE
-#define _GNU_SOURCE
-#endif
 #include <stdio.h>
 #include <locale.h>
 #include <iconv.h>
@@ -85,7 +84,7 @@ static struct {
     {"cz", "cz_CZ", "ISO8859-2"},
     {"da", "da_DK", "ISO8859-1:ISO8859-15:UTF-8"},
     {"de", "de_DE:de_DE@euro:de_AT:de_AT@euro:de_BE:de_BE@euro:de_CH:de_LI:de_LU:de_LU@euro", "ISO8859-1:ISO8859-15:UTF-8"},
-    {"el", "el_GR", "ISO8859-7:ISO8859-15:UTF-8"},
+    {"el", "el_GR:el_GR@euro", "ISO8859-7:ISO8859-15:UTF-8"},
     {"en", "en_US:en_AU:en_BE:en_BZ:en_BW:en_CA:en_GB:en_HK:en_IE:en_IE@euro:en_IN:en_JM:en_NZ:en_PH:en_SG:en_TT:en_UK:en_ZA", "ISO8859-1:ISO8859-15:UTF-8"},
     {"eo", "eo_XX:eo_EO", "ISO8859-3"},
     {"es", "es_ES:es_ES@euro:es_AR:es_BO:es_CL:es_CO:es_CR:es_DO:es_EC:es_GT:es_HN:es_MX:es_NI:es_PA:es_PE:es_PR:es_PY:es_SV:es_US:es_UY:es_VE", "ISO8859-1:ISO8859-15:UTF-8"},
@@ -142,8 +141,8 @@ static struct {
     {"sl", "sl_SI", "ISO8859-2:UTF-8"},
     {"sp", "sp_YU", "ISO8859-5"},
     {"sq", "sq_AL", "ISO8859-2:UTF-8"},
-    {"sr", "sr_YU:sr_SP", "ISO8859-2:ISO8859-5:CP1251:UTF-8"},
-    {"sv", "sv_SE:sv_FI:sv_FI@euro", "ISO8859-1:ISO8859-15:UTF-8"},
+    {"sr", "sr_YU:sr_YU@cyrillic:sr_SP", "ISO8859-2:ISO8859-5:CP1251:UTF-8"},
+    {"sv", "sv_SE:sv_SE@euro:sv_FI:sv_FI@euro", "ISO8859-1:ISO8859-15:UTF-8"},
     {"ta", "ta_IN", "TSCII-0:UTF-8"},
     {"te", "te_IN", "UTF-8"},
     {"tg", "tg_TJ", "KOI8-C:KOI8-T:UTF-8"},
