@@ -32,14 +32,13 @@
 
 // XIM Server supporting CJK languages
 // initialize many modules
-//
+
+#define _GNU_SOURCE // for asprintf on stdio.h with old glibc/gcc
+
 #ifdef HAVE_CONFIG_H
 # include "config.h"
 #endif
 
-#ifndef _GNU_SOURCE
-#define _GNU_SOURCE
-#endif
 #include <stdio.h>
 #include <string.h>
 #include <locale.h>
@@ -307,6 +306,10 @@ ProcXEvent(XEvent *e)
     case ClientMessage:
 	procXClientMessage(&e->xclient);
 	break;
+    case MappingNotify:
+	XRefreshKeyboardMapping((XMappingEvent *)e);
+	init_modifier_keys();
+	break;
     default:;
 	//printf("unknown type of X event. %d\n", e->type);
     }
@@ -513,7 +516,7 @@ terminate_x_connection()
     remove_current_fd_watch(fd);
 }
 
-static void
+void
 reload_uim(int x)
 {
     fprintf(stderr, "\nReloading uim...\n\n");
@@ -534,7 +537,7 @@ reload_uim(int x)
     uim_quit();
     clear_uim_info();
     get_uim_info();
-    print_uim_info();
+    //print_uim_info();
 
     for (it = XimServer::gServerMap.begin(); it != XimServer::gServerMap.end(); it++) {
 	XimServer *xs = it->second;
