@@ -46,7 +46,7 @@ int pad4(int);
 void hex_dump(unsigned char *buf, int len);
 
 // misc replacement functions
-#ifndef HAV_ASPRINTF
+#ifndef HAVE_ASPRINTF
 int asprintf(char **ptr, const char *format, ...);
 #endif
 
@@ -55,14 +55,4 @@ int asprintf(char **ptr, const char *format, ...);
 int vasprintf(char **ptr, const char *fmt, va_list ap);
 #endif
 
-#ifndef HAVE_STRSEP
-#ifdef __cplusplus
-extern "C" {
-#endif
-char *uim_strsep(char **stringp, const char *delim);
-#define strsep	uim_strsep
-#ifdef __cplusplus
-}
-#endif
-#endif
 #endif
