@@ -184,6 +184,7 @@ public:
     virtual void unset_ic_focus(int icid) = 0;
     virtual void forward_event(RxPacket *) = 0;
     virtual void send_sync_reply(int icid) = 0;
+    virtual void send_sync(int icid) = 0;
     virtual XimIC *get_ic_by_id(int icid) = 0;
     virtual void onSendPacket() = 0;
     virtual void changeContext(const char *engine) = 0;
@@ -290,7 +291,6 @@ public:
     void changeContext(const char *engine);
     const char *get_encoding();
     const char *get_lang_region();
-    void move_candwin();
 
 public:
     static XimIC *get_current_ic();
@@ -304,9 +304,10 @@ private:
     void send_key_event(XKeyEvent *k);
     int lookup_style(unsigned long);
     void set_ic_attr(int, C8 *, int );
+    void send_sync();
     
     Connection *mConn;
-    // mConvdisp is 0 until getting enough icxatr.  Need to delete
+    // mConvdisp is NULL until getting enough icxatr.  Need to delete
     // this after deletion of m_kkContext since it is also refered by
     // m_kkContext.
     Convdisp *mConvdisp;
@@ -326,8 +327,6 @@ struct input_style {
 };
 
 XimIC *create_ic(Connection *, RxPacket *, int imid, int id, const char *engine);
-void force_event(Window w);
-
 void procXClientMessage(XClientMessageEvent *m);
 
 #endif
