@@ -30,19 +30,17 @@
   SUCH DAMAGE.
 */
 
+#define _GNU_SOURCE // for asprintf on stdio.h with old glibc/gcc
+
 #ifdef HAVE_CONFIG_H
 # include "config.h"
 #endif
 
-#ifndef _GNU_SOURCE
-#define _GNU_SOURCE 
-#endif
 #include <stdio.h>
 #include <ctype.h>
 #include <locale.h>
 #include <X11/Xlib.h>
 #include <X11/Xatom.h>
-#include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/keysymdef.h>
 
@@ -54,6 +52,7 @@
 #include "helper.h"
 
 #include "uim/uim-helper.h"
+#include "uim/uim-im-switcher.h"
 #include "uim/uim-compat-scm.h"
 
 #ifndef __GNUC__
@@ -164,6 +163,32 @@ void XimServer::customContext(const char *custom, const char *val) {
 	check_candwin_pos_type();
 }
 
+void XimServer::reloadConfigs() {
+#if 0
+    // uim_prop_reload_configs() seems broken... --ekato Aug 19 2005
+    uim_prop_reload_configs();
+#else
+    reload_uim(0);
+#endif
+
+    // Updated global IM of XimServer
+    char *im = uim_scm_symbol_value_str("default-im-name");
+    if (im)
+	set_im(im);
+    free(im);
+
+#if HAVE_XFT_UTF8_STRING
+    update_default_xftfont();
+#endif
+
+    if (!uim_scm_symbol_value_bool("bridge-show-input-state?")) {
+	Canddisp *disp = canddisp_singleton();
+	disp->hide_caret_state();
+    }
+
+    check_candwin_pos_type();
+}
+
 bool
 XimServer::setupConnection(bool useDefaultIM)
 {
@@ -369,23 +394,26 @@ InputContext::createUimContext(const char *engine)
 					NULL, real_im, uim_iconv,
 					InputContext::commit_cb);
 
-    uim_set_preedit_cb(uc,
-		       InputContext::clear_cb,
-		       InputContext::pushback_cb,
-		       InputContext::update_cb);
-    uim_set_candidate_selector_cb(uc,
-				  InputContext::candidate_activate_cb,
-				  InputContext::candidate_select_cb,
-				  InputContext::candidate_shift_page_cb,
-				  InputContext::candidate_deactivate_cb);
-    uim_set_prop_list_update_cb(uc,
-				InputContext::update_prop_list_cb);
-    uim_set_prop_label_update_cb(uc,
-				 InputContext::update_prop_label_cb);
-
-    if (mFocusedContext == this)
-	uim_prop_list_update(uc);
+    if (uc) {
+	uim_set_preedit_cb(uc,
+			InputContext::clear_cb,
+			InputContext::pushback_cb,
+			InputContext::update_cb);
+	uim_set_candidate_selector_cb(uc,
+			InputContext::candidate_activate_cb,
+			InputContext::candidate_select_cb,
+			InputContext::candidate_shift_page_cb,
+			InputContext::candidate_deactivate_cb);
+	uim_set_prop_list_update_cb(uc,
+			InputContext::update_prop_list_cb);
+	uim_set_prop_label_update_cb(uc,
+			InputContext::update_prop_label_cb);
+	uim_set_configuration_changed_cb(uc,
+			InputContext::configuration_changed_cb);
 
+	if (mFocusedContext == this)
+	    uim_prop_list_update(uc);
+    }
     mUc = uc;
 }
 
@@ -413,6 +441,45 @@ InputContext::changeContext(const char *engine)
     }
 }
 
+void InputContext::configuration_changed()
+{
+    const char *engine = uim_get_current_im_name(mUc);
+
+    review_im(engine);
+}
+
+void InputContext::review_im(const char *engine)
+{
+    char *locale;
+    const char *client_locale, *engine_locales;
+    const char *encoding;
+
+    encoding = mXic->get_encoding();
+    client_locale = mXic->get_lang_region();
+    engine_locales = compose_localenames_from_im_lang(get_im_lang_from_engine(engine));
+
+    if (!strcmp(encoding, "UTF-8")) {
+	if (is_locale_included(engine_locales, client_locale))
+	    locale = strdup(client_locale);
+	else
+	    locale = get_prefered_locale(engine_locales);
+	locale = (char *)realloc(locale, strlen(locale) + strlen(".UTF-8") + 1);
+	strcat(locale, ".UTF-8");
+	setlocale(LC_CTYPE, locale);
+	free(mLocaleName);
+	mLocaleName = locale;
+	free(mEngineName);
+	mEngineName = strdup(engine);
+    } else {
+	if (!is_locale_included(engine_locales, client_locale))
+	    changeContext(mEngineName);
+	else {
+	    free(mEngineName);
+	    mEngineName = strdup(engine);
+	}
+    }
+}
+
 void
 InputContext::customContext(const char *custom, const char *val)
 {
@@ -544,6 +611,13 @@ void InputContext::update_prop_label_cb(void *ptr, const char *str)
       ic->update_prop_label(str);
 }
 
+void InputContext::configuration_changed_cb(void *ptr)
+{
+    InputContext *ic = (InputContext *)ptr;
+
+    ic->configuration_changed();
+}
+
 void InputContext::clear_pe_stat()
 {
     m_pe->clear();
@@ -1054,6 +1128,7 @@ void init_modifier_keys() {
 		      Mod1MaskSyms, Mod2MaskSyms, Mod3MaskSyms,
 		      Mod4MaskSyms, Mod5MaskSyms;
 
+    gXNumLockMask = 0;
     XModifierKeymap *map = XGetModifierMapping(XimServer::gDpy);
     XDisplayKeycodes(XimServer::gDpy, &min_keycode, &max_keycode);
     KeySym *sym = XGetKeyboardMapping(XimServer::gDpy, min_keycode,
