@@ -89,6 +89,7 @@ void init_modifier_keys();
 void init_default_xftfont();
 void update_default_xftfont();
 #endif
+void reload_uim(int x);
 void check_candwin_pos_type();
 
 
@@ -180,22 +181,25 @@ public:
     void changeContext(const char *engine);
     void customContext(const char *custom, const char *val);
     void createUimContext(const char *engine);
+    void configuration_changed();
 public:
-    static void commit_cb(void *, const char *);
-    static void clear_cb(void *);
-    static void pushback_cb(void *, int attr, const char *str);
-    static void update_cb(void *);
+    static void commit_cb(void *ptr, const char *str);
+    static void clear_cb(void *ptr);
+    static void pushback_cb(void *ptr, int attr, const char *str);
+    static void update_cb(void *ptr);
     static void candidate_activate_cb(void *ptr, int nr, int index);
     static void candidate_select_cb(void *ptr, int index);
     static void candidate_shift_page_cb(void *ptr, int direction);
     static void candidate_deactivate_cb(void *ptr);
     static void update_prop_list_cb(void *ptr, const char *str);
     static void update_prop_label_cb(void *ptr, const char *str);
+    static void configuration_changed_cb(void *ptr);
     static InputContext *focusedContext();
     static void deletefocusedContext();
 private:
     void commit_string(char *);
     void clear_pe_stat();
+    void review_im(const char *engine);
 
     XimIC *mXic;
     XimServer *mServer;
@@ -231,6 +235,7 @@ char *get_prefered_locale(const char *locales);
 const char *find_localename_from_encoding(const char *encoding);
 int utf8_mbtowc(uchar *wc, const unsigned char *src, int src_len);
 int utf8_wctomb(unsigned char *dest, uchar wc); // dest must have size 6
+int mb_string_to_utf8(char *utf8, const char *str, int to_len, const char *enc);
 
 class XimServer {
 public:
@@ -248,6 +253,7 @@ public:
     void set_im(const char *name);
     void changeContext(const char *engine);
     void customContext(const char *custom, const char *val);
+    void reloadConfigs();
     std::list<InputContext *> ic_list;
 public:
     static XimServer *findServer(Window w);
